#ifndef NETPBMWRITER_H
#define NETPBMWRITER_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CNetPBMWriter K-3D object, which writes a large number of bitmap image file formats, using the NetPBM package
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dobject.h>
#include <k3difileformat.h>
#include <k3dibitmapwriteformat.h>

#include <sdpxmldocument.h>

#include <iostream>

// Forward class declarations ...
class k3dPixel;

/// File filter for writing bitmap images, using the NetPBM package
class CNetPBMWriter : public k3dObject,
										public k3dIFileFormat,
										public k3dIBitmapWriteFormat
{
public:
	virtual ~CNetPBMWriter();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIFileFormat overrides ...
	K3DFILEFORMATHANDLE GetFirstFileFormat();
	K3DFILEFORMATHANDLE GetNextFileFormat(K3DFILEFORMATHANDLE Handle);
	SDPCSTRING GetFileFormat(K3DFILEFORMATHANDLE Handle);
	sdpResult QueryCanHandle(SDPCSTRING FilePath, SDPCSTRING Format);

	// k3dIBitmapWriteFormat overrides ...
	k3dIFileFormat* FileFormat();
	sdpResult PreWrite(k3dIBitmap* Bitmap, SDPCSTRING FilePath, SDPCSTRING Format);
	sdpResult WriteOptions(k3dIBitmap* Bitmap, SDPCSTRING FilePath, SDPCSTRING Format);
	sdpResult WriteFile(k3dIBitmap* Bitmap, SDPCSTRING FilePath, SDPCSTRING Format);

protected:
	/// Loads the netpbm.conf XML file, which describes how to identify and convert filetypes supported by NetPBM
	void LoadFormats();

	/// Returns a filter description from the netpbm.conf XML file, corresponding to the requested file format
	sdpXmlIElement* GetWriter(SDPCSTRING FilePath, SDPCSTRING Format);

	/// Saves a K-3D bitmap in PPM format
	bool SaveAsPPM(k3dIBitmap* Bitmap, SDPCSTRING FilePath);
	/// Saves image data in PPM format
	bool SaveAsPPM(k3dPixel* Data, gulong Width, gulong Height, std::ostream& Stream);

	sdpXmlDocument m_Formats;
};

#endif // NETPBMWRITER_H
