#ifndef RIENGINE_H
#define RIENGINE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CRiEngine class, which can render the 3D workspace to a RIB file
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3direnderengine.h>
#include <k3driengine.h>
#include <fstream>

// Forward class declarations
class CCamera;

//////////////////////////////////////////////////////////////////////////
// CRiEngine

class CRiEngine : public k3dIRenderEngine,
								public k3dRiEngine

{
friend class CCamera;
public:
	CRiEngine();
	virtual ~CRiEngine();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_FAIL; }
	gulong AddRef() { return 1; }
	gulong Release() { return 0; }

	// k3dIRenderEngine overrides ...
	sdpResult SetCamera(k3dICamera* Camera);
	sdpResult Redraw();
	sdpResult StartJob(k3dIRenderJob* Job);
	sdpResult QueueFrame(k3dIRenderFrame* Frame);
	sdpResult FinishJob(k3dIRenderJob* Job);

	// k3dIRiEngine overrides ...
	k3dICamera* Camera();

protected:
	// Owner camera ...
	CCamera* m_Camera;

	// Helper functions ...
	bool SetupRIBFile(k3dIRenderFrame* Frame, std::ofstream& File);
	void SetupDisplay(k3dIRenderFrame* Frame);
	void SetupView(k3dIRenderFrame* Frame);
	void SetupRadiosity(k3dIRenderFrame* Frame);
	void SetupAtmosphere(k3dIRenderFrame* Frame);
	void RenderAll(k3dIRenderFrame* Frame);
//	bool SetupProcess(k3dIRenderFrame* Frame, SDPCSTRING ProcessName, bool StartImmediately, bool Visible, SDPCSTRING RIBFilePath);
};

#endif // RIENGINE_H
