#include "stdafx.h"

#include "group.h"
#include "objectlib.h"




CGroup::CGroup()
{
	m_Sheet = NULL;
}


CGroup::~CGroup()
{
	if(m_Sheet)
		m_Sheet->DestroyWindow();
}




///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CGroup::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dILocationServer>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIDocumentEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIPointServer>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIPolygonServer>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIAnimate>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dITimeServer>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIPersist>(this, InterfaceID, Interface)) return SDP_S_OK;

	return SDP_E_NO_INTERFACE;
}


gulong CGroup::AddRef()
{
	return sdpComAddRef(m_ReferenceCount);
}


gulong CGroup::Release()
{
	return sdpComRelease(this, m_ReferenceCount);
}




///////////////////////////////////////////////////////////////
// k3dIObject overrides

sdpResult CGroup::EditObject()
{
	if(!KOpenDialog(m_Sheet, this)) KFAIL(SDP_E_FAIL);
	return SDP_S_OK;
}


///////////////////////////////////////////////////////////////
// k3dIDocumentEventClient overrides

sdpResult CGroup::OnPointsDeparting()
{
	KPointArray newlist;

	for(gulong i = 0; i < m_Points.size(); i++)
		{
			for(k3dIPoint* point = m_Document->GetFirstPoint(); point; point = m_Document->GetNextPoint(point))
				{
					if(point == m_Points[i])
						{
							newlist.push_back(m_Points[i]);
							break;
						}
				}
		}

	m_Points = newlist;

	if(m_Sheet)
		m_Sheet->m_GroupPage->OnUpdateNumbers();

	DeleteIfEmpty();

	return SDP_S_OK;
}


sdpResult CGroup::OnPolygonsDeparting()
{
	PolygonList newlist;

	for(gulong i = 0; i < m_Polygons.size(); i++)
		{
			for(k3dIPolygon* poly = m_Document->GetFirstPolygon(); poly; poly = m_Document->GetNextPolygon(poly))
				{
					if(poly == m_Polygons[i])
						{
							newlist.push_back(m_Polygons[i]);
							break;
						}
				}
		}

	m_Polygons = newlist;

	if(m_Sheet)
		m_Sheet->m_GroupPage->OnUpdateNumbers();

	DeleteIfEmpty();

	return SDP_S_OK;
}





sdpResult CGroup::ObjectRenamed(SDPCSTRING NewName)
{
	if(m_Sheet)	m_Sheet->RenameObject(NewName);

	return SDP_S_OK;
}


sdpResult CGroup::ObjectSelected(bool State)
{
	if(State) Select();
	OnModify();

	return SDP_S_OK;
}


////////////////////////////////////////////////////////////////////
// k3dILocation overrides

gulong CGroup::GetNumLocations()
{
	return GetPointCount();
}


K3DLOCATIONHANDLE CGroup::GetFirstLocation()
{
	return K3DLOCATIONHANDLE(GetPointCount() ? 1 : 0);
}


K3DLOCATIONHANDLE CGroup::GetNextLocation(K3DLOCATIONHANDLE Location)
{
	g_assert(Location);

	KPointArray::size_type i = (KPointArray::size_type)Location + 1;
	return K3DLOCATIONHANDLE(i <= m_Points.size() ? i : 0);
}


k3dILocation* CGroup::GetLocation(K3DLOCATIONHANDLE Location)
{
	g_assert(Location);

	return m_Points[KPointArray::size_type(Location) - 1]->Location();
}


///////////////////////////////////////////////////////////////
// k3dIPointServer overrides


///////////////////////////////////////////////////////////////
// k3dIPolygonServer overrides


///////////////////////////////////////////////////////////////
// k3dIAnimate overrides


///////////////////////////////////////////////////////////////
// k3dIPersist overrides

sdpResult CGroup::Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies)
{
	if(sdpFailed(k3dPersistObject::Save(Stream, Dependencies)))
		KFAIL(SDP_E_FAIL);

	if(sdpFailed(k3dPointServer::Save(Stream, Dependencies)))
		KFAIL(SDP_E_FAIL);

	if(sdpFailed(k3dPolygonServer::Save(Stream, Dependencies)))
		KFAIL(SDP_E_FAIL);

	return SDP_S_OK;
}


sdpResult CGroup::Load(sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(Element);

	if(sdpFailed(k3dPersistObject::Load(Element)))
		KFAIL(SDP_E_FAIL);

	if(sdpFailed(k3dPointServer::Load(Element)))
		KFAIL(SDP_E_FAIL);

	if(sdpFailed(k3dPolygonServer::Load(Element)))
		KFAIL(SDP_E_FAIL);

	return SDP_S_OK;
}


sdpResult CGroup::LoadComplete()
{
	if(sdpFailed(k3dPointServer::LoadComplete(m_Document)))
		KFAIL(SDP_E_FAIL);

	if(sdpFailed(k3dPolygonServer::LoadComplete(m_Document)))
		KFAIL(SDP_E_FAIL);

	return SDP_S_OK;
}


//////////////////////////////////////////////////////////////////////////
// Helper functions

void CGroup::Add()
{
	// Add Points
	for(k3dIPoint* point = m_Document->GetFirstPoint(); point; point = m_Document->GetNextPoint(point))
		{
			if(SDP_S_OK == point->State()->QueryState(k3dIState::VISIBLESELECTED))
				{
					AddPoint(point);
				}
		}

	// Add Polygons
	for(k3dIPolygon* polygon = m_Document->GetFirstPolygon(); polygon; polygon = m_Document->GetNextPolygon(polygon))
		{
			if(SDP_S_OK == polygon->State()->QueryState(k3dIState::VISIBLESELECTED))
				{
					AddPolygon(polygon);
				}
		}
}


void CGroup::Remove()
{
	// Remove Points
	for(k3dIPoint* point = m_Document->GetFirstPoint(); point; point = m_Document->GetNextPoint(point))
		{
			if(SDP_S_OK == point->State()->QueryState(k3dIState::VISIBLESELECTED))
				{
					RemovePoint(point);
				}
		}

	// Remove Polygons
	for(k3dIPolygon* polygon = m_Document->GetFirstPolygon(); polygon; polygon = m_Document->GetNextPolygon(polygon))
		{
			if(SDP_S_OK == polygon->State()->QueryState(k3dIState::VISIBLESELECTED))
				{
					RemovePolygon(polygon);
				}
		}
}


void CGroup::RemoveAll()
{
	m_Points.clear();
	m_Polygons.clear();

	DeleteIfEmpty();
}


void CGroup::DeleteIfEmpty()
{
	if(GetPointCount()) return;
	if(GetPolygonCount()) return;

	// If the document is being closed, all bets are off ...
	if(SDP_S_OK == m_Document->DocumentClosing()) return;

	sdpString title;
	title.format("%s Group:", m_ObjectContext->Name());
	if(SDP_S_OK != m_IO->Query("The Group is empty.  Delete? (No undo)", title)) return;

	k3dDeleteObject(this);
}


void CGroup::Select()
{
	SetPointState(k3dIState::VISIBLESELECTED, TRUE);
	SetPolygonState(k3dIState::VISIBLESELECTED, TRUE);
}



/////////////////////////////////////////////////////////////////////////////
// CGroupSheet

CGroupSheet::CGroupSheet(CGroup* Object)
	: KSheet(Object, theApp.m_hInstance, "Group - Groups points and polygons together", VERTICAL, "Group")
{
	m_Object = Object;

	m_GroupPage = NULL;
}



CGroupSheet::~CGroupSheet()
{
	m_Object->m_Sheet = NULL;
}



bool CGroupSheet::Create()
{
	m_GroupPage = new CGroupPage(this);

	if(m_GroupPage)
		{
			AddPage(m_GroupPage);

			Create(IDD);

			return TRUE;
		}

	KFAIL(FALSE);
}



void CGroupSheet::OnHelp()
{
	m_Object->StandardHelp("objects", "group.html");
}


BEGIN_MESSAGE_MAP(CGroupSheet, KSheet)
	//{{AFX_MSG_MAP(CGroupSheet)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CGroupSheet message handlers


BOOL CGroupSheet::OnInitDialog()
{
	KSheet::OnInitDialog();

	RenameObject(m_Object->m_ObjectContext->Name());


	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}





/////////////////////////////////////////////////////////////////////////////
// CGroupPage property page

CGroupPage::CGroupPage(CGroupSheet* Parent)
  : KPage(IDD, Parent, "Type")
{
	//{{AFX_DATA_INIT(CGroupPage)
	//}}AFX_DATA_INIT

	m_Sheet = Parent;
}




BEGIN_MESSAGE_MAP(CGroupPage, KPage)
	//{{AFX_MSG_MAP(CGroupPage)
	ON_BN_CLICKED(IDC_ADD, OnAdd)
	ON_BN_CLICKED(IDC_REMOVE, OnRemove)
	ON_BN_CLICKED(IDC_REMOVEALL, OnRemoveall)
	ON_BN_CLICKED(IDC_SELECT, OnSelect)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CGroupPage message handlers

BOOL CGroupPage::OnInitDialog()
{
	KPage::OnInitDialog();

	OnUpdateNumbers();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


void CGroupPage::OnAdd()
{
	m_Sheet->m_Object->Add();
	OnUpdateNumbers();
}



void CGroupPage::OnRemove()
{
	m_Sheet->m_Object->Remove();
	OnUpdateNumbers();
}

void CGroupPage::OnRemoveall()
{
	if(sdpSucceeded(m_Sheet->m_Object->m_IO->Query("Remove all points and polygons from this Group? (No Undo)", "Remove All:")))
		{
			m_Sheet->m_Object->RemoveAll();
			OnUpdateNumbers();
		}
}

void CGroupPage::OnSelect()
{
	m_Sheet->m_Object->Select();
	OnModify();
}


void CGroupPage::OnUpdateNumbers()
{
	k3dSetLong(GetDlgItem(IDC_POINTCOUNT)->m_hWnd, m_Sheet->m_Object->GetPointCount());
	k3dSetLong(GetDlgItem(IDC_POLYGONCOUNT)->m_hWnd, m_Sheet->m_Object->GetPolygonCount());
}


