
#include "resource.h"

#include <k3dpointclient.h>
#include <k3dpolygonclient.h>

// Forward class references
class CGroupPage;
class CGroupSheet;


class CGroup : public k3dPersistObject,
									public k3dPointServer,
									public k3dPolygonServer,
									public k3dILocationServer
{
public:
	CGroup();
	~CGroup();

	CGroupSheet* m_Sheet;

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectRenamed(SDPCSTRING NewName);
	sdpResult ObjectSelected(bool State);

	// k3dIDocumentEventClient overrides
	sdpResult OnPointsDeparting();
	sdpResult OnPolygonsDeparting();

	// k3dILocationServer overrides
	gulong GetNumLocations();
	K3DLOCATIONHANDLE GetFirstLocation();
	K3DLOCATIONHANDLE GetNextLocation(K3DLOCATIONHANDLE Location);
	k3dILocation* GetLocation(K3DLOCATIONHANDLE Location);

	// k3dIPointServer overrides

	// k3dIPolygonServer overrides

	// k3dIAnimate overrides

	// k3dIPersist overrides
	sdpResult Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies);
	sdpResult Load(sdpXmlIElement* Element);
	sdpResult LoadComplete();

	// Helper functions
	void Add();
	void Remove();
	void RemoveAll();
	void Select();
	void DeleteIfEmpty();
};







/////////////////////////////////////////////////////////////////////////////
// CGroupSheet

class CGroupSheet : public KSheet
{
public:
	CGroupSheet(CGroup* Group);
	~CGroupSheet();

	bool Create();
	void OnHelp();


	CGroup* m_Object;

	CGroupPage* m_GroupPage;

	//{{AFX_VIRTUAL(CGroupSheet)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CGroupSheet)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};





/////////////////////////////////////////////////////////////////////////////
// CGroupPage dialog

class CGroupPage : public KPage
{
public:
	CGroupPage(CGroupSheet* Parent);

	void OnUpdateNumbers();

	CGroupSheet* m_Sheet;

	//{{AFX_DATA(CGroupPage)
	//}}AFX_DATA

	//{{AFX_VIRTUAL(CGroupPage)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CGroupPage)
	virtual BOOL OnInitDialog();
	afx_msg void OnAdd();
	afx_msg void OnRemove();
	afx_msg void OnRemoveall();
	afx_msg void OnSelect();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};






