#include "stdafx.h"
#include "geometryformatemf.h"
#include "geometryfilter.h"



///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CGeometryFormatEMF::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIGeometryReadFormat>(this, InterfaceID, Interface)) return SDP_S_OK;

	return SDP_E_NO_INTERFACE;
}


gulong CGeometryFormatEMF::AddRef()
{
	return sdpCOMAddRef(m_ReferenceCount);
}


gulong CGeometryFormatEMF::Release()
{
	return sdpCOMRelease(this, m_ReferenceCount);
}




///////////////////////////////////////////////////////////////
// k3dIGeometryFormat overrides

SDPCSTRING CGeometryFormatEMF::GetReadFilterString()
{
	return "Windows Enhanced Metafile (*.emf)|*.emf";
}


sdpResult CGeometryFormatEMF::QueryCanRead(SDPCSTRING FilePath)
{
	sdpString filepath = FilePath;

	filepath.tolower();
	if("emf" == filepath.right(3)) return SDP_S_OK;

	return SDP_S_FALSE;
}



sdpResult CGeometryFormatEMF::ReadFile(SDPCSTRING FilePath)
{
	HENHMETAFILE hfile = GetEnhMetaFile(FilePath);
	if(!hfile)
		{
			m_IO->Error("Unable to open .emf file", "Load .emf file:");
			KFAIL(SDP_E_FAIL);
		}

	m_Group = m_Document->GetActiveLayer()->CreateGroup();	// Create a group object
	if(!m_Group) KFAIL(SDP_E_FAIL);

	bool returnvalue = EnumEnhMetaFile(NULL,													// handle to device context
																		 hfile,													// handle to enhanced metafile
																		 (ENHMFENUMPROC)&ExternalMetaFileProc,	// pointer to callback function
																		 (LPVOID)this,									// pointer to callback-function data
																		 NULL);													// pointer to bounding rectangle

	DeleteEnhMetaFile(hfile);

	if(!returnvalue)
		{
			m_IO->Error("Error parsing .emf file", "Load .emf file:");
			KFAIL(SDP_E_FAIL);
		}

	return SDP_S_OK;
}


int CALLBACK CGeometryFormatEMF::ExternalMetaFileProc(HDC hDC, HANDLETABLE FAR *lpHTable, ENHMETARECORD FAR *lpEMFR, int nObj,	LPARAM lpData)
{
	CGeometryFormatEMF* format = (CGeometryFormatEMF*)lpData;
	g_assert(format);

	return format->MetaFileProc(hDC, lpHTable, lpEMFR, nObj);
}

int CGeometryFormatEMF::MetaFileProc(HDC hDC, HANDLETABLE FAR *lpHTable, ENHMETARECORD FAR *lpEMFR, int nObj)
{
	switch(lpEMFR->iType)
		{
			case EMR_HEADER:
				break;

			case EMR_POLYBEZIER:
				break;

			case EMR_POLYGON:
				break;

			case EMR_POLYLINE:
				break;

			case EMR_POLYBEZIERTO:
				break;

			case EMR_POLYLINETO:
				break;

			case EMR_POLYPOLYLINE:
				break;

			case EMR_POLYPOLYGON:
				break;

			case EMR_SETWINDOWEXTEX:
				break;

			case EMR_SETWINDOWORGEX:
				break;

			case EMR_SETVIEWPORTEXTEX:
				break;

			case EMR_SETVIEWPORTORGEX:
				break;

			case EMR_MOVETOEX:
				break;

			case EMR_SCALEVIEWPORTEXTEX:
				break;

			case EMR_SCALEWINDOWEXTEX:
				break;

			case EMR_SETWORLDTRANSFORM:
				break;

			case EMR_MODIFYWORLDTRANSFORM:
				break;

			case EMR_ANGLEARC:
				break;

			case EMR_ELLIPSE:
				break;

			case EMR_RECTANGLE:
				break;

			case EMR_ROUNDRECT:
				break;

			case EMR_ARC:
				break;

			case EMR_CHORD:
				break;

			case EMR_PIE:
				break;

			case EMR_LINETO:
				break;

			case EMR_ARCTO:
				break;

			case EMR_POLYDRAW:
				break;

			case EMR_SETARCDIRECTION:
				break;

			case EMR_BEGINPATH:
				break;

			case EMR_ENDPATH:
				break;

			case EMR_CLOSEFIGURE:
				break;

			case EMR_POLYBEZIER16:
				break;

			case EMR_POLYGON16:
				break;

			case EMR_POLYLINE16:
				break;

			case EMR_POLYBEZIERTO16:
				break;

			case EMR_POLYLINETO16:
				break;

			case EMR_POLYPOLYLINE16:
				break;

			case EMR_POLYPOLYGON16:
				{
					PEMRPOLYPOLYGON16 polypolygon16 = (PEMRPOLYPOLYGON16)lpEMFR;
					POINTS* ppoints = (POINTS*)&polypolygon16->aPolyCounts[polypolygon16->nPolys];

					gulong i = 0;

					for(gulong npolys = 0; npolys < polypolygon16->nPolys; npolys++)
						{
							k3dIPolygon* poly = m_Group->CreatePolygon();
							if(!poly) KFAIL(0);

							for(gulong npoints = 0; npoints < polypolygon16->aPolyCounts[npolys]; npoints++)
								{
									k3dIPoint* point = m_Group->CreatePoint();
									if(!point) KFAIL(0);
										{
											sdpVector3 coords((double)ppoints[i].x * 0.1, (double)ppoints[i].y * -0.1, 0.0);
											point->Location()->SetOldXYZ(coords);
											point->Location()->SetLocalXYZ(coords);

											poly->Points()->AddPoint(point);
											i++;
										}
								}
						}
				}
				break;

			case EMR_POLYDRAW16:
				break;

			default:
				break;
		}

	return 1;
}
