// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the functions necessary to export K-3D objects from the geometrylib module
		\author Timothy M. Shead (tshead@k-3d.com)
*/
		
#include <gmodule.h>

#include <strstream>
#include <sdpxml.h>

#include <k3dcom.h>
#include <k3dmisc.h>

#include "objreader.h"

#include "../../k3d_version.h"

extern "C" {

/// Stores information in XML format about the types of objects exported by this module
static std::strstream ClassInfo;

/// Called to create an XML stream object containing information on the types of objects exported by this module
static void BuildClassInfo()
{
	// Generate an XML document ...
	sdpXmlDocument document;
	sdpXmlIElement* object;
	sdpXmlIElement*  library = k3dStartLibraryInfo(&document, "geometrylib", VERSION_STRING);

	// Wavefront .obj geometry format ...
	object = k3dAddObject(library, sdpGeometryReadFormatOBJ, "OBJReader");
	k3dAddInterface(object, k3dIGeometryReadFormat::iid);
	
	ClassInfo << document << std::ends;
}

/// Called when this shared library is loaded
G_MODULE_EXPORT const gchar* g_module_check_init(GModule *module)
{
	// Sanity checks ...
	g_assert(module);

	// Build our XML sdpComClassInfo document ...
	BuildClassInfo();

	return 0;
}

/// Called when this shared library is about to be unloaded
G_MODULE_EXPORT void g_module_unload(GModule *module)
{
	// Sanity checks ...
	g_assert(module);

	// Unlock memory on our XML sdpComClassInfo document ...
	ClassInfo.rdbuf()->freeze(false);
}

/// Called to obtain an XML document describing the objects supported by this library
G_MODULE_EXPORT SDPCSTRING sdpComLibraryInfo()
{
	return ClassInfo.str();
}

/// Called to create an instance of an object class and return an interface of the requested type
G_MODULE_EXPORT sdpResult sdpComClassProcedure(const sdpCLSID& ClassID, const sdpIID& InterfaceID, void** Interface)
{
	if(ClassID == sdpGeometryReadFormatOBJ)
		return sdpCreateClassInstance<COBJReader>(InterfaceID, Interface) ? SDP_S_OK : SDP_E_FAIL;

	return SDP_E_UNKNOWN_OBJECT;
}

}	// extern "C"

