#ifndef APPLICATIONFRAME_H
#define APPLICATIONFRAME_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CApplicationFrame class, which implements the per-application UI
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpgtkobjectcontainer.h>

// Forward class references ...
class CApplication;
class CScriptEditor;

////////////////////////////////////////////////////////////////////////////
// CAboutBox

/// Creates a modal "about" box that displays program version and copyright
class CAboutBox : public sdpGtkObjectContainer
{
public:
	bool Create();

protected:
	/// Overrides sdpGtkObjectContainer to handle custom events
	void OnEvent(sdpGtkEvent* Event);
};

////////////////////////////////////////////////////////////////////////////
// CSplashBox

/// Creates a "splash" screen while the program is loading
class CSplashBox : public sdpGtkObjectContainer
{
public:
	~CSplashBox();

	bool Create();

protected:
	/// Overrides sdpGtkObjectContainer to handle custom events
	void OnEvent(sdpGtkEvent* Event);
};

////////////////////////////////////////////////////////////////////////////
// CApplicationFrame

/// Implements the application user interface
class CApplicationFrame : public sdpGtkObjectContainer
{
//friend class CApplication;
//friend class CScriptEditor;

public:
	CApplicationFrame();
	virtual ~CApplicationFrame();

	/// Loads the GTKML template and creates the interface
	bool Create(CApplication* Application);

	/// Returns a pointer to the owning application
	CApplication* Application() { return m_Application; }

	/// Called when the user wants to create a new document
	void OnFileNew();
	/// Called when the user wants to open an existing document
	void OnFileOpen();
	/// Called to exit the program gracefully
	void OnFileExit();
	/// Called when the user wants to edit / test scripts
	void OnToolsScriptEditor();
	/// Called when the user wants to modify the program options
	void OnToolsOptions();
	/// Called to display an index of help topics
	void OnHelpIndex();
	/// Called to display information about the help system
	void OnHelpHowTo();
	/// Called to display an index of help tutorials
	void OnHelpTutorials();
	/// Called to display the tip of the day
	void OnHelpTipOfTheDay();
	/// Called to open the K-3D website, http://www.k-3d.com
	void OnHelpOnline();
	/// Called to open the help search system
	void OnHelpSearch();
	/// Called to display an "about" box with program version and copyright information
	void OnHelpAbout();

	/// Tells the frame to get rid of a script editor window
	void DestroyScriptEditor(CScriptEditor* ScriptEditor);

protected:
	/// Overrides sdpGtkObjectContainer to handle custom events
	void OnEvent(sdpGtkEvent* Event);
	/// Called to see if its OK to delete the frame
	void OnDelete(sdpGtkEvent* Event);
	/// Called when the frame widgets are about to be destroyed
	void OnDestroy();
	/// Called when the user clicks on the tools tree
	void OnToolsClicked(sdpGtkEvent* Event);
	/// Called when the user clicks on the objects tree
	void OnObjectsClicked(sdpGtkEvent* Event);
	/// Called when the event loop is idle
	void OnIdle();

	/// Populates the tools and objects trees
	void UpdateTrees();

	/// Prompt the user to see if it's safe to go exit the program
	bool SafeToExit();

	/// Returns the tools tree widget
	sdpGtkCTree ToolsTree() { return CTree("tools"); }
	/// Returns the objects tree widget
	sdpGtkCTree ObjectsTree() { return CTree("objects"); }

	/// Stores a pointer to the main application
	CApplication* m_Application;

	/// Stores ScriptEditor pointers so they can be cleaned-up
	typedef std::set<CScriptEditor*> ScriptEditors;
	typedef ScriptEditors::iterator ScriptEditorIterator;
	ScriptEditors m_ScriptEditors;

	/// Stores the last active document revision to avoid unnecessary UI updates
	gulong m_LastActiveDocumentRevision;
};

#endif APPLICATIONFRAME_H
