#ifndef DIALOGS_H
#define DIALOGS_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CMessageBox, CErrorBox, and CQueryBox informational dialogs
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpgtkobjectcontainer.h>

////////////////////////////////////////////////////////////////////////////
// CMessageBox

/// Creates a modal message box that can be displayed by clients2
class CMessageBox : public sdpGtkObjectContainer
{
public:
	void DoModal(SDPCSTRING Caption, SDPCSTRING Message);

protected:
	/// Handles creation of the GTK geometry
	bool Create(SDPCSTRING Caption, SDPCSTRING Message);
	/// Overrides sdpGtkObjectContainer to handle custom events
	void OnEvent(sdpGtkEvent* Event);
};

////////////////////////////////////////////////////////////////////////////
// CErrorBox

/// Creates a modal error message box that can be displayed by clients
class CErrorBox : public sdpGtkObjectContainer
{
public:
	void DoModal(SDPCSTRING Caption, SDPCSTRING Message);

protected:
	/// Handles creation of the GTK geometry
	bool Create(SDPCSTRING Caption, SDPCSTRING Message);
	/// Overrides sdpGtkObjectContainer to handle custom events
	void OnEvent(sdpGtkEvent* Event);
};

////////////////////////////////////////////////////////////////////////////
// CQueryBox

/// Creates a modal message box that can prompt the user for yes/no/ok/cancel information
class CQueryBox : public sdpGtkObjectContainer
{
public:
	CQueryBox();
	
	gint DoModal(SDPCSTRING Caption, SDPCSTRING Message, SDPCSTRING Label1, SDPCSTRING Label2);

protected:
	/// Handles creation of the GTK geometry
	bool Create(SDPCSTRING Caption, SDPCSTRING Message, SDPCSTRING Label1, SDPCSTRING Label2);
	/// Overrides sdpGtkObjectContainer to handle custom events
	void OnEvent(sdpGtkEvent* Event);
	
	gint m_Result;
};

#endif // DIALOGS_H

