#ifndef RENDERJOB_H
#define RENDERJOB_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CRenderJob class, which encapsulates a job to be rendered by the render farm
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3direnderjob.h>
#include <renderframe.h>

#include <list>

//////////////////////////////////////////////////////////////////////////////////
// CRenderJob

class CRenderJob : public k3dIRenderJob
{
public:
	CRenderJob(k3dIRenderFarm* Farm, SDPCSTRING JobName);
	virtual ~CRenderJob();

	// sdpIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_NO_INTERFACE; }
	gulong AddRef() { return 0; }
	gulong Release() { return 0; }

	// k3dIRenderJob overrides ...
	k3dIRenderFarm* Farm();
	SDPCSTRING Name();
	SDPCSTRING Path();
	sdpResult GetUniqueFile(SDPCSTRING File, SDPSTRING Buffer, gulong* BufferLength);
	k3dIRenderFrame* CreateFrame(SDPCSTRING FrameName);

	/// Called to write control files for the job and all frames
	bool WriteControlFiles();
	/// Called to create the "ready" status file for this job (marking it ready-to-render)
	bool MarkReady();

protected:
	k3dIRenderFarm* m_Farm;
	sdpString m_Name;
	sdpString m_Path;
	guchar m_Priority;
	
	RenderFrameList m_Frames;
};

//////////////////////////////////////////////////////////////////////////////////
// JobArray

typedef std::list<CRenderJob*> RenderJobList;
typedef RenderJobList::iterator RenderJobIterator;

#endif // RENDERJOB_H
