#ifndef SCRIPTEDITOR_H
#define SCRIPTEDITOR_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CScriptEditor class, which provides a UI for editing / executing scripts
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpgtkobjectcontainer.h>

// Forward class references ...
class CApplicationFrame;
class k3dIScriptEngine;

////////////////////////////////////////////////////////////////////////////
// CScriptEditor

/// Implements the application user interface
class CScriptEditor : public sdpGtkObjectContainer
{
public:
	CScriptEditor(CApplicationFrame* Frame);
	virtual ~CScriptEditor();

	/// Loads the GTKML template and creates the interface
	bool Create();

protected:
	/// Overrides sdpGtkObjectContainer to handle custom events
	void OnEvent(sdpGtkEvent* Event);

	/// Called when the frame widgets are about to be destroyed
	void OnDestroy();
	/// Called to begin a new script
	void OnFileNew();
	/// Called to load a script
	void OnFileOpen();
	/// Called to save the script
	void OnFileSave();
	/// Called to save the script under a new filename
	void OnFileSaveAs();
	/// Called to close the editor window
	void OnFileClose();
	/// Called to execute the script contained in the text widget
	void OnExecute();

	/// Called write a script to a file
	void SaveScript();
			
	/// The owning frame for this editor
	CApplicationFrame* m_Frame;
	/// The scripting engine to handle execution of scripts
	k3dIScriptEngine* m_ScriptEngine;
	/// The current script filepath
	sdpString m_FilePath;
};

#endif SCRIPTEDITOR_H
