/** Name = Checkerboard (Solid) **/
/** CreatedBy = Shead Data Processing **/

/** Ka:Name = Ambient **/
/** Ka:HelpString = Percentage of ambient light reflection **/

/** Kd:Name = Diffuse **/
/** Kd:HelpString = Percentage of diffuse light reflection **/

/** Ks:Name = Specular **/
/** Ks:HelpString = Percentage of specular light reflection **/

/** roughness:Name = Roughness **/
/** roughness:HelpString = Controls the size of specular reflections **/

/** specularcolor:Name = Specular Color **/
/** specularcolor:HelpString = Controls the color of specular reflections **/

/** TileX:Name = Tile X dimension **/
/** TileX:HelpString = Dimension of each tile along the "X" axis **/

/** TileY:Name = Tile Y dimension **/
/** TileY:HelpString = Dimension of each tile along the "Y" axis **/

/** TileZ:Name = Tile Z dimension **/
/** TileZ:HelpString = Dimension of each tile along the "Z" axis **/

/** EvenColor:Name = Even Color **/
/** EvenColor:HelpString = The color of even tiles **/


surface k3d_checkerboard_solid(float Ka = 1.0; float Kd = 1.0; float Ks = 0.5; float roughness = 0.1; color specularcolor = 1.0; uniform float TileX = 1.0;
															 uniform float TileY = 1.0; uniform float TileZ = 1.0; color EvenColor = 0.0;
	)
{
	color Ct;
	point Nf;
	float x, y, z, sum;

	x = mod(floor(xcomp(P) / TileX), 2.0);
	y = mod(floor(ycomp(P) / TileY), 2.0);
	z = mod(floor(zcomp(P) / TileZ), 2.0);

	sum = mod(x + y + z, 2.0);
	if (sum >= 0.5)
		{
			Ct = EvenColor;
		}
	else
		{
			Ct = Cs;
		}

	// Diffuse reflection model ...
	Nf = normalize(faceforward(N, I));
	Oi = Os;
	Ci = Os * (Ct * (Ka * ambient() + Kd * diffuse(Nf)) + specularcolor * Ks * specular(Nf, -normalize(I), roughness));
}
