/** Name = Spaceship Hull **/
/** CreatedBy = Shead Data Processing **/

/** Ka:Name = Ambient **/
/** Ka:HelpString = Percentage of ambient light reflection **/

/** Kd:Name = Diffuse **/
/** Kd:HelpString = Percentage of diffuse light reflection **/

/** Ks:Name = Specular **/
/** Ks:HelpString = Percentage of specular light reflection **/

/** roughness:Name = Roughness **/
/** roughness:HelpString = Controls the size of specular reflections **/

/** specularcolor:Name = Specular Color **/
/** specularcolor:HelpString = Controls the color of specular reflections **/

/** width:Name = Panel Width **/
/** width:HelpString = Panel width as a percentage **/

/** height:Name = Panel Height **/
/** height:HelpString = Panel height as a percentage **/

/** lumavary:Name = Panel Luma Variation **/
/** lumavary:HelpString = Variation of panel luma as a percentage **/

/** rowvary:Name = Panel Offset Variation **/
/** rowvary:HelpString = Random offset of each panel row as a percentage **/

/** grime:Name = Grime Strength **/
/** grime:HelpString = Grime strength as a percentage **/


#define snoise(x) (2 * noise(x) - 1)


float
fractalsum(point Q)
{
	float f;
	float value = 0;

	for (f = 1; f < 8; f *= 2)
		{
			value += snoise(Q * f) / f;
		}

	return value;
}



surface k3d_spaceshiphull1(float Ka = 1; float Kd = 1; float Ks = 0.25; float roughness = 0.8; color specularcolor = 1; float width = .1; float height = .06;
													 float lumavary = .4; float rowvary = 3.0; float grime = 0.1)
{
	float ss, tt, splate, tplate, platespecular;
	color platecolor;
	point Nf;
	point V;

	// Figure out which row we're in  
	ss = s / width;
	tt = t / height;

	tplate = floor(tt);

	ss += rowvary * noise(tplate + 0.3);
	splate = floor(ss);

	// Calculate plate color
	platecolor = Cs - (lumavary * noise(splate + 0.4, tplate + 0.5));

	// Add grime
	platecolor -= grime * fractalsum(P);

	// Add specular contribution
	platespecular = Ks * noise(splate + 0.6, tplate + 0.7);

	Nf = faceforward(normalize(N), I);
	V = normalize(-I);
	Oi = Os;
	Ci = Os * platecolor * (Ka * ambient() + Kd * diffuse(Nf)) + specularcolor * platespecular * specular(Nf, V, roughness);
}
