#ifndef K3DIBITMAP_H
#define K3DIBITMAP_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIBitmap interface, which encapsulates a 32 bit, RGBA bitmap image
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>

// Forward class references
class k3dIBitmapReadFormat;
class k3dIBitmapWriteFormat;

/// Encapsulates a single pixel within a K-3D bitmap image
class k3dPixel
{
public:
	guchar Red;
	guchar Green;
	guchar Blue;
	guchar Alpha;
};

//////////////////////////////////////////////////////////////////////////////////
// k3dIBitmap

/// Encapsulates a 32 bit, RGBA image
class k3dIBitmap : public virtual sdpIUnknown
{
public:
	/// Stores the interface ID
	static const sdpIID iid;

	/// Allocate memory for an image with the given width and height
	virtual sdpResult Allocate(gulong Width, gulong Height) = 0;
	/// Free all memory associated with the image
	virtual sdpResult Free() = 0;

	/// Return a pointer to the image data (a 2D array of k3dPixels)
	virtual k3dPixel* GetData() = 0;
	/// Returns the current image width
	virtual gulong GetWidth() = 0;
	/// Returns the current image height
	virtual gulong GetHeight() = 0;

	/// Loads an image from the filesystem
	virtual sdpResult LoadFile(SDPCSTRING FilePath, SDPCSTRING Format, k3dIBitmapReadFormat* FormatFilter) = 0;
	/// Saves an image to the filesystem
	virtual sdpResult SaveFile(SDPCSTRING FilePath, SDPCSTRING Format, k3dIBitmapWriteFormat* FormatFilter) = 0;

	/// Mirrors the current image around the X axis
	virtual sdpResult FlipVertical() = 0;
	/// Mirrors the current image around the Y axis
	virtual sdpResult FlipHorizontal() = 0;
	/// Scales the image in-place
	virtual sdpResult Scale(gulong NewWidth, gulong NewHeight) = 0;
};

#endif // K3DIBITMAP_H
