#ifndef K3DINURBSCURVE_H
#define K3DINURBSCURVE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dINURBSCurve interface, which encapsulates a NURBS curve of arbitrary degree and configuration
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dcom.h>

#include <vector>
#include <list>
#include <set>

// Forward class declarations
class k3dIMesh;
class k3dIState;
class k3dIPolyPointServer;
class k3dILocationServer;
class k3dIKnotServer;

SDP_DECLARE_HANDLE(K3DNURBSCURVEHANDLE);

class k3dINURBSCurve : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	virtual k3dIMesh* Mesh() = 0;
	virtual k3dIPolyPointServer* Points() = 0;
	virtual k3dILocationServer* Locations() = 0;
	virtual k3dIKnotServer* Knots() = 0;
	virtual k3dIState* State() = 0;
	virtual gulong Order() = 0;
	virtual const k3dID ID() = 0;
};

/// Contains an array of NURBS curves
typedef std::vector<k3dINURBSCurve*> k3dNURBSCurveArray;
/// Contains a linked-list of NURBS curves
typedef std::list<k3dINURBSCurve*> k3dNURBSCurveList;
/// Contains a set of NURBS curves
typedef std::set<k3dINURBSCurve*> k3dNURBSCurveSet;

#endif // K3DINURBSCURVE_H
