#ifndef K3DIRENDERFARM_H
#define K3DIRENDERFARM_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIRenderFarm interface, which encapsulates a (possibly virtual) "render farm" capable of rendering "jobs"
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>

// Forward class declarations
class k3dIRenderJob;

/// sdpCOM interface that encapsulates a virtual "render farm" capable of rendering "jobs"
class k3dIRenderFarm : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	/// Returns the unique render farm name
	virtual SDPCSTRING Name() = 0;
	/// Returns the path to the shared filesystem for storing data to be rendered by the farm
	virtual SDPCSTRING Path() = 0;
	/// Creates a new job, in a "waiting" state
	virtual k3dIRenderJob* CreateJob(SDPCSTRING JobName) = 0;
	/// Tells the render farm to begin executing the given job
	virtual sdpResult StartJob(k3dIRenderJob* Job) = 0;
};

#endif // K3DIRENDERFARM_H
