#ifndef K3DIRENDERFRAME_H
#define K3DIRENDERFRAME_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIRenderFrame interface, which encapsulates a single animation frame to be rendered as part of a k3dIRenderJob
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>

// Forward class declarations ...
class k3dIRenderJob;

SDP_DECLARE_HANDLE(K3DRENDERFRAMEHANDLE);

class k3dIRenderFrame : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	/// Defines some standard render priorities
	typedef enum
	{
		IMMEDIATE_PRIORITY = 0,
		HIGHEST_PRIORITY = 1,
		HIGH_PRIORITY = 64,
		MODERATE_PRIORITY = 128,
		LOW_PRIORITY = 190,
		LOWEST_PRIORITY = 255
	};

	/// Returns the job that owns this frame
	virtual k3dIRenderJob* Job() = 0;
	/// Returns the frame name, guaranteed to be unique within the job
	virtual SDPCSTRING Name() = 0;
	/// Returns the frame path, where per-frame data should be stored
	virtual SDPCSTRING Path() = 0;
	/// Modifies the requested file name as needed to guarantee its uniqueness within the frame, and returns the result
	virtual sdpResult GetUniqueFile(SDPCSTRING File, SDPSTRING Buffer, gulong* BufferLength) = 0;
	/// Sets the render priority of this frame
	virtual sdpResult SetPriority(guchar Priority) = 0;
	/// Returns the render priority of this frame
	virtual guchar GetPriority() = 0;
	/// Sets whether the frame should be visible during rendering
	virtual sdpResult SetViewDuringRender(bool View) = 0;
	/// Returns whether the frame should be visible during rendering
	virtual bool GetViewDuringRender() = 0;
	/// Sets whether the completed image should be shown after rendering
	virtual sdpResult SetViewAfterRender(bool View) = 0;
	/// Returns whether the completed image should be shown after rendering
	virtual bool GetViewAfterRender() = 0;
	/// Sets the requested render engine type (will be matched against available choices in the user options file)
	virtual sdpResult SetEngineType(SDPCSTRING EngineType) = 0;
	/// Returns the requested render engine type
	virtual SDPCSTRING GetEngineType() = 0;
	/// Sets the requested render engine (will be matched against available choices in the user options file)
	virtual sdpResult SetEngine(SDPCSTRING Engine) = 0;
	/// Returns the requested render engine
	virtual SDPCSTRING GetEngine() = 0;
	
	/// Defines the name of the control file for this frame
	static SDPCSTRING const ControlFile;
	/// Defines the name of the render-engine log file
	static SDPCSTRING const EngineLogFile;
	/// Defines the name of the render-engine error log file
	static SDPCSTRING const EngineErrorFile;
	/// Defines the name of the render-server log file
	static SDPCSTRING const SystemLogFile;
	/// Defines the name of the render-server error log file
	static SDPCSTRING const SystemErrorFile;
	/// Defines the name of the status file when a frame is ready for rendering
	static SDPCSTRING const StatusReadyFile;
	/// Defines the name of the status file while a frame is being rendered
	static SDPCSTRING const StatusRunningFile;
	/// Defines the name of the status file when a frame was aborted due to errors
	static SDPCSTRING const StatusErrorFile;
	/// Defines the name of the status file when a frame was rendered successfully
	static SDPCSTRING const StatusCompleteFile;
	/// Defines the name of the file the rendered image will be stored in
	static SDPCSTRING const OutputImageFile;
};

#endif // K3DIRENDERFRAME_H
