#ifndef K3DIRENDERJOB_H
#define K3DIRENDERJOB_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIRenderJob interface, which encapsulates a "job" consisting of zero-or-more k3dIRenderFrame objects, to be rendered by a k3dIRenderFarm
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>

// Forward class declarations ...
class k3dIRenderFarm;
class k3dIRenderFrame;

SDP_DECLARE_HANDLE(K3DRENDERJOBHANDLE);

/// Encapsulates 1-to-N "frames" to be rendered by a virtual "render farm"
class k3dIRenderJob : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	/// Returns the render farm that owns this job
	virtual k3dIRenderFarm* Farm() = 0;
	/// Returns the name of a job, which is guaranteed unique within a render farm
	virtual SDPCSTRING Name() = 0;
	/// Returns the job path, which can be used to store any per-job files
	virtual SDPCSTRING Path() = 0;
	/// Modifies the requested file name as needed to guarantee its uniqueness within the job, and returns the result
	virtual sdpResult GetUniqueFile(SDPCSTRING File, SDPSTRING Buffer, gulong* BufferLength) = 0;
	/// Adds a new "frame" to the job, to be rendered when the job is run
	virtual k3dIRenderFrame* CreateFrame(SDPCSTRING FrameName) = 0;
	
	static SDPCSTRING const ControlFile;
	static SDPCSTRING const SystemLogFile;
	static SDPCSTRING const SystemErrorFile;
	static SDPCSTRING const StatusReadyFile;
	static SDPCSTRING const StatusRunningFile;
	static SDPCSTRING const StatusErrorFile;
	static SDPCSTRING const StatusCompleteFile;
};

#endif // K3DIRENDERJOB_H
