#ifndef K3DISHADERINFO_H
#define K3DISHADERINFO_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIShaderInfo interface, which encapsulates information about a Pixar Renderman Shading Language shader
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dishaderinfoargument.h>

/// An opaque handle for accessing individual shader infos within a collection
SDP_DECLARE_HANDLE(K3DSHADERINFOHANDLE);

/// Encapsulates a Pixar Renderman Shading Language shader
class k3dIShaderInfo : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	// Enumerates possible shader types
	typedef enum
	{
		SURFACE =					1 << 1,
		LIGHT =						1 << 2,
		VOLUME =					1 << 3,
		DISPLACEMENT =		1 << 4,
		TRANSFORMATION =	1 << 5,
		IMAGER =					1 << 6,
	} ShaderType;

	/// Returns the shader name
	virtual SDPCSTRING Name() = 0;
	/// Returns the shader type
	virtual ShaderType Type() = 0;
	/// Returns a handle to the first shader argument
	virtual K3DSHADERINFOARGUMENTHANDLE GetFirstArgument() = 0;
	/// Returns a handle to the next shader argument
	virtual K3DSHADERINFOARGUMENTHANDLE GetNextArgument(K3DSHADERINFOARGUMENTHANDLE Handle) = 0;
	/// Converts a shader argument handle into a k3dIShaderArgument interface
	virtual k3dIShaderInfoArgument* GetArgument(K3DSHADERINFOARGUMENTHANDLE Handle) = 0;
};

#endif // K3DISHADERINFO_H
