#ifndef K3DISHADERINFOARGUMENT_H
#define K3DISHADERINFOARGUMENT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIShaderInfoArgument interface, which encapsulates information about a single argument to a Pixar Renderman Shading Language shader
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>

SDP_DECLARE_HANDLE(K3DSHADERINFOARGUMENTHANDLE);

/// Encapsulates an argument to a Pixar Renderman Shader
class k3dIShaderInfoArgument : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	/// Enumerates the argument data type
	typedef enum
	{
		UNKNOWN =						0,
		POINT =							1 << 1,
		VECTOR =						1 << 2,
		ORIENTATION =				1 << 3,
		SCALE =							1 << 4,
		NORMAL =						1 << 5,
		COLOR =							1 << 6,
		SCALAR =						1 << 7,
		TIME =								1 << 8,
		ANGLE =							1 << 9,
		DISTANCE =					1 << 10,
		AREA =							1 << 11,
		VOLUME =						1 << 12,
		MASS =							1 << 13,
		FORCE =							1 << 14,
		PRESSURE =					1 << 15,
		BOOLEAN =						1 << 16,
		OPTIONLIST =				1 << 17,
		STRING =						1 << 18,
		TEXTURE =						1 << 19,
		COORDINATESYSTEM =	1 << 20,
	} ArgumentType;

	/// Returns the argument name
	virtual SDPCSTRING Name() = 0;
	/// Returns the argument data type
	virtual ArgumentType Type() = 0;
	/// Returns the argument's default value
	virtual SDPCSTRING Default() = 0;
};

#endif // K3DISHADERINFOARGUMENT_H
