#ifndef K3DISTATE_H
#define K3DISTATE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIState interface, which encapsulates a set of states that apply to a majority of objects
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>

class k3dIState : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	/// Enumerates possible state bitflags
	typedef enum
	{
		/// Controls whether an entity is visible/enabled
		VISIBLE = 0x00000001,
		/// Controls the selection state of an entity
		SELECTED = 0x00000002,
		/// Controls whether an entity can be selected
		SELECTABLE = 0x00000004,
		/// Controls whether an entity should be visible to the user through any UIs
		USERVIEWABLE = 0x00000008,
		/// Controls whether the entity should be saved as part of a document
		SAVEABLE = 0x00000010,
		/// Deprecated flag
		UNUSED2 = 0x00000020,
		/// User-defined flag, for use in tools
		USER1 = 0x80000000,

		/// Masks all possible values
		ALL = 0xFFFFFFFF,
		/// Combination of "visible" and "selected"
		VISIBLESELECTED = 0x00000003,
		/// Combination of "visible" and "selectable"
		VISIBLESELECTABLE = 0x00000005,
	} State;

	virtual sdpResult QueryState(gulong State) = 0;
	virtual sdpResult SetState(gulong Mask, bool State) = 0;
};

#endif // K3DISTATE_H
