#ifndef K3DCHANNELBUTTON_H
#define K3DCHANNELBUTTON_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dChannelButton class, which provides a UI for representing objects that support k3dIChannel
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dichannel.h>
#include <k3deventclient.h>
#include <sdpgtkobjectcontainer.h>
#include <sdpgtkopengldrawingarea.h>
#include <list>

///////////////////////////////////////////////////////////////////////////
// k3dChannelButton

/// Provides a UI for editing objects that support k3dIChannel
class k3dChannelButton : public sdpGtkObjectContainer,
												 public k3dEventClient
{
public:
	k3dChannelButton();
	~k3dChannelButton();

	// sdpIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_FAIL; }
	gulong AddRef() { return 1; }
	gulong Release() { return 0; }

	/// Creates the UI from a GTKML (XML) document
	bool Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element);
	/// Creates the button and attaches it to the given channel
	bool Create(k3dIChannel* Channel, bool ShowCurve, GtkContainer* Parent);

	/// Connects the UI to an actual channel
	bool Connect(k3dIChannel* Channel);
	/// Deletes the button and detaches it from the channel (if any)
	void ReleaseChannel();

	const sdpString CustomType() const;

protected:
	/// Common UI creation
	bool Create();
	/// Creates the OpenGL value widget if required
	bool CreateValueWidget();

	/// Called to handle SDPGTK events
	void OnEvent(sdpGtkEvent* Event);
	/// Called when the custom-drawn value needs to be updated
	void OnValueDraw();
	/// Called when the user begins editing the channel in place
	void OnValueFocusIn();
	/// Called when the user finishes editing the channel in place
	void OnValueFocusOut();
	/// Called to open the edit the channel
	void OnEdit();
	/// Called to edit the channel quickly
	void OnQuickEdit();
	/// Called when the the button needs to be updated
	void Update();
	/// Called when the GTK+ widgets are about to go away
	void OnDestroy();

	/// Called to handle K-3D events from the channel we're fronting for
	sdpResult OnEvent(k3dIObject* Object, SDPCSTRING Event, gulong ClientData);

	/// Stores the channel we represent
	k3dIChannel* m_Channel;
	/// Allows the developer to choose whether to display the channel start-value or its curve on the button face
	bool m_ShowCurve;

	/// Stores the OpenGL context for the custom-drawn value
	sdpGtkOpenGLDrawingArea m_ValueWidget;
	
	static sdpVector3 m_BackgroundColor;
	static sdpVector3 m_NormalColor;
};

#endif // K3DCHANNELBUTTON_H
