#ifndef K3DDEPENDENCIES_H
#define K3DDEPENDENCIES_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dDependencies class, which provides an implementation of k3dIDependencies
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3didependencies.h>
#include <sdpstring.h>
#include <map>
#include <set>

// Forward class declarations
class sdpXmlIElement;
class k3dIObject;

////////////////////////////////////////////////////////////////////
// k3dDependencies

class k3dDependencies : public k3dIDependencies
{
public:
	virtual ~k3dDependencies() {}

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_NO_INTERFACE; }
	gulong AddRef() { return 1; }
	gulong Release() { return 0; }

	// k3dIDependencies overrides
	sdpResult RegisterObject(k3dIObject* Object);
	sdpResult RegisterFile(SDPCSTRING FilePath);

	// Helper functions ...
	sdpResult SaveObjects(sdpXmlIElement* Element);
	sdpResult SaveFiles(sdpXmlIElement* Element);

protected:
	// The set of dependent objects ...
	typedef std::map<k3dIObject*, bool> ObjectMap;
	typedef ObjectMap::iterator ObjectIterator;
	ObjectMap m_Objects;

	// The set of dependent files ...
	typedef std::set<sdpString> FileMap;
	typedef FileMap::iterator FileIterator;
	FileMap m_Files;
};

#endif K3DDEPENDENCIES_H
