#ifndef K3DDIALOG_H
#define K3DDIALOG_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dDialog class, which provides a standard dialog implementation for K-3D objects
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpgtkobjectcontainer.h>
#include <k3dichannel.h>

// Forward class declarations ...
class k3dUserObject;
class k3dChannelButton;
class k3dShaderSelector;

///////////////////////////////////////////////////////////////////////////
// k3dDialog

class k3dDialog : public sdpGtkObjectContainer
{
public:
	k3dDialog();
	~k3dDialog();

	/// Standard UI creation
	bool Create(k3dUserObject* Object, bool AutoDestroy, bool ShowObjectName, SDPCSTRING TemplateFile, SDPCSTRING HelpFile);

	/// Called once when the dialog is first created, to initialize any custom controls
	virtual bool InitializeControls();
	
	/// Called to update controls when their underlying data changes
	virtual void UpdateControls();
	
	/// Called to clean up controls before the dialog is destroyed
	virtual void FinalizeControls();

	// Events that most dialogs need to take care of ...
	void ObjectRenamed(SDPCSTRING NewName);
	void ObjectDeleted();

	// Data access ...
	k3dUserObject* Object() { return m_Object; }
	bool AutoDestroy() { return m_AutoDestroy; }
	sdpString TemplateFile() { return m_TemplateFile; }
	sdpString HelpFile() { return m_HelpFile; }

	// Event-handling ...
	void OnEvent(sdpGtkEvent* Event);

	// Built-in events ...
	virtual void OnOK();
	virtual void OnCancel();
	virtual void OnHelp();
	virtual void OnSetFocus();
	virtual void OnDelete(sdpGtkEvent* Event);
	virtual void OnDestroy();

	// Control updates ...
	bool UpdateControl(sdpGtkEntry Control, double& Storage, k3dIChannel::DataType DataType, gulong Precision);
	bool UpdateControl(sdpGtkEntry Control, double& Storage, k3dIChannel::DataType DataType, double MinValue, double MaxValue, gulong Precision);
	bool UpdateControl(sdpGtkEntry Control, gulong& Storage, k3dIChannel::DataType DataType);
	bool UpdateControl(sdpGtkEntry Control, gulong& Storage, k3dIChannel::DataType DataType, gulong MinValue, gulong MaxValue);

	// Custom control access ...
	k3dChannelButton* ChannelButton(SDPCSTRING Name);
	k3dShaderSelector* ShaderSelector(SDPCSTRING Name);

	void ConnectChannelButton(k3dIChannel* Channel, SDPCSTRING Name);

protected:
	/// Allows us to instantiate UI elements using custom tags in the GTKML source document
	sdpGtkObject* CreateCustomObject(sdpXmlIElement* Element);

	k3dUserObject* m_Object;
	bool m_AutoDestroy;
	sdpString m_TemplateFile;
	sdpString m_HelpFile;
};

#endif // K3DDIALOG_H
