#ifndef K3DLOCATIONSERVER_H
#define K3DLOCATIONSERVER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dLocation and k3dLocationServer classes, which provide default implementations of k3dILocation and k3dILocationServer, respectively
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dilocation.h>
#include <k3dilocationserver.h>
#include <k3dstate.h>
#include <sdpmath.h>

///////////////////////////////////////////////////////////////////
// k3dLocation

class k3dLocation : public k3dILocation,
										public k3dState
{
friend class k3dLocationServer;
public:
	// sdpIUnknown overrides ...
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_NO_INTERFACE; }
	gulong AddRef() { return 0; }
	gulong Release() { return 0; }

	// k3dILocation overrides ...
	k3dIState* State();
	sdpResult GetOldXYZ(double Coords[3]);
	sdpResult GetLocalXYZ(double Coords[3]);
	sdpResult GetAnimXYZ(double Coords[3]);
	sdpResult GetWorldXYZ(double Coords[3]);
	sdpResult GetCameraXYZ(double Coords[3]);
	sdpResult GetScreenXYZ(double Coords[3]);
	sdpResult GetNormalXYZ(double Coords[3]);
	sdpResult GetTextureXYZ(double Coords[3]);
	sdpResult GetImplicitTextureXYZ(double Coords[3]);
	sdpResult SetOldXYZ(double Coords[3]);
	sdpResult SetLocalXYZ(double Coords[3]);
	sdpResult SetAnimXYZ(double Coords[3]);
	sdpResult SetWorldXYZ(double Coords[3]);
	sdpResult SetCameraXYZ(double Coords[3]);
	sdpResult SetScreenXYZ(double Coords[3]);
	sdpResult SetNormalXYZ(double Coords[3]);
	sdpResult SetTextureXYZ(double Coords[3]);
	sdpResult SetImplicitTextureXYZ(double Coords[3]);

	sdpVector3 m_OldXYZ;
	sdpVector3 m_LocalXYZ;
	sdpVector3 m_AnimXYZ;
	sdpVector3 m_WorldXYZ;
	sdpVector3 m_CameraXYZ;
	sdpVector3 m_ScreenXYZ;
	sdpVector3 m_NormalXYZ;
	sdpVector3 m_TextureXYZ;
	sdpVector3 m_ImplicitTextureXYZ;
};

inline bool operator < (const k3dLocation& A, const k3dLocation& B) { return false; };
inline bool operator == (const k3dLocation& A, const k3dLocation& B) { return false; };


////////////////////////////////////////////////////////////////////
// k3dLocationServer

class k3dLocationServer : public k3dILocationServer
{
public:
	k3dLocationServer();
	virtual ~k3dLocationServer();

	// k3dILocationServer overrides
	gulong GetLocationCount();
	K3DLOCATIONHANDLE GetFirstLocation();
	K3DLOCATIONHANDLE GetNextLocation(K3DLOCATIONHANDLE Location);
	k3dILocation* GetLocation(K3DLOCATIONHANDLE Location);

	// Helper functions
	sdpResult Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies);
	sdpResult Load(sdpXmlIElement* Element);

	// Location access ...
	void ReserveLocations(gulong Quantity);
	k3dLocation& Location(gulong Index) { return m_Locations[Index]; }

	typedef std::vector<k3dLocation> LocationContainer;
	typedef std::vector<k3dLocation>::iterator LocationIterator;
	LocationContainer m_Locations;
};

#endif // K3DLOCATIONSERVER_H
