#ifndef K3DPOLYPOINT_H
#define K3DPOLYPOINT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dPolyPoint class, which encapsulates the many-to-many relationship between k3dIPoint and k3dIPolygon/k3dINURBSCurve
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dipolypoint.h>
#include <k3dilocation.h>
#include <k3dactionclient.h>

#include <sdpmath.h>

// Forward class references ...
class k3dIPolygon;
class k3dINURBSCurve;
class k3dPolygon;
class k3dNURBSCurve;
class k3dPoint;
class k3dMesh;
class sdpXmlIElement;
class k3dIDependencies;

/////////////////////////////////////////////////////////////////////////////
// k3dPolyPoint

class k3dPolyPoint : public k3dIPolyPoint,
									public k3dILocation,
									public k3dActionClient
{
public:
	k3dPolyPoint();
	k3dPolyPoint(k3dPolygon* Polygon);
	k3dPolyPoint(k3dPoint* Point, k3dID ID);
	k3dPolyPoint(k3dPolygon* Polygon, k3dPoint* Point, k3dID ID);
	virtual ~k3dPolyPoint();

	k3dPolygon* Polygon() { return m_Polygon; }

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_NO_INTERFACE; }
	gulong AddRef() { return 0; }
	gulong Release() { return 0; }

	// k3dIPolyPoint overrides
	k3dIPoint* Point();
	k3dIState* State();
	k3dILocation* Location();
	const k3dID ID();

	// k3dILocation overrides
//	k3dIState* State();	Already declared above ... the compiler will sort it out!
	sdpResult GetOldXYZ(double Coords[3]);
	sdpResult GetLocalXYZ(double Coords[3]);
	sdpResult GetAnimXYZ(double Coords[3]);
	sdpResult GetWorldXYZ(double Coords[3]);
	sdpResult GetCameraXYZ(double Coords[3]);
	sdpResult GetScreenXYZ(double Coords[3]);
	sdpResult GetNormalXYZ(double Coords[3]);
	sdpResult GetTextureXYZ(double Coords[3]);
	sdpResult GetImplicitTextureXYZ(double Coords[3]);
	sdpResult SetOldXYZ(double Coords[3]);
	sdpResult SetLocalXYZ(double Coords[3]);
	sdpResult SetAnimXYZ(double Coords[3]);
	sdpResult SetWorldXYZ(double Coords[3]);
	sdpResult SetCameraXYZ(double Coords[3]);
	sdpResult SetScreenXYZ(double Coords[3]);
	sdpResult SetNormalXYZ(double Coords[3]);
	sdpResult SetTextureXYZ(double Coords[3]);
	sdpResult SetImplicitTextureXYZ(double Coords[3]);

	// k3dIActionClient overrides ...
	sdpResult HandleStep(gulong ValueCode, gulong Value, gulong ValueSize);

	// Helper functions ...
	sdpResult Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies);
	sdpResult Load(sdpXmlIElement* Element);
	sdpResult LoadComplete(k3dMesh* Mesh);

	k3dPolygon* m_Polygon;
	k3dPoint* m_Point;
	k3dID m_ID;
	sdpVector3 m_NormalXYZ;
	sdpVector3 m_TextureXYZ;
	sdpVector3 m_ImplicitTextureXYZ;
	
	/// Enumerates undo-able steps
	typedef enum
	{
		STEP_TEXTUREXYZ,
	} StepType;
};

#endif POLYPOINT_H
