#ifndef K3DRIENGINE_H
#define K3DRIENGINE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dRiEngine class, which provides an implementation of k3dIRiEngine
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3diriengine.h>
#include <sdpstring.h>
#include <iostream>
#include <map>

// Forward class declarations ...
class k3dICamera;

class k3dRiEngine : public k3dIRiEngine
{
public:
	k3dRiEngine();
	virtual ~k3dRiEngine();
	
	gulong RiAreaLightSourceV(SDPCSTRING Name, gulong ParameterCount,	SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiAtmosphereV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiAttributeBegin();
	sdpResult RiAttributeEnd();
	sdpResult RiAttributeV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiBasis(gdouble UBasis[16], gint UStep, gdouble VBasis[16], gint VStep);
	sdpResult RiBound(gdouble Bound[6]);
	sdpResult RiClipping(gdouble Hither, gdouble Yon);
	sdpResult RiColor(gdouble Color[3]);
	sdpResult RiColorSamples(gulong ParameterCount, gdouble nRGB[], gdouble RGBn[]);
	sdpResult RiComment(SDPCSTRING Comment);
	sdpResult RiConcatTransform(gdouble Transform[16]);
	sdpResult RiConeV(gdouble Height, gdouble Radius, gdouble ThetaMax, 	gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiCoordSysTransform(SDPCSTRING Space);
	sdpResult RiCoordinateSystem(SDPCSTRING Space);
	sdpResult RiCropWindow(gdouble XMin, gdouble XMax, gdouble YMin, gdouble YMax);
	sdpResult RiCurvesV(SDPCSTRING Degree, gulong ParameterCountcurves, gulong ParameterCountverts[], SDPCSTRING Wrap, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiCylinderV(gdouble Radius, gdouble ZMin, gdouble ZMax, gdouble ThetaMax,	gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiDeclare(SDPCSTRING Name, SDPCSTRING Type);
	sdpResult RiDeformationV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiDepthOfField(gdouble FStop, gdouble FocalLength, gdouble FocalDistance);
	sdpResult RiDetail(gdouble Bound[6]);
	sdpResult RiDetailRange(gdouble MinVis, gdouble LowTran, gdouble UpTran, gdouble MaxVis);
	sdpResult RiDiskV(gdouble Height, gdouble Radius, gdouble ThetaMax, 	gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiDisplacementV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiDisplayV(SDPCSTRING Name, SDPCSTRING Type, SDPCSTRING Mode, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiErrorAbort(gint Code, gint Severity, SDPCSTRING Message);
	sdpResult RiErrorIgnore(gint Code, gint Severity, SDPCSTRING Message);
	sdpResult RiErrorPrint(gint Code, gint Severity, SDPCSTRING Message);
	sdpResult RiExposure(gdouble Gain, gdouble Gamma);
	sdpResult RiExteriorV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiFormat(gint XResolution, gint YResolution, gdouble AspectRatio);
	sdpResult RiFrameAspectRatio(gdouble AspectRatio);
	sdpResult RiFrameBegin(gint FrameNumber);
	sdpResult RiFrameEnd();
	sdpResult RiGeneralPolygonV(gulong LoopCount, gulong VertexCounts[], gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiGeometricApproximation(SDPCSTRING Type, gdouble Value);
	sdpResult RiGeometricRepresentation(SDPCSTRING Type);
	sdpResult RiGeometryV(SDPCSTRING Type, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiHiderV(SDPCSTRING Type, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiHyperboloidV(gdouble Point1[3], gdouble Point2[3], gdouble ThetaMax, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiIdentity();
	sdpResult RiIlluminate(gulong LightHandle, bool OnOff);
	sdpResult RiImagerV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiInteriorV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	gulong RiLightSourceV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiMatte(bool OnOff);
	sdpResult RiMotionBeginV(gulong ParameterCount, gdouble Times[]);
	sdpResult RiMotionEnd();
	sdpResult RiNewline();
	sdpResult RiNuPatchV(gulong UCount, gint UOrder, gdouble UKnot[], gdouble UMin, gdouble UMax, gulong VCount, gint VOrder, gdouble VKnot[], gdouble VMin, gdouble VMax, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	gulong RiObjectBegin();
	sdpResult RiObjectEnd();
	sdpResult RiObjectInstance(gulong Handle);
	sdpResult RiOpacity(gdouble Color[3]);
	sdpResult RiOptionV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiOrientation(SDPCSTRING Orientation);
	sdpResult RiParaboloidV(gdouble RMax, gdouble ZMin, gdouble ZMax, gdouble ThetaMax, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiPatchMeshV(SDPCSTRING Type, gulong UCount, SDPCSTRING UWrap, gulong VCount, SDPCSTRING VWrap, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiPatchV(SDPCSTRING Type, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiPerspective(gdouble FieldOfView);
	sdpResult RiPixelFilter(SDPCSTRING Filtername, gdouble XWidth, gdouble YWidth);
	sdpResult RiPixelSamples(gdouble XSamples, gdouble YSamples);
	sdpResult RiPixelVariance(gdouble Variation);
	sdpResult RiPointsGeneralPolygonsV(gulong PolygonCount, gulong LoopCounts[], gulong VertexCounts[], gulong Vertices[], gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiPointsPolygonsV(gulong PolygonCount, gulong VertexCounts[], gulong Vertices[], gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiPolygonV(gulong VertexCount, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiProjectionV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Params[]);
	sdpResult RiQuantize(SDPCSTRING Type, gint One, gint QMin, gint QMax, gdouble Amplitude);
	sdpResult RiReadArchive(SDPCSTRING Archive);
	sdpResult RiRelativeDetail(gdouble RelativeDetail);
	sdpResult RiReverseOrientation();
	sdpResult RiRotate(gdouble angle, gdouble DX, gdouble DY, gdouble DZ);
	sdpResult RiScale(gdouble DX, gdouble DY, gdouble DZ);
	sdpResult RiScreenWindow(gdouble Left, gdouble Right, gdouble Bottom, gdouble Top);
	sdpResult RiShadingInterpolation(SDPCSTRING Type);
	sdpResult RiShadingRate(gdouble Size);
	sdpResult RiShutter(gdouble SMin, gdouble SMax);
	sdpResult RiSides(gint Sides);
	sdpResult RiSkew(gdouble Angle, gdouble DX1, gdouble DY1, gdouble DZ1, gdouble DX2, gdouble DY2, gdouble DZ2);
	sdpResult RiSolidBegin(SDPCSTRING Type);
	sdpResult RiSolidEnd();
	sdpResult RiSphereV(gdouble Radius, gdouble ZMin, gdouble ZMax, gdouble ThetaMax, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiStructure(SDPCSTRING Structure);
	sdpResult RiSurfaceV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiTextureCoordinates(gdouble S1, gdouble T1, gdouble S2, gdouble T2, gdouble S3, gdouble T3, gdouble S4, gdouble T4);
	sdpResult RiTorusV(gdouble MajorRadius, gdouble MinorRadius, gdouble PhiMin,	gdouble PhiMax, gdouble ThetaMax,	gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpResult RiTransform(gdouble Transform[16]);
	sdpResult RiTransformBegin();
	sdpResult RiTransformEnd();
	sdpResult RiTranslate(gdouble DX, gdouble DY, gdouble DZ);
	sdpResult RiTrimCurve(gulong LoopCount, gulong CurveCounts[], gulong Order[],	gdouble Knot[], gdouble AMin[], gdouble AMax[], gulong ParameterCount, gdouble U[], gdouble V[], gdouble W[]);
	sdpResult RiWorldBegin();
	sdpResult RiWorldEnd();
	
protected:
	/// Redirects RIB output to the given stream
	sdpResult SetOutputStream(const std::ostream* Stream);

	/// Writes header data to the stream
	sdpResult WriteRIBHeader();

	sdpString FormatParameters(gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpString FormatParameters(gulong ArraySize, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]);
	sdpString FormatToken(SDPCSTRING Token);
	sdpString FormatArray(SDPCSTRING Contents);
	sdpString FormatParameterArray(gulong ArraySize, SDPCSTRING Token, gpointer Parameters);
	sdpString FormatIntegerArray(gulong ArraySize, gpointer Parameters);
	sdpString FormatFloatArray(gulong ArraySize, gpointer Parameters);
	sdpString FormatColorArray(gulong ArraySize, gpointer Parameters);
	sdpString FormatPointArray(gulong ArraySize, gpointer Parameters);
	sdpString FormatStringArray(gulong ArraySize, gpointer Parameters);
	sdpString FormatInteger(gint Integer);
	sdpString FormatFloat(gdouble Float);
	sdpString FormatColor(gdouble Color[3]);
	sdpString FormatPoint(gdouble Point[3]);
	sdpString FormatString(SDPCSTRING String);
	sdpString FormatBound(gdouble Bound[6]);
	sdpString FormatTransform(gdouble Transform[16]);
	sdpString FormatBasis(gdouble Basis[16]);

	/// Stores the RIB output stream
	std::ostream* m_Stream;
	/// Stores the current object handle
	gulong m_ObjectHandle;
	/// Stores the current light handle
	gulong m_LightHandle;
	
	/// Keeps track of argument declarations
	typedef std::map<sdpString, sdpString> Declarations;
	typedef Declarations::iterator DeclarationIterator;
	
	/// Stores declarations that are defined by the Renderman Interface
	Declarations m_StandardDeclarations;
	/// Stores user declarations
	Declarations m_UserDeclarations;
};

#endif // K3DRIENGINE_H
