#ifndef K3DTOOL_H
#define K3DTOOL_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dTool class, which encapsulates a K-3D tool object (a K-3D object that isn't designed to be persistent or serializable)
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3duserobject.h>
#include <k3dilocation.h>
#include <k3dilocationserver.h>
#include <k3didocument.h>


class k3dTool : public k3dUserObject
{
public:
	k3dTool();

	virtual void OnSet();

	// Handle object modification in a standardized way ...
	virtual void OnModify(bool SaveAction);
	virtual void ModifyObject(k3dIObject* Object);
	virtual void ModifyLocationServer(k3dIObject* Object);
	virtual void ModifyLocation(k3dILocation* Location);

	virtual void GetObjectInfo(k3dILocationServer* LocationServer);

	// sdpIUnknown overrides

	// k3dIObject overrides
	sdpResult QueryRenameObject();
	sdpResult SetObjectContext(k3dIObjectContext* ObjectContext);


protected:
	sdpVector3 m_Min;
	sdpVector3 m_Max;
	sdpVector3 m_Size;
	sdpVector3 m_Center;
	sdpVector3 m_Origin;
	sdpVector3 m_OriginSize;

	k3dIDocument::Axis m_ActiveAxis;
};

#endif // K3DTOOL_H
