#ifndef K3DTRANSFORM_H
#define K3DTRANSFORM_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dTransform class, which provides a pluggable implementation of k3dITransform
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dobject.h>

#include <k3ditransform.h>
#include <k3diriengine.h>
#include <k3diobjectcontext.h>

#include <sdpxmlielement.h>
#include <sdpmath.h>

class k3dTransform : public k3dITransform
{
public:
	k3dTransform();

	// k3dITransform overrides ...
	sdpResult GetPosition(const ReferenceFrame Frame, gdouble Offsets[3]);
	sdpResult GetOrientation(const ReferenceFrame Frame, gdouble AngleAxis[4]);
	sdpResult GetScale(const ReferenceFrame Frame, gdouble Scale[3]);
	sdpResult SetPosition(const ReferenceFrame Frame, gdouble Offsets[3]);
	sdpResult SetOrientation(const ReferenceFrame Frame, gdouble AngleAxis[4]);
	sdpResult SetScale(const ReferenceFrame Frame, gdouble Scale[3]);

protected:
	// Data ...
	sdpVector3 m_Position[6];
	sdpAngleAxis m_Orientation[6];
	sdpVector3 m_Scale[6];

	// Persistence ...
	sdpResult Save(k3dObject* Object, sdpXmlIElement* Element);
	sdpResult Load(k3dObject* Object, sdpXmlIElement* Element);
};

#endif // K3DTRANSFORM_H
