#ifndef K3DUSEROBJECT_H
#define K3DUSEROBJECT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dUserObject class, which serves as a base class for K-3D objects that are intended to respond to user events
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dobject.h>
#include <k3dusereventclient.h>
#include <k3deventclient.h>

// Forward class declarations ...
class k3dVariableBase;
class k3dObjectVariableBase;

//////////////////////////////////////////////////////////////////////////////
// k3dUserObject

class k3dUserObject : public k3dObject,
										public k3dUserEventClient,
										public k3dEventClient
{
public:
	k3dUserObject();
	virtual ~k3dUserObject();

	virtual bool Create();

	typedef std::list<k3dVariableBase*> VariableList;
	typedef VariableList::iterator VariableIterator;
	VariableList m_Variables;

	typedef std::list<k3dObjectVariableBase*> ObjectList;
	typedef ObjectList::iterator ObjectIterator;
	ObjectList m_Objects;

	// k3dIObject overrides ...

	// k3dIUserEventClient overrides ...

	// k3dIEventClient overrides ...

	// Helper functions ...
	virtual void OnModify(bool SaveAction);

protected:
	// Helper functions ...
	void AddVariable(k3dVariableBase* Variable);
	void AddObject(k3dObjectVariableBase* Object);

	sdpResult DetachObjects();

	virtual void OnSet();
};

#endif // K3DUSEROBJECT_H
