// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements convenience functions for working with file filter objects
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dfilefilters.h>
#include <k3difileformat.h>


sdpStringMultiSet k3dFileFormats(k3dIDocument* Document, const sdpIID& InterfaceID)
{
	// Sanity checks ...
	g_assert(Document);

	sdpStringMultiSet results;

	// Get the set of objects that support the requested interface ...
	k3dObjectInfoSet objectinfos;
	k3dGetObjectInfo(Document->Application(), InterfaceID, objectinfos);

	// For each object type ...
	for(k3dObjectInfoSet::iterator objectinfo = objectinfos.begin(); objectinfo != objectinfos.end(); objectinfo++)
		{
			// Create an object of this type ...
			k3dIObject* object = Document->CreateObject((*objectinfo)->ClassID(), false, false, false);
			g_return_val_if_fail(object, sdpStringMultiSet());

			// Get the requested interface pointer ...
			k3dIFileFormat* fileformat = sdpQueryInterface<k3dIFileFormat>(object);
			g_return_val_if_fail(fileformat, sdpStringMultiSet());
			
			for(K3DFILEFORMATHANDLE handle = fileformat->GetFirstFileFormat(); handle; handle = fileformat->GetNextFileFormat(handle))
				results.insert(sdpString(fileformat->GetFileFormat(handle)));
			
			// Get rid of the object and continue ...
			k3dDeleteObject(fileformat);
		}
		
	return results;
}

