
// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dLocation and k3dLocationServer classes, which provide default implementations of k3dILocation and k3dILocationServer, respectively
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpxml.h>
#include <k3dlocationserver.h>


////////////////////////////////////////////////////////////////
// k3dLocation

k3dIState* k3dLocation::State()
{
	return this;
}

sdpResult k3dLocation::GetOldXYZ(double Coords[3])
{
	m_OldXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult k3dLocation::GetLocalXYZ(double Coords[3])
{
	m_LocalXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult k3dLocation::GetAnimXYZ(double Coords[3])
{
	m_AnimXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult k3dLocation::GetWorldXYZ(double Coords[3])
{
	m_WorldXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult k3dLocation::GetCameraXYZ(double Coords[3])
{
	m_CameraXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult k3dLocation::GetScreenXYZ(double Coords[3])
{
	m_ScreenXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult k3dLocation::GetNormalXYZ(double Coords[3])
{
	m_NormalXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult k3dLocation::GetTextureXYZ(double Coords[3])
{
	m_TextureXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult k3dLocation::GetImplicitTextureXYZ(double Coords[3])
{
	m_ImplicitTextureXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult k3dLocation::SetOldXYZ(double Coords[3])
{
	m_OldXYZ = Coords;
	return SDP_S_OK;
}

sdpResult k3dLocation::SetLocalXYZ(double Coords[3])
{
	m_LocalXYZ = Coords;
	return SDP_S_OK;
}

sdpResult k3dLocation::SetAnimXYZ(double Coords[3])
{
	m_AnimXYZ = Coords;
	return SDP_S_OK;
}

sdpResult k3dLocation::SetWorldXYZ(double Coords[3])
{
	m_WorldXYZ = Coords;
	return SDP_S_OK;
}

sdpResult k3dLocation::SetCameraXYZ(double Coords[3])
{
	m_CameraXYZ = Coords;
	return SDP_S_OK;
}

sdpResult k3dLocation::SetScreenXYZ(double Coords[3])
{
	m_ScreenXYZ = Coords;
	return SDP_S_OK;
}

sdpResult k3dLocation::SetNormalXYZ(double Coords[3])
{
	m_NormalXYZ = Coords;
	return SDP_S_OK;
}

sdpResult k3dLocation::SetTextureXYZ(double Coords[3])
{
	m_TextureXYZ = Coords;
	return SDP_S_OK;
}

sdpResult k3dLocation::SetImplicitTextureXYZ(double Coords[3])
{	
	m_ImplicitTextureXYZ = Coords;
	return SDP_S_OK;
}


////////////////////////////////////////////////////////////////
// k3dLocationServer

k3dLocationServer::k3dLocationServer()
{
}

k3dLocationServer::~k3dLocationServer()
{
}

void k3dLocationServer::ReserveLocations(gulong Quantity)
{
	m_Locations.resize(Quantity);
}

////////////////////////////////////////////////////////////////
// k3dILocationServer overrides


gulong k3dLocationServer::GetLocationCount()
{
	return m_Locations.size();
}


K3DLOCATIONHANDLE k3dLocationServer::GetFirstLocation()
{
	return K3DLOCATIONHANDLE(m_Locations.size() ? 1 : 0);

	return 0;
}


K3DLOCATIONHANDLE k3dLocationServer::GetNextLocation(K3DLOCATIONHANDLE Location)
{
	g_assert(Location);

	LocationContainer::size_type i = (LocationContainer::size_type)Location + 1;
	return K3DLOCATIONHANDLE(i <= m_Locations.size() ? i : 0);
}


k3dILocation* k3dLocationServer::GetLocation(K3DLOCATIONHANDLE Location)
{
	g_assert(Location);

	return &m_Locations[LocationContainer::size_type(Location) - 1];
}


/////////////////////////////////////////////////////////////////////////////
// Helper functions

sdpResult k3dLocationServer::Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies)
{
	// Sanity checks ...
	g_assert(Element);
	g_assert(Dependencies);

	// Don't bother if we don't have any locations to save ...
	if(!m_Locations.size())
		return SDP_S_FALSE;

	sdpXmlIElement* locations = sdpXmlCreateElement(Element, "locations");

	for(gulong i = 0; i < m_Locations.size(); i++)
		sdpXmlCreateElement(locations, "location", 0, "position", m_Locations[i].m_OldXYZ);

	return SDP_S_OK;
}

class k3dLocationServerLoadLocation
{
public:
	k3dLocationServerLoadLocation(k3dLocationServer* Client) { m_Index = 0; m_Client = Client; }

	bool HandleElement(sdpXmlIElement* Element)
	{
		sdpXmlGetAttribute(Element, "position", m_Client->m_Locations[m_Index].m_OldXYZ);
		m_Client->m_Locations[m_Index].m_LocalXYZ = m_Client->m_Locations[m_Index].m_OldXYZ;

		m_Index++;
		
		return true;
	}

	gulong m_Index;
	k3dLocationServer* m_Client;
};


sdpResult k3dLocationServer::Load(sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(Element);

	sdpXmlIElement* locations = sdpXmlFindElement(Element, "locations");
	if(!locations)
		return SDP_S_FALSE;

	gulong quantity = sdpXmlCountElements(locations, "location");
	ReserveLocations(quantity);

	k3dLocationServerLoadLocation functor(this);
	sdpXmlForEachElement(locations, "location", functor);

	return SDP_S_OK;
}

