// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dPrimToolDialog class, which is a k3dDialog specialized for use with a k3dPrimTool-derived class
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dprimtooldialog.h>
#include <k3dprimtool.h>
#include <k3dmisc.h>

k3dPrimToolDialog::k3dPrimToolDialog()
{
	m_Object = 0;
}

bool k3dPrimToolDialog::Create(k3dPrimTool* Object, bool AutoDestroy, SDPCSTRING TemplateFile, SDPCSTRING HelpFile)
{
	// Sanity checks ...
	g_assert(Object);
	g_assert_string(TemplateFile);

	// Store our object ...
	m_Object = Object;

	return k3dDialog::Create(Object, AutoDestroy, false, TemplateFile, HelpFile);
}

void k3dPrimToolDialog::OnOK()
{
	// Rebuild the object from scratch for undo purposes ...
	m_Object->m_CreateGeometry = true;
	m_Object->OnModify(true);
	m_Object->CreateAll();

	// Update the display ...
	k3dRedrawAll(m_Object->Document(), false);

	k3dDialog::OnOK();
}

void k3dPrimToolDialog::OnCancel()
{
	// Get rid of existing geometry ...
	m_Object->DeleteAll();

	// Update the display ...
	k3dRedrawAll(m_Object->Document(), false);

	k3dDialog::OnCancel();
}


