// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dShadingNode class, which caches information used to simulate smooth-shading of adjacent polygons
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dshadingnode.h>
#include <k3dpolypoint.h>
#include <k3dpolygon.h>
#include <k3dmesh.h>

#include <k3dipolygon.h>
#include <k3distate.h>

#include <sdpxmlutility.h>

////////////////////////////////////////////////////////////////////////
// k3dShadingNode

k3dShadingNode::k3dShadingNode()
{
}

k3dShadingNode::~k3dShadingNode()
{
}

void k3dShadingNode::RemovePolygons(gulong State)
{
	// For each polypoint ...
	PolyPointIterator nextpolypoint;
	PolyPointIterator end = m_PolyPoints.end();
	for(PolyPointIterator polypoint = m_PolyPoints.begin(); polypoint != end; polypoint = nextpolypoint)
		{
			nextpolypoint = polypoint;
			nextpolypoint++;

			// If the polypoint's polygon matches our criteria, remove it (the polypoint) from the list ...
			if(SDP_S_OK == (*polypoint)->Polygon()->State()->QueryState(State))
				m_PolyPoints.erase(polypoint);
		}
}

sdpResult k3dShadingNode::Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies)
{
	// Sanity checks ...
	g_assert(Element);
	g_assert(Dependencies);

	sdpXmlIElement* node = sdpXmlCreateElement(Element, "node");

	for(PolyPointIterator polypoint = m_PolyPoints.begin(); polypoint != m_PolyPoints.end(); polypoint++)
		sdpXmlCreateElement(node, "polypoint", 0, "id", (*polypoint)->ID());

	return SDP_S_OK;
}

/// Functor object for loading each shading-node polypoint
class k3dShadingNodeLoadShadingNodePolyPoint
{
public:
	k3dShadingNodeLoadShadingNodePolyPoint(k3dMesh* Mesh, k3dShadingNode* Node) { m_Mesh = Mesh; m_Node = Node; }

	bool HandleElement(sdpXmlIElement* Element)
	{
		// Get the ID for this polypoint ...
		k3dID id;
		g_return_val_if_fail(sdpXmlGetAttribute(Element, "id", id), true);

		// Convert it into a CPolyPoint pointer and push it onto our list ...
		k3dPolyPoint* polypoint = m_Mesh->FindPolyPoint(id);
		g_return_val_if_fail(polypoint, true);

		m_Node->m_PolyPoints.push_back(polypoint);
		
		return true;
	}

	k3dMesh* m_Mesh;
	k3dShadingNode* m_Node;
};

sdpResult k3dShadingNode::Load(sdpXmlIElement* Element, k3dMesh* Mesh)
{
	// Sanity checks ...
	g_assert(Element);

	k3dShadingNodeLoadShadingNodePolyPoint functor(Mesh, this);
	sdpXmlForEachElement(Element, "polypoint", functor);

	return SDP_S_OK;
}


