#ifndef IMAGELAYER_H
#define IMAGELAYER_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CImageLayer K-3D object, which renders 2D graphics either in front of or behind all 3D geometry.
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dpersistobject.h>
#include <k3deditorrender.h>
#include <k3drirender.h>
#include <k3dvariable.h>
#include <k3dchannelbutton.h>
#include <k3dshaderselector.h>
#include <k3dshaderargumentspage.h>

#include <k3ddialog.h>

// Forward class declarations ...
class CImageLayer;

/////////////////////////////////////////////////////////////////////////////
// CImageLayerDialog

class CImageLayerDialog : public k3dDialog
{
public:
	CImageLayerDialog();

	bool Create(CImageLayer* Object);

protected:
	CImageLayer* m_Object;
	k3dObjectSelector m_TextureSelector;

	bool InitializeControls();
	void UpdateControls();

	// Event handling ...
	void OnEvent(sdpGtkEvent* Event);
	void OnEditor();
	
	/// Provides the UI for selecting shaders
	k3dShaderSelector m_ShaderSelector;
	/// Provides the UI for controlling shader arguments
	k3dShaderArgumentsPage m_ShaderPage;
};

/////////////////////////////////////////////////////////////////////////////
// CImageLayer

class CImageLayer :	public k3dPersistObject,
										public k3dEditorRender,
										public k3dRiRender
{
friend class CImageLayerDialog;
public:
	CImageLayer();
	~CImageLayer();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult SetObjectContext(k3dIObjectContext* Context);
	sdpResult EditObject();
	sdpResult ObjectRenamed(SDPCSTRING NewName);
	sdpResult ObjectDeleted();

	// k3dIEditorRender overrides
	sdpResult EditorPreRender(k3dIEditorEngine* Engine);
	sdpResult EditorPostRender(k3dIEditorEngine* Engine);

	// k3dIRiRender overrides
	sdpResult RiPreRender(k3dIRenderFrame* Frame, k3dIRiEngine* Engine);
	sdpResult RiRender(k3dIRenderFrame* Frame, k3dIRiEngine* Engine);

	// k3dITimeServer overrides

	// k3dIPersist overrides
	sdpResult InitNew();

protected:
	// Helper functions
	sdpResult DrawEditor(k3dIEditorEngine* Engine);

/*
	typedef enum
	{
		NORMAL = 1,
		ADDITIVE = 2,
		MULTIPLY = 3,
	} Mode;

	class k3dVariableMode : public k3dCustomVariable<Mode>
	{
	public:
		sdpString ConvertToString() { return sdpFormatString("%d", m_Value); }
		bool ConvertFromString(SDPCSTRING String) { m_Value = Mode(atoi(String)); return true; }
	};
*/

	typedef enum
	{
		FOREGROUND = 1,
		BACKGROUND = 2,
	} Layer;

	class k3dVariableLayer : public k3dCustomVariable<Layer>
	{
	public:
		sdpString ConvertToString() { return sdpFormatString("%d", m_Value); }
		bool ConvertFromString(SDPCSTRING String) { m_Value = Layer(atoi(String)); return true; }
	};

	k3dVariable<bool> m_Editor;
//	k3dVariableMode m_Mode;
	k3dVariableLayer m_Layer;

	k3dColorChannel m_Color;
	k3dScalarChannel m_Strength;
	k3dShaderInstance m_Shader;

	CImageLayerDialog m_UserInterface;

	// This is so not thread safe!!
	static double m_ForegroundCount;
	static double m_BackgroundCount;
};

#endif // IMAGELAYER_H
