#ifndef FREEHAND_H
#define FREEHAND_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares CFreehand, a K-3D tool for creating freehand geometry
		\author Timothy M. Shead (tshead@k-3d.com)
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3dprimtool.h>
#include <k3dprimtooldialog.h>
#include <k3ddialog.h>
#include <k3deditorrender.h>

#include <sdpgl.h>

// Forward class references
class CFreehand;

/////////////////////////////////////////////////////////////////////////////
// CFreehandDialog

/// Provides the user interface for the Freehand tool
class CFreehandDialog : public k3dPrimToolDialog
{
friend class CFreehand;
public:
	CFreehandDialog();

	/// Called to create the UI
	bool Create(CFreehand* Object);

protected:
	/// Stores the pointer to the owning Freehand object
	CFreehand* m_Object;

	// Event handling ...
	void OnEvent(sdpGtkEvent* Event);
};

/////////////////////////////////////////////////////////////////////////////
// CFreehand

class CFreehand : public k3dPrimTool, public k3dEditorRender
{
friend class CFreehandDialog;
public:
	CFreehand();

	CFreehandDialog m_UserInterface;

	// k3dTool overrides ...
	void OnModify(bool SaveAction);

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	//k3dIObject overrides
	sdpResult SetObjectContext(k3dIObjectContext* ObjectContext);
	sdpResult EditObject();
	sdpResult ObjectDeleted();

	// k3dIUserEventClient overrides
	sdpResult OnMouseMove(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

	// k3dIEditorRender overrides
	sdpResult EditorPostRender(k3dIEditorEngine* Engine);

protected:
	/// Stores the position of the mouse in three dimensions
	sdpVector3 m_MouseCursor;

	/// Stores the 3D polygon points
	typedef std::vector<sdpVector3> PolygonPoints;
	PolygonPoints m_PolygonPoints;

	/// Converts mouse coordinates into 3D points
	sdpVector3 ScreenToWorld(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
};


#endif // FREEHAND_H
