#ifndef MIRROR_H
#define MIRROR_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CMirror K-3D tool, which reflects geometry about an axis
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dtool.h>
#include <k3ddialog.h>

// Forward class declarations
class CMirror;

/////////////////////////////////////////////////////////////////////////////
// CMirrorDialog

class CMirrorDialog : public k3dDialog
{
public:
	CMirrorDialog();

	bool Create(CMirror* Object);

	// Update controls ...
	void UpdateControls();

protected:
	CMirror* m_Object;

	// Event handling ...	
	void OnEvent(sdpGtkEvent* Event);

	void OnX();
	void OnY();
	void OnZ();
	void OnApply();
};

/////////////////////////////////////////////////////////////////////////////
// CMirror

class CMirror : public k3dTool
{
friend class CMirrorDialog;
public:
	CMirror();

	CMirrorDialog m_UserInterface;

	// k3dTool overrides ...
	void OnModify(bool SaveAction);

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectDeleted();
	
	typedef enum
	{
		X,
		Y,
		Z,
	} AxisType;

	AxisType m_Axis;
};

#endif // MIRROR_H
