#ifndef SNAP_H
#define SNAP_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CSnap K-3D tool, which "snaps" a point to another point's location
		\author Tim Shead (tshead@k-3d.com)
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3dtool.h>
#include <k3deditorrender.h>
#include <k3ddialog.h>

// Forward class references
class CSnap;

/////////////////////////////////////////////////////////////////////////////
// CSnapDialog

/// Provides the user interface for the Snap tool
class CSnapDialog : public k3dDialog
{
friend class CSnap;
public:
	CSnapDialog();

	/// Called to create the UI
	bool Create(CSnap* Object);

protected:
	/// Stores a pointer to the owning Snap object
	CSnap* m_Object;
};


/////////////////////////////////////////////////////////////////////////////
// CSnap

/// Implements the Snap tool
class CSnap : public k3dTool, public k3dEditorRender
{
public:
	CSnap();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult SetObjectContext(k3dIObjectContext* ObjectContext);
	sdpResult EditObject();
	sdpResult ObjectDeleted();

	// k3dIUserEventClient overrides
	sdpResult OnLButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);

	// k3dIEditorRender overrides
	sdpResult EditorPostRender(k3dIEditorEngine* Engine);
		
	// k3dTool overrides ...
	void OnModify(bool SaveAction);

protected:
	CSnapDialog m_UserInterface;

	k3dIPoint* m_Point1;
	k3dIPoint* m_Point2;

	bool m_MovePolys;
};


#endif // SNAP_H
