#ifndef WAVE_H
#define WAVE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Defines the CWave K-3D tool, which applies a "wave" transformation to geometry
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3dtool.h>
#include <k3ddialog.h>

// Forward class references
class CWave;

/////////////////////////////////////////////////////////////////////////////
// CWaveDialog

class CWaveDialog : public k3dDialog
{
public:
	CWaveDialog();

	bool Create(CWave* Object);

	// Update controls ...
	void UpdateControls();

protected:
	CWave* m_Object;

	// Event handling ...	
	void OnEvent(sdpGtkEvent* Event);

	void OnDisplaceX();
	void OnDisplaceY();
	void OnDisplaceZ();
	void OnAmplitude();
	void OnWavelength();
	void OnPhase();
};

/////////////////////////////////////////////////////////////////////////////
// CWave

class CWave : public k3dTool
{
friend class CWaveDialog;
public:
	CWave();

	CWaveDialog m_UserInterface;

	// k3dTool overrides ...
	void OnModify(bool SaveAction);
	void ModifyObject(k3dIObject* Object);
	void ModifyLocationServer(k3dIObject* Object);
	void ModifyLocation(k3dILocation* Location);

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectDeleted();

	// k3dIUserEventClient overrides
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

protected:
//	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_RangeChannel;
//	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Wave;

	bool	m_DisplaceX;
	bool	m_DisplaceY;
	bool	m_DisplaceZ;
	double	m_Amplitude;
	double	m_WaveLength;
	double	m_Phase;
	bool	m_CustomWave;
	bool	m_FiniteRange;
	double	m_Range;
};

#endif // WAVE_H
