#include "stdafx.h"

#include "tlbbevel.h"
#include "toollib.h"



/////////////////////////////////////////////////////////////////////////////
// CBevelPage property page

CBevelPage::CBevelPage(CBevelSheet* Sheet)
	: KPage(IDD, Sheet, "Bevel")
{
	//{{AFX_DATA_INIT(CBevelPage)
	//}}AFX_DATA_INIT

	m_Sheet = Sheet;
}

BEGIN_MESSAGE_MAP(CBevelPage, KPage)
	//{{AFX_MSG_MAP(CBevelPage)
	ON_EN_k3dILLFOCUS(IDC_WIDTH, OnKillfocusWidth)
	ON_EN_k3dILLFOCUS(IDC_HEIGHT, OnKillfocusHeight)
	ON_EN_k3dILLFOCUS(IDC_SEGMENTS, OnKillfocusSegments)
	ON_BN_CLICKED(IDC_FACES, OnFaces)
	ON_BN_CLICKED(IDC_BEVELS, OnBevels)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_PERCENTAGE, OnPercentage)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CBevelPage message handlers

BOOL CBevelPage::OnInitDialog()
{
	KPage::OnInitDialog();

	k3dSetDouble(k3dIChannel::DATA_DISTANCE, GetDlgItem(IDC_WIDTH)->m_hWnd, m_Sheet->m_Object->m_Width);
	k3dSetDouble(k3dIChannel::DATA_DISTANCE, GetDlgItem(IDC_HEIGHT)->m_hWnd, m_Sheet->m_Object->m_Height);
	k3dSetLong(GetDlgItem(IDC_SEGMENTS)->m_hWnd, m_Sheet->m_Object->m_Segments);

	CheckDlgButton(IDC_FACES, m_Sheet->m_Object->m_CreateFaces);
	CheckDlgButton(IDC_BEVELS, m_Sheet->m_Object->m_CreateBevels);
	CheckDlgButton(IDC_PERCENTAGE, m_Sheet->m_Object->m_PercentageWidth);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}



void CBevelPage::OnKillfocusWidth()
{
	KUpdateDouble(k3dIChannel::DATA_DISTANCE, GetDlgItem(IDC_WIDTH)->m_hWnd, m_Sheet->m_Object->m_Width);
	OnModify();
}


void CBevelPage::OnKillfocusHeight()
{
	KUpdateDouble(k3dIChannel::DATA_DISTANCE, GetDlgItem(IDC_HEIGHT)->m_hWnd, m_Sheet->m_Object->m_Height);
	OnModify();
}


void CBevelPage::OnKillfocusSegments()
{
	KValidateLong(GetDlgItem(IDC_SEGMENTS)->m_hWnd, m_Sheet->m_Object->m_Segments, 1, LONG_MAX);
	m_Sheet->OnNewGeometry();
}


void CBevelPage::OnFaces()
{
	m_Sheet->m_Object->m_CreateFaces = IsDlgButtonChecked(IDC_FACES) ? TRUE : FALSE;
	m_Sheet->OnNewGeometry();
}

void CBevelPage::OnBevels()
{
	m_Sheet->m_Object->m_CreateBevels = IsDlgButtonChecked(IDC_BEVELS) ? TRUE : FALSE;
	m_Sheet->OnNewGeometry();
}

void CBevelPage::OnPercentage()
{
	m_Sheet->m_Object->m_PercentageWidth = IsDlgButtonChecked(IDC_PERCENTAGE) ? TRUE : FALSE;
	OnModify();
}


////////////////////////////////////////////////////////////////////////////
// CBevelSheet

CBevelSheet::CBevelSheet(CBevel* Object)
: KPrimSheet(Object, theApp.m_hInstance, "Bevel - LMB drag bevel selected polygons", VERTICAL, "Bevel", TRUE)
{
	m_BevelPage = NULL;
	m_Object = Object;
}


CBevelSheet::~CBevelSheet()
{
	m_Object->m_Sheet = NULL;
}


bool CBevelSheet::Create()
{
	m_BevelPage = new CBevelPage(this);

	if(m_BevelPage)
		{
			AddPage(m_BevelPage);

			Create(IDD);
			return TRUE;
		}

	KFAIL(FALSE);
}


BEGIN_MESSAGE_MAP(CBevelSheet, KPrimSheet)
	//{{AFX_MSG_MAP(CBevelSheet)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()





/////////////////////////////////////////////////////////////////////////////
// CBevel

CBevel::CBevel() : k3dPrimTool()
{
	m_Width = 1.0;
	m_Height = 0.0;
	m_Segments = 1;

	m_CreateBevels = TRUE;
	m_CreateFaces = TRUE;
	m_PercentageWidth = FALSE;

	m_Sensitivity = .01;
}

CBevel::~CBevel()
{
	if(m_Sheet)
		m_Sheet->DestroyWindow();
}


///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CBevel::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;

	if(sdpSupportInterface<k3dIDocumentEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIUserEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	return SDP_E_NO_INTERFACE;
}


gulong CBevel::AddRef()
{
	return sdpComAddRef(m_ReferenceCount);
}


gulong CBevel::Release()
{
	return sdpComRelease(this, m_ReferenceCount);
}





///////////////////////////////////////////////////////////////
// k3dIObject overrides

sdpResult CBevel::EditObject()
{
	if(!KOpenDialog(m_Sheet, this)) KFAIL(SDP_E_FAIL);

	OnNewGeometry();
	return SDP_S_OK;
}






///////////////////////////////////////////////////////////////
// k3dIDocumentEventClient overrides

sdpResult CBevel::OnUndo(k3dIObject* Sender)
{
	m_Height = 0.0;
	m_Width = 1.0;
	m_Segments = 1;

	k3dSetDouble(k3dIChannel::DATA_DISTANCE, m_Sheet->m_BevelPage->GetDlgItem(IDC_WIDTH)->m_hWnd, m_Width);
	k3dSetDouble(k3dIChannel::DATA_DISTANCE, m_Sheet->m_BevelPage->GetDlgItem(IDC_HEIGHT)->m_hWnd, m_Height);
	k3dSetLong(m_Sheet->m_BevelPage->GetDlgItem(IDC_SEGMENTS)->m_hWnd, m_Segments);

	OnModify();

	return SDP_S_OK;
}



sdpResult CBevel::OnSelectChanged()
{
	m_CreateGeometry = TRUE;
	OnModify();

	return SDP_S_OK;
}







///////////////////////////////////////////////////////////////
// k3dIUserEventClient overrides

sdpResult CBevel::OnLButtonStartDrag(long Flags, long X, long Y)
{
	KUserEvent::OnLButtonStartDrag(Flags, X, Y);
	return SDP_S_OK;
}

sdpResult CBevel::OnLButtonDrag(long Flags, long X, long Y)
{
	double xfactor, yfactor;

	KUserEvent::OnLButtonDrag(Flags, X, Y);

	xfactor = ((double)(CurrentMouse()[0] - LastMouse()[0]) * m_Sensitivity);
	yfactor = ((double)(CurrentMouse()[1] - LastMouse()[1]) * m_Sensitivity);

	if(ShiftDown())
		{
			m_Width += (fabs(xfactor)>=fabs(yfactor) ? xfactor : yfactor);
		}
	else if(ControlDown())
		{
			m_Height -= (fabs(xfactor)>=fabs(yfactor) ? xfactor : yfactor);
		}
	else
		{
			m_Width += xfactor;
			m_Height -= yfactor;
		}

	k3dSetDouble(k3dIChannel::DATA_DISTANCE, m_Sheet->m_BevelPage->GetDlgItem(IDC_WIDTH)->m_hWnd, m_Width);
	k3dSetDouble(k3dIChannel::DATA_DISTANCE, m_Sheet->m_BevelPage->GetDlgItem(IDC_HEIGHT)->m_hWnd, m_Height);

	OnModify();

	return SDP_S_OK;
}


sdpResult CBevel::OnLButtonEndDrag(long Flags, long X, long Y)
{
	KUserEvent::OnLButtonEndDrag(Flags, X, Y);

	k3dSetDouble(k3dIChannel::DATA_DISTANCE, m_Sheet->m_BevelPage->GetDlgItem(IDC_WIDTH)->m_hWnd, m_Width);
	k3dSetDouble(k3dIChannel::DATA_DISTANCE, m_Sheet->m_BevelPage->GetDlgItem(IDC_HEIGHT)->m_hWnd, m_Height);

	return SDP_S_OK;
}


void CBevelSheet::OnHelp()
{
	m_Object->StandardHelp("tools", "bevel.hmtl");
}


/////////////////////////////////////////////////////////////////////////////
// CBevel message handlers


void CBevel::GetGroupInfo(k3dIGroup* Group, gulong& SelectedPoints, gulong& SelectedPolys, gulong& NewPoints, gulong& NewPolys)
{
	k3dPrimTool::GetGroupInfo(Group, SelectedPoints, SelectedPolys, NewPoints, NewPolys);

	// Calculate new points
	NewPoints = SelectedPoints;

	// Calculate new polys
	NewPolys = 0;
	if(m_CreateFaces) NewPolys += SelectedPolys;
	if(m_CreateBevels) NewPolys += SelectedPolys * SelectedPoints;
}


void CBevel::CreateGroupGeometry(k3dIGroup* Group, gulong SelectedPoints, gulong SelectedPolys, gulong NewPoints, gulong NewPolys)
{
	k3dPrimTool::CreateGroupGeometry(Group, SelectedPoints, SelectedPolys, NewPoints, NewPolys);

	if(!m_CreateGeometry) return;

	// Create a subgroup ...
	k3dIObject* subgroup = CreateSubGroup(Group, "Bevel", NewPoints, NewPolys);

/*
	// Attach points to polys ...
	gulong polyindex = 0;
	gulong pointindex = 0;

	for(K3DPOINTHANDLE pointhandle = Group->Points()->GetFirstPoint(); pointhandle; pointhandle = Group->Points()->GetNextPoint(pointhandle))
		{
			if(m_CreateFaces)
				{
					m_PolygonList2->SetHandle(m_PolygonArray[polyindex++]);

					for(hr = m_GroupPolygon->SetFirstPoint(); SDP_S_OK == hr; hr = m_GroupPolygon->SetNextPoint())
						{
							m_Polygon2->AttachPointAfter(m_PointArray[pointindex++]);
						}
				}

			if(m_CreateBevels)
				{
					if(m_CreateFaces) pointindex -= m_GroupPolygon->GetPointCount();

					for(hr = m_GroupPolygon->SetFirstPoint(); SDP_S_OK == hr;)
						{
							m_PolygonList2->SetHandle(m_PolygonArray[polyindex++]);
							m_Polygon2->AttachPointAfter(m_GroupPolygon->GetPointHandle());

							hr = m_GroupPolygon->SetNextPoint();
							if(SDP_S_OK == hr)
								{
									m_Polygon2->AttachPointAfter(m_GroupPolygon->GetPointHandle());
									m_Polygon2->AttachPointAfter(m_PointArray[pointindex + 1]);
									m_Polygon2->AttachPointAfter(m_PointArray[pointindex]);
								}
							else
								{
									m_GroupPolygon->SetFirstPoint();
									m_Polygon2->AttachPointAfter(m_GroupPolygon->GetPointHandle());
									m_Polygon2->AttachPointAfter(m_PointArray[pointindex - m_GroupPolygon->GetPointCount() + 1]);
									m_Polygon2->AttachPointAfter(m_PointArray[pointindex]);
								}

							pointindex += 1;
						}
				}
		}
*/
}


void CBevel::CalculateGroupGeometry(k3dIGroup* Group, gulong SelectedPoints, gulong SelectedPolys, gulong NewPoints, gulong NewPolys)
{
	if(m_PercentageWidth)
		{
			CalculateGroupPercentage(Group, SelectedPoints, SelectedPolys, NewPoints, NewPolys);
		}
	else
		{
			CalculateGroupNormal(Group, SelectedPoints, SelectedPolys, NewPoints, NewPolys);
		}
}


void CBevel::CalculateGroupPercentage(k3dIGroup* Group, gulong SelectedPoints, gulong SelectedPolys, gulong NewPoints, gulong NewPolys)
{
	gulong pointindex = 0;
	sdpVector3 coords, polycenter, polynormal;

	for(K3DPOLYGONHANDLE polyhandle = Group->Polygons()->GetFirstPolygon(); polyhandle; polyhandle = Group->Polygons()->GetNextPolygon(polyhandle))
		{
			k3dIPolyPointServer* points = Group->Polygons()->GetPolygon(polyhandle)->Points();
			if(points->GetPointCount() < 3) continue;

			// Calculate polygon center
			polycenter = 0.0;
			for(K3DPOLYPOINTHANDLE polypointhandle = points->GetFirstPolyPoint(); polypointhandle; polypointhandle = points->GetNextPolyPoint(polypointhandle))
				{
					points->GetPolyPoint(polypointhandle)->Location()->GetOldXYZ(coords);
					polycenter += coords;
				}
			polycenter /= double(points->GetPointCount());

			// Get polygon normal
			Group->Polygons()->GetPolygon(polyhandle)->GetPolygonNormal(polynormal);

			for(polypointhandle = points->GetFirstPolyPoint(); polypointhandle; polypointhandle = points->GetNextPolyPoint(polypointhandle))
				{
					points->GetPolyPoint(polypointhandle)->Location()->GetOldXYZ(coords);

					coords = ((coords - polycenter) * m_Width) + polycenter + (polynormal * m_Height);

					m_PointArray[pointindex]->Location()->SetOldXYZ(coords);
					m_PointArray[pointindex]->Location()->SetLocalXYZ(coords);

					pointindex++;
				}
		}
}



void CBevel::CalculateGroupNormal(k3dIGroup* Group, gulong SelectedPoints, gulong SelectedPolys, gulong NewPoints, gulong NewPolys)
{
/*
	for(K3DPOINTHANDLE pointhandle = Group->Points()->GetFirstPoint(); pointhandle; pointhandle = Group->Points()->GetNextPoint(pointhandle))
		{
			numpolypoints = m_GroupPolygon->GetPointCount();
			if(numpolypoints > 2)
				{
					m_PolygonList2->SetHandle(m_PolygonArray[polyindex++]);

					m_GroupPolygon->SetFirstPoint();
					m_GroupPolygonLocation->GetOldXYZ(coords1);
					m_GroupPolygon->SetNextPoint();
					m_GroupPolygonLocation->GetOldXYZ(coords2);
					m_GroupPolygon->SetNextPoint();
					m_GroupPolygonLocation->GetOldXYZ(coords3);

					edge1 = coords1 - coords2;
					edge2 = coords3 - coords2;
					normal = edge1 ^ edge2;			// Cross product
					normal.Normalize();

					// Scale normal	using "Height" factor
					height = normal * m_Height;

					// Get the polygon's last point
					for(hr = m_GroupPolygon->SetFirstPoint(); SDP_S_OK == hr; hr = m_GroupPolygon->SetNextPoint())
						{
							m_GroupPolygonLocation->GetOldXYZ(coords1);
						}

					m_GroupPolygon->SetFirstPoint();
					m_GroupPolygonLocation->GetOldXYZ(coords2);
					firstcoords = coords2;

					for(hr = m_GroupPolygon->SetNextPoint(); SDP_S_OK == hr; hr = m_GroupPolygon->SetNextPoint())
						{
							m_GroupPolygonLocation->GetOldXYZ(coords3);

							edge1 = coords2 - coords1;
							edge2 = coords3 - coords2;

							edge3 = edge1 ^ normal;			// Cross product
							edge4 = edge2 ^ normal;			// Cross product

							edge3.Normalize();
							edge4.Normalize();

							width = edge3 + edge4;
							width.Normalize();
							width *= m_Width;

							// Calculate angle adjustment
							theta = acos((edge1 * edge2) / (edge1.Length() * edge2.Length()));
							width *= 1.0 / cos(theta * 0.5);

							coords = coords2 + width + height;

							m_PointList2->SetHandle(m_PointArray[pointindex++]);
							m_PointLocation2->SetOldXYZ(coords);
							m_PointLocation2->SetLocalXYZ(coords);

							coords1 = coords2;
							coords2 = coords3;
						}

					edge1 = coords2 - coords1;
					edge2 = firstcoords - coords2;

					edge3 = edge1 ^ normal;			// Cross product
					edge4 = edge2 ^ normal;			// Cross product

					edge3.Normalize();
					edge4.Normalize();

					width = edge3 + edge4;
					width.Normalize();
					width *= m_Width;

					// Calculate angle adjustment
					theta = acos((edge1 * edge2) / (edge1.Length() * edge2.Length()));
					width *= 1.0 / cos(theta * 0.5);

					coords = coords2 + width + height;

					m_PointList2->SetHandle(m_PointArray[pointindex++]);
					m_PointLocation2->SetOldXYZ(coords);
					m_PointLocation2->SetLocalXYZ(coords);
				}
		}
*/
}







