#include "stdafx.h"
#include "tlbbridge.h"
#include "toollib.h"





CBridgePage::CBridgePage(CBridgeSheet* Sheet)
	: KPage(IDD, Sheet, "Bridge")
{
	//{{AFX_DATA_INIT(CBridgePage)
	//}}AFX_DATA_INIT

	m_Sheet = Sheet;
}




BEGIN_MESSAGE_MAP(CBridgePage, KPage)
	//{{AFX_MSG_MAP(CBridgePage)
	ON_BN_CLICKED(IDC_FIXEDSIZE, OnFixedsize)
	ON_EN_k3dILLFOCUS(IDC_POINTCOUNT, OnKillfocusPointcount)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CBridgePage message handlers


BOOL CBridgePage::OnInitDialog()
{
	KPage::OnInitDialog();

	if(m_Sheet->m_Object->m_FixedPointCount)
		{
			CheckDlgButton(IDC_FIXEDSIZE, 1);
		}

	sdpString s;
	s.format("%d", m_Sheet->m_Object->m_PointCount);
	SetDlgItemText(IDC_POINTCOUNT, s);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}




void CBridgePage::OnFixedsize()
{
	m_Sheet->m_Object->m_FixedPointCount = IsDlgButtonChecked(IDC_FIXEDSIZE) ? TRUE : FALSE;
	if(m_Sheet->m_Object->m_FixedPointCount)
		{
			GetDlgItem(IDC_POINTCOUNT)->EnableWindow(TRUE);
		}
	else
		{
			GetDlgItem(IDC_POINTCOUNT)->EnableWindow(FALSE);
		}
}



void CBridgePage::OnKillfocusPointcount()
{
	KValidateLong(GetDlgItem(IDC_POINTCOUNT)->m_hWnd, m_Sheet->m_Object->m_PointCount, 2, LONG_MAX);
}



////////////////////////////////////////////////////////////////////////////
// CBridgeSheet

CBridgeSheet::CBridgeSheet(CBridge* Object)
: KPrimSheet(Object, theApp.m_hInstance, "Bridge", VERTICAL, "Bridge", TRUE)
{
	m_BridgePage = NULL;
	m_Object = Object;
}


CBridgeSheet::~CBridgeSheet()
{
	m_Object->m_Sheet = NULL;
}



bool CBridgeSheet::Create()
{
	m_BridgePage = new CBridgePage(this);

	if(m_BridgePage)
		{
			AddPage(m_BridgePage);

			Create(IDD);
			return TRUE;
		}

	KFAIL(FALSE);
}


BEGIN_MESSAGE_MAP(CBridgeSheet, KPrimSheet)
	//{{AFX_MSG_MAP(CBridgeSheet)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()






/////////////////////////////////////////////////////////////////////////////
// CBridge

CBridge::CBridge() : k3dPrimTool()
{
	m_FixedPointCount = TRUE;
	m_PointCount = 3;
	m_Group = NULL;

	m_Sheet = NULL;
}


CBridge::~CBridge()
{
	if(m_Sheet)
		m_Sheet->DestroyWindow();
}


///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CBridge::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIDocumentEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIUserEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIOpenGLRender>(this, InterfaceID, Interface)) return SDP_S_OK;

	return SDP_E_NO_INTERFACE;
}


gulong CBridge::AddRef()
{
	return sdpComAddRef(m_ReferenceCount);
}


gulong CBridge::Release()
{
	return sdpComRelease(this, m_ReferenceCount);
}






///////////////////////////////////////////////////////////////
// k3dIObject overrides

sdpResult CBridge::SetObjectContext(k3dIApplication* Server, k3dIObjectContext* Context)
{
	if(sdpSucceeded(k3dPrimTool::SetObjectContext(Server, Context)))
		{
			Context->State()->SetState(k3dIState::VISIBLE, true);
			return SDP_S_OK;
		}
	KFAIL(SDP_E_FAIL);
}


sdpResult CBridge::EditObject()
{
	if(!KOpenDialog(m_Sheet, this)) KFAIL(SDP_E_FAIL);
	return SDP_S_OK;
}




///////////////////////////////8////////////////////////////////
// k3dIDocumentEventClient overrides


sdpResult CBridge::OnUndo(k3dIObject* Sender)
{
	KillAll();
	m_PointArray.clear();
	m_Group = NULL;

	OnModify();
	return SDP_S_OK;
}




///////////////////////////////////////////////////////////////
// k3dIOpenGLRender overrides

sdpResult CBridge::OpenGLRender(k3dICamera* Camera, k3dIOpenGLEngine* Engine)
{
	kglPushAttrib(GL_ALL_ATTRIB_BITS);

	kglDisable(GL_LIGHTING);
	kglDisable(GL_TEXTURE_1D);
	kglDisable(GL_TEXTURE_2D);

	kglColor3d(1.0, 0.0, 0.0);
	kglPointSize(4.0f);

	// Draw selected points
	sdpVector3 coords;
	kglBegin(GL_POINTS);
	for(gulong i = 0; i < m_PointArray.size(); i++)
		{
			m_PointArray[i]->Location()->GetAnimXYZ(coords);
			kglVertex3dv(coords);
		}
	kglEnd();

	kglPopAttrib();

	return SDP_S_OK;
}




///////////////////////////////////////////////////////////////
// k3dIUserEventClient overrides

void CBridgeSheet::OnHelp()
{
	m_Object->StandardHelp("tools", "bridge.html");
}



/////////////////////////////////////////////////////////////////////////////
// CBridge message handlers


sdpResult CBridge::OnLButtonClick(long Flags, long X, long Y)
{
	KUserEvent::OnLButtonClick(Flags, X, Y);

	// Update screen coordinates ...
	m_Document->GetActiveView()->Camera()->SnapshotScreenCoordinates();

	// Iterate all points ...
	sdpVector3 screencoords;
	for(k3dIPoint* point = m_Document->GetFirstPoint(); point; point = m_Document->GetNextPoint(point))
		{
			if(SDP_S_OK != point->State()->QueryVisible()) continue;

			point->Location()->GetScreenXYZ(screencoords);

			if((abs(screencoords[0] - X) < 4) && (abs(screencoords[1] - Y) < 4))
				{
					AddPoint(point);
					if(m_PointArray.size() >= m_PointCount && m_FixedPointCount) CreateBridgePolygon();
					break;
				}
		}

	OnModify();
	return SDP_S_OK;
}


sdpResult CBridge::OnRButtonClick(long Flags, long X, long Y)
{
	KUserEvent::OnRButtonClick(Flags, X, Y);

	// Update screen coordinates ...
	m_Document->GetActiveView()->Camera()->SnapshotScreenCoordinates();

	sdpVector3 screencoords;
	for(k3dIPoint* point = m_Document->GetFirstPoint(); point; point = m_Document->GetNextPoint(point))
		{
			if(SDP_S_OK != point->State()->QueryState(k3dIState::VISIBLESELECTED)) continue;

			point->Location()->GetScreenXYZ(screencoords);

			if((abs(screencoords[0] - X) < 4) &&
				 (abs(screencoords[1] - Y) < 4))
				{
					AddPoint(point);
					if(!(m_FixedPointCount)) CreateBridgePolygon();
					break;
				}
		}

	OnModify();
	return SDP_S_OK;
}


void CBridge::AddPoint(k3dIPoint* Point)
{
	g_assert(Point);

	// Make sure we're only choosing from a single group!
	k3dIGroup* group = Point->Group();
	g_assert(group);
	if(m_Group && (group != m_Group))
		{
			m_IO->Error("You must only choose points from the same group!", "Add Point");
			return;
		}

	m_Group = group;
	m_PointArray.push_back(Point);
}


void CBridge::CreateBridgePolygon()
{
	// Sanity check ...
	g_assert(m_PointArray.size() > 2);

	if(m_PointArray.size() > 2)
		{
			// Allocate new polygon
			k3dIPolygon* polygon = CreatePolygon(m_Group);

			// Add selected points to this polygon
			for(gulong i = 0; i < m_PointArray.size(); i++)
				{
					polygon->Points()->AddPoint(m_PointArray[i]);
				}

			m_PointArray.clear();
			m_Group = NULL;
			OnModify();
		}
}


void CBridge::CreateAll()
{
	k3dPrimTool::CreateAll();
	m_PointArray.clear();
	m_Group = NULL;
}


void CBridge::KillAll()
{
	k3dPrimTool::KillAll();
	m_PointArray.clear();
	m_Group = NULL;
}


void CBridge::OnModify()
{
	// Skip the base class ...
	KUserObject::OnModify();

	// Update dialog as needed ...
	if(m_Sheet)
		m_Sheet->UpdateNumbers();
}

