#include "stdafx.h"
#include "tlbmatchpoints.h"
#include "toollib.h"




CMatchPointsDialog::CMatchPointsDialog(CMatchPoints* Object)
	: k3dDialog(Object, theApp.m_hInstance, "MatchPoints - \"Snaps\" selected points together", VERTICAL, "MatchPoints", TRUE)
{
	//{{AFX_DATA_INIT(CMatchPointsDialog)
	//}}AFX_DATA_INIT

	m_Object = Object;
}


CMatchPointsDialog::~CMatchPointsDialog()
{
	m_Object->m_Dialog = NULL;
}


void CMatchPointsDialog::OnHelp()
{
	m_Object->StandardHelp("tools", "matchpoints.html");
}





bool CMatchPointsDialog::Create()
{
	Create(IDD);
	return TRUE;
}



BEGIN_MESSAGE_MAP(CMatchPointsDialog, k3dDialog)
	//{{AFX_MSG_MAP(CMatchPointsDialog)
	ON_BN_CLICKED(IDC_MATCH, OnMatch)
	ON_EN_k3dILLFOCUS(IDC_RANGE, OnKillfocusRange)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CMatchPointsDialog message handlers


BOOL CMatchPointsDialog::OnInitDialog()
{
	k3dDialog::OnInitDialog();

	m_Object->k3dSetDouble(k3dIChannel::DATA_DISTANCE, GetDlgItem(IDC_RANGE)->m_hWnd, m_Object->m_Range);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}



void CMatchPointsDialog::OnMatch()
{
	m_Object->OnMatch();
}




void CMatchPointsDialog::OnKillfocusRange()
{
	KValidateDouble(k3dIChannel::DATA_DISTANCE, GetDlgItem(IDC_RANGE)->m_hWnd, m_Object->m_Range, 0, DBL_MAX);
}




/////////////////////////////////////////////////////////////////////////////////
// CMatchPoints

CMatchPoints::CMatchPoints() : k3dTool()
{
	m_Range = 0.01;

	m_Point2 = NULL;
	m_PointList2 = NULL;
	m_PointMatchList2 = NULL;
	m_PointLocation2 = NULL;
}




CMatchPoints::~CMatchPoints()
{
	if(m_Point2) m_Point2->Release();
	if(m_PointList2) m_PointList2->Release();
	if(m_PointMatchList2) m_PointMatchList2->Release();
	if(m_PointLocation2) m_PointLocation2->Release();

	if(m_Dialog) m_Dialog->DestroyWindow();
}



bool CMatchPoints::Create()
{
	if(!k3dTool::Create()) KFAIL(FALSE);

	if(sdpFailed(m_Document->CreateObject(KPointEnumerator, IID_k3dIPoint, FALSE, FALSE, FALSE, (void**)&m_Point2))) KFAIL(FALSE);
	if(sdpFailed(m_Point2->QueryInterface(IID_k3dIList, (void**)&m_PointList2))) KFAIL(FALSE);
	if(sdpFailed(m_Point2->QueryInterface(IID_k3dIMatchList, (void**)&m_PointMatchList2))) KFAIL(FALSE);
	if(sdpFailed(m_Point2->QueryInterface(IID_k3dILocation, (void**)&m_PointLocation2))) KFAIL(FALSE);

	return TRUE;
}



///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CMatchPoints::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;

	if(sdpSupportInterface<k3dIDocumentEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIUserEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	return SDP_E_NO_INTERFACE;
}


gulong CMatchPoints::AddRef()
{
	return sdpComAddRef(m_ReferenceCount);
}


gulong CMatchPoints::Release()
{
	return sdpComRelease(this, m_ReferenceCount);
}



///////////////////////////////////////////////////////////////
// k3dIObject overrides

sdpResult CMatchPoints::EditObject()
{
	if(!KOpenDialog(m_Sheet, this)) KFAIL(SDP_E_FAIL);
	return SDP_S_OK;
}



void CMatchPoints::OnMatch()
{
	sdpResult hr;
	double coords[3], coords2[3];

	for(hr = m_PointList2->SetFirst();	SDP_S_OK == hr; hr = m_PointList2->SetNext())
		{
			if(SDP_S_OK == m_Point2->QueryShow() && SDP_S_OK == m_Point2->QuerySelected())
				{
					m_Point2->SetUser1(TRUE);
				}
			else
				{
					m_Point2->SetUser1(FALSE);
				}
		}

	m_PointMatchList->ClearCriteria();
	m_PointMatchList->SetShowCriteria(TRUE, TRUE);
	m_PointMatchList->SetSelectableCriteria(TRUE, TRUE);

	m_PointMatchList2->ClearCriteria();
	m_PointMatchList2->SetUser1Criteria(TRUE, TRUE);


	for(hr = m_PointMatchList->SetFirstMatch(); SDP_S_OK == hr; hr = m_PointMatchList->SetNextMatch())
		{
			m_PointLocation->GetOldXYZ(coords);

			for(hr = m_PointMatchList2->SetFirstMatch(); SDP_S_OK == hr; hr = m_PointMatchList2->SetNextMatch())
				{
					if(m_Point->GetHandle() == m_Point2->GetHandle()) continue;

					m_PointLocation2->GetOldXYZ(coords2);

					if((fabs(coords[0] - coords2[0]) <= m_Range) &&
						 (fabs(coords[1] - coords2[1]) <= m_Range) &&
						 (fabs(coords[2] - coords2[2]) <= m_Range))
						{
							m_Point->SetUser1(FALSE);
							m_Point2->SetUser1(FALSE);
							m_PointLocation2->SetLocalXYZ(coords);
						}
				}

		}

	OnModify();
}


