#ifndef SPHEREIZER_H
#define SPHEREIZER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Defines the CSphereizer K-3D object, which applies an animated "sphereize" transformation to geometry
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3dpersistobject.h>
#include <k3ddialog.h>
#include <k3dchannelbutton.h>
#include <k3dvariable.h>

// Forward class references ...
class CSphereizer;

/////////////////////////////////////////////////////////////////////////////////////
// CSphereizerDialog

/// Encapsulates the Sphereizer UI
class CSphereizerDialog : public k3dDialog
{
friend class CSphereizer;
public:
	CSphereizerDialog();
	
	/// Called to initialize the UI
	bool Create(CSphereizer* Object);

protected:
	/// Stores a pointer to the owning object
	CSphereizer* m_Object;

	/// Handles our custom events
	void OnEvent(sdpGtkEvent* Event);

	/// Called to initialize controls at creation time
	bool InitializeControls();

};

////////////////////////////////////////////////////////////////////////////////////
// CSphereizer

/// Encapsulates the Sphereizer object
class CSphereizer : public k3dPersistObject
{
friend class CSphereizerDialog;
public:
	CSphereizer();
	~CSphereizer();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectRenamed(SDPCSTRING NewName);
	sdpResult ObjectDeleted();

	// k3dIAnimate overrides
	sdpResult PopAnimate();

	// k3dIPersist overrides
	sdpResult InitNew();

protected:
	k3dScalarChannel m_SphereizeFactor;
	
	CSphereizerDialog m_UserInterface;
};

#endif // SPHEREIZER_H
