#ifndef TRANSFORMER_H
#define TRANSFORMER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CTransformer class, which is used to animate the position, orientation, and scale of objects
		\author Timothy M. Shead (tshead@k-3d.com)
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3dmesh.h>

#include <k3ddialog.h>
#include <k3dchannelbutton.h>

// Forward class declarations ...
class CTransformer;
class sdpGtkEvent;

/////////////////////////////////////////////////////////////////////////////
// CTransformerDialog

class CTransformerDialog : public k3dDialog
{
public:
	CTransformerDialog();

	bool Create(CTransformer* Object);

	CTransformer* m_Object;

	// Event handling ...
	void OnEvent(sdpGtkEvent* Event);
	void OnTranslateLast();
	void OnTranslateType();
	
	// Update controls ...
	bool InitializeControls();
	void UpdateControls();
};

//////////////////////////////////////////////////////////////////////
// CTransformer

/// Used to animate position, orientation, and scale
class CTransformer : public k3dMesh
{
public:
	CTransformer();
	~CTransformer();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectRenamed(SDPCSTRING NewName);
	sdpResult ObjectDeleted();

	// k3dIEditorRender overrides
	sdpResult EditorRender(k3dIEditorEngine* Engine);

	// k3dIPersist overrides
	sdpResult InitNew();

	CTransformerDialog m_UserInterface;

	k3dVariable<gdouble> m_GlobalCycles;
	k3dVariable<gdouble> m_GlobalPhase;

	k3dVariable<bool> m_EnableTranslation;
	k3dVariable<bool> m_AbsolutePath;
	k3dVariable<bool> m_TranslateLast;

	typedef enum
	{
		POINT,
		LINEAR,
		NURBS,
	} TranslateType;

	class VariableTranslateType : public k3dCustomVariable<TranslateType>
	{
	protected:
		sdpString ConvertToString() { return sdpFormatString("%d", m_Value); }
		bool ConvertFromString(SDPCSTRING String) { m_Value = TranslateType(atoi(String)); return true; }
	};

	VariableTranslateType m_TranslateType;

	k3dScalarChannel m_XRotation;
	k3dScalarChannel m_YRotation;
	k3dScalarChannel m_ZRotation;
	k3dScalarChannel m_XScale;
	k3dScalarChannel m_YScale;
	k3dScalarChannel m_ZScale;
	k3dScalarChannel m_Translation;
	
	sdpVector3 GetPointPosition(gdouble PathPercent);
	sdpVector3 GetLinearPosition(gdouble PathPercent);
	sdpVector3 GetNURBSPosition(gdouble PathPercent);
	
	sdpVector3 GetPosition(gdouble Time, gdouble Delta);
	sdpVector3 GetOrientation(gdouble Time, gdouble Delta);
	sdpVector3 GetScale(gdouble Time, gdouble Delta);
};

#endif // !TRANSFORMER_H
