#ifndef TWISTER_H
#define TWISTER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Defines the CTwister K-3D object, which applies an animated "twist" transformation to geometry
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3dpersistobject.h>
#include <k3ddialog.h>
#include <k3dchannelbutton.h>
#include <k3dvariable.h>

// Forward class references ...
class CTwister;

/////////////////////////////////////////////////////////////////////////////////////
// CTwisterDialog

/// Encapsulates the Twister UI
class CTwisterDialog : public k3dDialog
{
friend class CTwister;
public:
	CTwisterDialog();
	
	/// Called to initialize the UI
	bool Create(CTwister* Object);

protected:
	/// Stores a pointer to the owning object
	CTwister* m_Object;

	/// Handles our custom events
	void OnEvent(sdpGtkEvent* Event);

	/// Called to initialize controls at creation time
	bool InitializeControls();

};

////////////////////////////////////////////////////////////////////////////////////
// CTwister

/// Encapsulates the Twister object
class CTwister : public k3dPersistObject
{
friend class CTwisterDialog;
public:
	CTwister();
	~CTwister();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectRenamed(SDPCSTRING NewName);
	sdpResult ObjectDeleted();

	// k3dIAnimate overrides
	sdpResult PopAnimate();

	// k3dIPersist overrides
	sdpResult InitNew();

protected:
	k3dScalarChannel m_X;
	k3dScalarChannel m_Y;
	k3dScalarChannel m_Z;
	
	CTwisterDialog m_UserInterface;
/*
	k3dVariablebool						m_XFalloff;
	k3dVariablebool						m_YFalloff;
	k3dVariablebool						m_ZFalloff;
	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Xfalloff;
	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Yfalloff;
	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Zfalloff;

	double m_StartPoint[3];
*/	
};

#endif // TWISTER_H
