#include "stdafx.h"

#include "olbffdeformer.h"
#include "objectlib.h"



////////////////////////////////////////////////////////////////////////
// CPoint

CFFDeformer::CPoint::CPoint()
{
}

CFFDeformer::CPoint::CPoint(k3dIPoint* Point, sdpVector3 Original)
{
	m_Point = Point;
	m_Original = Original;
}


/////////////////////////////////////////////////////////////////////////
// CFFDeformer

CFFDeformer::CFFDeformer() : k3dPersistObject()
{
	m_Sheet = NULL;

	m_Type() = BONES;
	m_CustomStrength() = FALSE;

	AddVariable(&m_Type, "type", V_TYPE);
	AddVariable(&m_CustomStrength, "customstrength", V_CUSTOMSTRENGTH);

	AddObject(&m_Range, "range", OC_RANGE);
	AddObject(&m_Strength, "strength", OC_STRENGTH);
	AddObject(&m_Strengthfalloff, "strengthfalloff", OC_STRENGTHFALLOFF);
}


CFFDeformer::~CFFDeformer()
{
	if(m_Sheet)
		m_Sheet->DestroyWindow();
}



///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CFFDeformer::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIDatabaseNotify>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIUserEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIAnimate>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dITimeServer>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIPersist>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIPointServer>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIPolygonServer>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dITarget>(this, InterfaceID, Interface)) return SDP_S_OK;

	return SDP_E_NO_INTERFACE;
}


gulong CFFDeformer::AddRef()
{
	return sdpComAddRef(m_ReferenceCount);
}


gulong CFFDeformer::Release()
{
	return sdpComRelease(this, m_ReferenceCount);
}



///////////////////////////////////////////////////////////////
// k3dIObject overrides

sdpResult CFFDeformer::EditObject()
{
	if(!KOpenDialog(m_Sheet, this)) KFAIL(SDP_E_FAIL);
	return SDP_S_OK;
}



sdpResult CFFDeformer::CopyFrom(k3dIObject* Object)
{
	if(sdpFailed(k3dPersistObject::CopyFrom(Object))) KFAIL(SDP_E_FAIL);

	CFFDeformer* object = (CFFDeformer*)Object;

	m_ActiveList = object->m_ActiveList;
	m_GridList = object->m_GridList;
	m_GridPolys = object->m_GridPolys;

	return SDP_S_OK;
}



///////////////////////////////////////////////////////////////
// k3dIDatabaseNotify overrides

sdpResult CFFDeformer::OnPointsDeparting()
{
	gulong i;
	KHANDLE handle;

	// Save the active points that still exist ...
	vector<CPoint> newactivelist;
	for(i = 0; i < m_ActiveList.size(); i++)
		{
			handle = m_ActiveList[i].m_Point;

			for(k3dIPoint* point = m_Document->GetFirstPoint(); point; point = m_Document->GetNextPoint(point))
				{
					if(point == handle)
						{
							newactivelist.push_back(m_ActiveList[i]);
							break;
						}
				}
		}

	m_ActiveList = newactivelist;

	// Save the grid points that still exist ...
	vector<CPoint> newgridlist;
	for(i = 0; i < m_GridList.size(); i++)
		{
			handle = m_GridList[i].m_Point;

			for(k3dIPoint* point = m_Document->GetFirstPoint(); point; point = m_Document->GetNextPoint(point))
				{
					if(point == handle)
						{
							newgridlist.push_back(m_GridList[i]);
							break;
						}
				}
		}

	m_GridList = newgridlist;

	// Update UI ...
	if(m_Sheet)
		m_Sheet->m_FFDeformerPage->OnUpdateNumbers();

	return SDP_S_OK;
}


sdpResult CFFDeformer::OnPolygonsDeparting()
{
	// Store the remaining polys ...
	vector<k3dIPolygon*> newpolylist;
	for(gulong i = 0; i < m_GridPolys.size(); i++)
		{
			for(k3dIPolygon* poly = m_Document->GetFirstPolygon(); poly; poly = m_Document->GetNextPolygon(poly))
				{
					if(poly == m_GridPolys[i])
						{
							newpolylist.push_back(m_GridPolys[i]);
							break;
						}
				}
		}

	m_GridPolys = newpolylist;

	// Update UI ...
	if(m_Sheet)
		m_Sheet->m_FFDeformerPage->OnUpdateNumbers();

	return SDP_S_OK;
}



///////////////////////////////////////////////////////////////


sdpResult CFFDeformer::ObjectRenamed(SDPCSTRING NewName)
{
	if(m_Sheet)
		m_Sheet->RenameObject(NewName);

	return SDP_S_OK;
}




///////////////////////////////////////////////////////////////
// k3dIAnimate overrides

sdpResult CFFDeformer::PopAnimate()
{
	gulong i, j;

	sdpVector3 coords1, coords2;
	sdpVector3 delta;
	double distance, pstrength;

	// Calculate FFDeformer values ...
	double range = m_Range->GetValue1(m_CurrentTime, m_DeltaTime);
	double rangex2 = range * range;
	double strength = m_Strength->GetValue1(m_CurrentTime, m_DeltaTime);


	// Move active points ...
	if(BONES == m_Type())
		{
			// Calculate & store grid point deltas
			for(i = 0; i < m_GridList.size(); i++)
				{
					m_GridList[i].m_Point->Location()->GetAnimXYZ(coords1);
					m_GridList[i].m_Delta = coords1 - m_GridList[i].m_Original;
				}

			// Enumerate active points
			for(i = 0; i < m_ActiveList.size(); i++)
				{
					coords2 = m_ActiveList[i].m_Original;

					// Enumerate grid points
					for(j = 0; j < m_GridList.size(); j++)
						{
							delta = m_GridList[j].m_Original - m_ActiveList[i].m_Original;

							distance = delta.Length2();				// Don't waste time calculating square roots ...
							if(distance > rangex2) continue;

							pstrength = sqrt(distance) / range;

							if(m_CustomStrength())
								{
									pstrength = m_Strengthfalloff->GetValue1(pstrength, 0.001) * strength;
								}
							else
								{
									pstrength = (1.0 - pstrength) * strength;
								}

							coords2 += pstrength * m_GridList[j].m_Delta;
						}

					m_ActiveList[i].m_Point->Location()->SetAnimXYZ(coords2);
				}
		}
	else if(MAGNETS == m_Type())
		{
			// Calculate & store grid point locations
			for(i = 0; i < m_GridList.size(); i++)
				{
					m_GridList[i].m_Point->Location()->GetAnimXYZ(m_GridList[i].m_Delta);
				}

			// Enumerate active points
			for(i = 0; i < m_ActiveList.size(); i++)
				{
					coords2 = m_ActiveList[i].m_Original;

					// Enumerate grid points
					for(j = 0; j < m_GridList.size(); j++)
						{
							delta = m_GridList[j].m_Delta - coords2;

							distance = delta.Length2();		// Don't waste time calculating square roots!
							if(distance > rangex2) continue;

							pstrength = sqrt(distance) / range;

							if(m_CustomStrength())
								{
									pstrength = m_Strengthfalloff->GetValue1(pstrength, .001) * strength;
								}
							else
								{
									pstrength = (1.0 - pstrength) * strength;
								}

							coords2 += pstrength * delta;
						}

					m_ActiveList[i].m_Point->Location()->SetAnimXYZ(coords2);
				}

			// Enumerate child objects and alter
			for(k3dIObjectContext* context = m_Document->GetFirstObjectContext(); context; context = m_Document->GetNextObjectContext(context))
				{
					// Is the object a descendant of this FFDeformer?
					if(SDP_S_OK != context->QueryAncestor(m_ObjectContext)) continue;

					// Does this object support k3dILocationServer?
					k3dILocationServer* locationserver;
					if(SDP_S_OK != context->Object()->QueryInterface(IID_k3dILocationServer, (void**)&locationserver)) continue;

					for(K3DLOCATIONHANDLE location = locationserver->GetFirstLocation(); location; location = locationserver->GetNextLocation(location))
						{
							locationserver->GetLocation(location)->GetAnimXYZ(coords2);

							// Enumerate grid points
							for(i = 0; i < m_GridList.size(); i++)
								{
									delta = m_GridList[i].m_Delta - coords2;

									distance = delta.Length2();
									if(distance > rangex2) continue;

									pstrength = sqrt(distance) / range;

									if(m_CustomStrength())
										{
											pstrength = m_Strengthfalloff->GetValue1(pstrength, .001) * strength;
										}
									else
										{
											pstrength = (1.0 - pstrength) * strength;
										}

									coords2 += pstrength * delta;
								}

							locationserver->GetLocation(location)->SetAnimXYZ(coords2);
						}
					locationserver->Release();
				}
		}

	return SDP_S_OK;
}



///////////////////////////////////////////////////////////////
// k3dIPersist overrides

sdpResult CFFDeformer::InitNew()
{
	if(sdpFailed(k3dPersistObject::InitNew()))
		KFAIL(SDP_E_FAIL);

	if(sdpFailed(m_Range.CreateScalar(m_Document, "FFDeformer range", 10.0, FALSE, TRUE, -DBL_MAX, DBL_MAX, "Time", "Range", k3dIChannel::DATA_DISTANCE, TRUE, this, m_ObjectContext->ObjectID()))) KFAIL(SDP_E_FAIL);
	if(sdpFailed(m_Strength.CreateScalar(m_Document, "FFDeformer strength", 1.0, FALSE, TRUE, -DBL_MAX, DBL_MAX, "Time", "Strength", k3dIChannel::DATA_SCALAR, TRUE, this, m_ObjectContext->ObjectID()))) KFAIL(SDP_E_FAIL);
	if(sdpFailed(m_Strengthfalloff.CreateScalar(m_Document, "FFDeformer strength falloff", 0.0, TRUE, FALSE, 0.0, 1.0, "Distance", "Strength", k3dIChannel::DATA_SCALAR, TRUE, this, m_ObjectContext->ObjectID()))) KFAIL(SDP_E_FAIL);
	m_Strengthfalloff->SetLinear1(1.0, 0.0);

	return SDP_S_OK;
}

sdpResult CFFDeformer::Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies)
{
	if(sdpFailed(k3dPersistObject::Save(Stream)))
		KFAIL(SDP_E_FAIL);

	if(sdpFailed(SaveActivePoints(Stream)))
		KFAIL(SDP_E_FAIL);

	if(sdpFailed(SaveGridPoints(Stream)))
		KFAIL(SDP_E_FAIL);

	if(sdpFailed(SaveGridPolygons(Stream)))
		KFAIL(SDP_E_FAIL);

	return SDP_S_OK;
}

sdpResult CFFDeformer::SaveActivePoints(LPSTORAGE Storage)
{
	KStream stream;
	if(sdpFailed(stream.Create(Storage, "ActivePoints"))) KFAIL(SDP_E_FAIL);

	gulong quantity = m_ActiveList.size();
	stream << quantity;

	for(gulong i = 0; i < m_ActiveList.size(); i++)
		{
			stream << m_ActiveList[i].m_Point;
			stream << m_ActiveList[i].m_Original;
		}

	return stream.GetState();
}


sdpResult CFFDeformer::SaveGridPoints(LPSTORAGE Storage)
{
	KStream stream;
	if(sdpFailed(stream.Create(Storage, "GridPoints"))) KFAIL(SDP_E_FAIL);

	gulong quantity = m_GridList.size();
	stream << quantity;

	for(gulong i = 0; i < m_GridList.size(); i++)
		{
			stream << m_GridList[i].m_Point;
			stream << m_GridList[i].m_Original;
		}

	return stream.GetState();
}


sdpResult CFFDeformer::SaveGridPolygons(LPSTORAGE Storage)
{
	KStream stream;
	if(sdpFailed(stream.Create(Storage, "GridPolygons"))) KFAIL(SDP_E_FAIL);

	gulong quantity = m_GridPolys.size();
	stream << quantity;

	for(gulong i = 0; i < m_GridPolys.size(); i++)
		{
			stream << m_GridPolys[i];
		}

	return stream.GetState();
}



sdpResult CFFDeformer::Load(LPSTORAGE Storage)
{
	if(sdpFailed(k3dPersistObject::Load(Storage))) KFAIL(SDP_E_FAIL);
	if(sdpFailed(LoadActivePoints(Storage))) KFAIL(SDP_E_FAIL);
	if(sdpFailed(LoadGridPoints(Storage))) KFAIL(SDP_E_FAIL);
	if(sdpFailed(LoadGridPolygons(Storage))) KFAIL(SDP_E_FAIL);

	return SDP_S_OK;
}


sdpResult CFFDeformer::LoadActivePoints(LPSTORAGE Storage)
{
	KStream stream;
	if(sdpFailed(stream.Open(Storage, "ActivePoints"))) KFAIL(SDP_E_FAIL);

	gulong quantity;
	stream >> quantity;

	for(gulong i = 0; i < quantity; i++)
		{
			KHANDLE savehandle;
			stream >> savehandle;

			sdpVector3 original;
			stream >> original;

			m_ActiveList.push_back(CPoint((k3dIPoint*)savehandle, original));
		}

	return stream.GetState();
}


sdpResult CFFDeformer::LoadGridPoints(LPSTORAGE Storage)
{
	KStream stream;
	if(sdpFailed(stream.Open(Storage, "GridPoints"))) KFAIL(SDP_E_FAIL);

	gulong quantity;
	stream >> quantity;

	for(gulong i = 0; i < quantity; i++)
		{
			KHANDLE savehandle;
			stream >> savehandle;

			sdpVector3 original;
			stream >> original;

			m_GridList.push_back(CPoint((k3dIPoint*)savehandle, original));
		}

	return stream.GetState();
}


sdpResult CFFDeformer::LoadGridPolygons(LPSTORAGE Storage)
{
	KStream stream;
	if(sdpFailed(stream.Open(Storage, "GridPolygons"))) KFAIL(SDP_E_FAIL);

	gulong quantity;
	stream >> quantity;

	for(gulong i = 0; i < quantity; i++)
		{
			KHANDLE savehandle;
			stream >> savehandle;

			m_GridPolys.push_back((k3dIPolygon*)savehandle);
		}

	return stream.GetState();
}


sdpResult CFFDeformer::LoadComplete()
{
	gulong i;

	// Convert activepoint save handles ...
	for(i = 0; i < m_ActiveList.size(); i++)
		{
			m_ActiveList[i].m_Point = m_Document->ConvertSavedPoint(m_ActiveList[i].m_Point);
		}

	// Convert gridpoint save handles ...
	for(i = 0; i < m_GridList.size(); i++)
		{
			m_GridList[i].m_Point = m_Document->ConvertSavedPoint(m_GridList[i].m_Point);
		}

	// Convert gridpolygon save handles ...
	for(i = 0; i < m_GridPolys.size(); i++)
		{
			m_GridPolys[i] = m_Document->ConvertSavedPolygon(m_GridPolys[i]);
		}

	return SDP_S_OK;
}


//////////////////////////////////////////////////////////////////////////////
// User methods

void CFFDeformer::OnActiveAdd()
{
	sdpVector3 coords;

	for(k3dIPoint* point = m_Document->GetFirstPoint(); point; point = m_Document->GetNextPoint(point))
		{
			if(SDP_S_OK != point->State()->QueryState(k3dIState::VISIBLESELECTED)) continue;

			point->Location()->GetLocalXYZ(coords);
			m_ActiveList.push_back(CPoint(point, coords));
		}
}



void CFFDeformer::OnActiveRemove()
{
	for(k3dIPoint* point = m_Document->GetFirstPoint(); point; point = m_Document->GetNextPoint(point))
		{
			if(SDP_S_OK != point->State()->QueryState(k3dIState::VISIBLESELECTED)) continue;

			for(PointIterator i = m_ActiveList.begin(); i != m_ActiveList.end(); i++)
				{
					if((*i).m_Point == point)
						{
							m_ActiveList.erase(i);
							break;
						}
				}
		}
}


void CFFDeformer::OnActiveRemoveAll()
{
	m_ActiveList.clear();
}


void CFFDeformer::OnActiveSelect()
{
	for(PointIterator i = m_ActiveList.begin(); i != m_ActiveList.end(); i++)
		{
			(*i).m_Point->State()->SetState(k3dIState::VISIBLESELECTED, TRUE);
		}
}


void CFFDeformer::OnGridAdd()
{
	sdpVector3 coords;

	for(k3dIPoint* point = m_Document->GetFirstPoint(); point; point = m_Document->GetNextPoint(point))
		{
			if(SDP_S_OK != point->State()->QueryState(k3dIState::VISIBLESELECTED)) continue;

			point->Location()->GetLocalXYZ(coords);
			m_GridList.push_back(CPoint(point, coords));
		}
}



void CFFDeformer::OnGridRemove()
{
	for(k3dIPoint* point = m_Document->GetFirstPoint(); point; point = m_Document->GetNextPoint(point))
		{
			if(SDP_S_OK != point->State()->QueryState(k3dIState::VISIBLESELECTED)) continue;

			for(PointIterator i = m_GridList.begin(); i != m_GridList.end(); i++)
				{
					if((*i).m_Point == point)
						{
							m_GridList.erase(i);
							break;
						}
				}
		}
}


void CFFDeformer::OnGridRemoveAll()
{
	m_GridList.clear();
}


void CFFDeformer::OnGridSelect()
{
	for(PointIterator i = m_GridList.begin(); i != m_GridList.end(); i++)
		{
			(*i).m_Point->State()->SetState(k3dIState::VISIBLESELECTED, TRUE);
		}
}



/////////////////////////////////////////////////////////////////////////////
// CFFDeformerSheet

CFFDeformerSheet::CFFDeformerSheet(CFFDeformer* FFDeformer)
	: KSheet((k3dPersistObject*)FFDeformer, theApp.m_hInstance, "FFDeformer - deforms \"active\" points under \"grid\" point control", VERTICAL, "FFDeformer")
{
	m_Object = FFDeformer;
	
	m_FFDeformerPage = NULL;
}



CFFDeformerSheet::~CFFDeformerSheet()
{
	m_Object->m_Sheet = NULL;
}



bool CFFDeformerSheet::Create()
{
	m_FFDeformerPage = new CFFDeformerPage(this);

	if(m_FFDeformerPage)
		{
			AddPage(m_FFDeformerPage);
			Create(IDD);

			return TRUE;
		}

	KFAIL(FALSE);
}



void CFFDeformerSheet::OnHelp()
{
	m_Object->StandardHelp("objects", "ffdeformer.html");
}


BEGIN_MESSAGE_MAP(CFFDeformerSheet, KSheet)
	//{{AFX_MSG_MAP(CFFDeformerSheet)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CFFDeformerSheet message handlers


BOOL CFFDeformerSheet::OnInitDialog() 
{
	KSheet::OnInitDialog();
	
	RenameObject(m_Object->m_ObjectContext->Name());

	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}





/////////////////////////////////////////////////////////////////////////////
// CFFDeformerPage property page

CFFDeformerPage::CFFDeformerPage(CFFDeformerSheet* Parent)
  : KPage(IDD, Parent, "Type")
{
	//{{AFX_DATA_INIT(CFFDeformerPage)
	//}}AFX_DATA_INIT

	m_Sheet = Parent;
}




BEGIN_MESSAGE_MAP(CFFDeformerPage, KPage)
	//{{AFX_MSG_MAP(CFFDeformerPage)
	ON_BN_CLICKED(IDC_ACTIVEADD, OnActiveadd)
	ON_BN_CLICKED(IDC_ACTIVEREMOVE, OnActiveremove)
	ON_BN_CLICKED(IDC_ACTIVEREMOVEALL, OnActiveremoveall)
	ON_BN_CLICKED(IDC_ACTIVESELECT, OnActiveselect)
	ON_BN_CLICKED(IDC_GRIDADD, OnGridadd)
	ON_BN_CLICKED(IDC_GRIDREMOVE, OnGridremove)
	ON_BN_CLICKED(IDC_GRIDREMOVEALL, OnGridremoveall)
	ON_BN_CLICKED(IDC_GRIDSELECT, OnGridselect)
	ON_BN_CLICKED(IDC_BONES, OnBones)
	ON_BN_CLICKED(IDC_MAGNETS, OnMagnets)
	ON_BN_CLICKED(IDC_RESET, OnReset)
	ON_BN_CLICKED(IDC_CUSTOMSTRENGTH, OnCustomstrength)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CFFDeformerPage message handlers

BOOL CFFDeformerPage::OnInitDialog() 
{
	KPage::OnInitDialog();

	CheckDlgButton(IDC_CUSTOMSTRENGTH, m_Sheet->m_Object->m_CustomStrength());

	if(CFFDeformer::BONES == m_Sheet->m_Object->m_Type())
		{
			CheckRadioButton(IDC_BONES, IDC_MAGNETS, IDC_BONES);
		}
	else if(CFFDeformer::MAGNETS == m_Sheet->m_Object->m_Type())
		{
			CheckRadioButton(IDC_BONES, IDC_MAGNETS, IDC_MAGNETS);
		}

	m_Sheet->m_Object->m_Strength->Attach(GetDlgItem(IDC_STRENGTH)->m_hWnd);
	m_Sheet->m_Object->m_Range->Attach(GetDlgItem(IDC_RANGE)->m_hWnd);
	m_Sheet->m_Object->m_Strengthfalloff->Attach(GetDlgItem(IDC_STRENGTHFALLOFF)->m_hWnd);

	OnUpdateNumbers();
		
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


void CFFDeformerPage::OnUpdateNumbers()
{
	k3dSetLong(GetDlgItem(IDC_ACTIVEPOINTS)->m_hWnd, m_Sheet->m_Object->m_ActiveList.size());
	k3dSetLong(GetDlgItem(IDC_GRIDPOINTS)->m_hWnd, m_Sheet->m_Object->m_GridList.size());
}



void CFFDeformerPage::OnActiveadd() 
{
	m_Sheet->m_Object->OnActiveAdd();
	OnUpdateNumbers();
	OnModify();
}

void CFFDeformerPage::OnActiveremove() 
{
	m_Sheet->m_Object->OnActiveRemove();
	OnUpdateNumbers();
	OnModify();
}

void CFFDeformerPage::OnActiveremoveall() 
{
	m_Sheet->m_Object->OnActiveRemoveAll();
	OnUpdateNumbers();
	OnModify();
}


void CFFDeformerPage::OnActiveselect() 
{
	m_Sheet->m_Object->OnActiveSelect();
	OnModify();
}


void CFFDeformerPage::OnGridadd() 
{
	m_Sheet->m_Object->OnGridAdd();
	OnUpdateNumbers();
	OnModify();
}

void CFFDeformerPage::OnGridremove() 
{
	m_Sheet->m_Object->OnGridRemove();
	OnUpdateNumbers();
	OnModify();
}

void CFFDeformerPage::OnGridremoveall() 
{
	m_Sheet->m_Object->OnGridRemoveAll();
	OnUpdateNumbers();
	OnModify();
}

void CFFDeformerPage::OnGridselect() 
{
	m_Sheet->m_Object->OnGridSelect();
	OnModify();
}

void CFFDeformerPage::OnBones() 
{
	m_Sheet->m_Object->m_Type() = CFFDeformer::BONES;	
	OnModify();
}

void CFFDeformerPage::OnMagnets() 
{
	m_Sheet->m_Object->m_Type() = CFFDeformer::MAGNETS;	
	OnModify();
}

void CFFDeformerPage::OnReset() 
{
//	m_Sheet->OnReset();	
}

void CFFDeformerPage::OnCustomstrength() 
{
	m_Sheet->m_Object->m_CustomStrength() = IsDlgButtonChecked(IDC_CUSTOMSTRENGTH) ? TRUE : FALSE;
	OnModify();
}



