
#include "resource.h"

// Forward class references
class CFFDeformerPage;
class CFFDeformerSheet;



class CFFDeformer : public k3dPersistObject
{
friend class CFFDeformerSheet;
friend class CFFDeformerPage;
public:
	typedef enum
	{
		BONES,
		MAGNETS,
	} Type;

	class CPoint
	{
	public:
		CPoint();
		CPoint(k3dIPoint* Point, sdpVector3 Original);

		k3dIPoint* m_Point;
		sdpVector3 m_Original;
		sdpVector3 m_Delta;
	};

	CFFDeformerSheet* m_Sheet;

	class k3dVariableType : public k3dVariable<Type>
	{
	public:
		virtual sdpString ToString() { return sdpFormatString("%d", m_Variable); }
		sdpResult FromString(SDPCSTRING String) { return SDP_E_FAIL; }
	};

	k3dVariableType m_Type;
	k3dVariablebool m_CustomStrength;	

	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Range;
	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Strength;
	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Strengthfalloff;

	typedef std::vector<CPoint> PointList;
	typedef PointList::iterator PointIterator;
	PointList m_ActiveList;
	PointList m_GridList;

	typedef std::vector<k3dIPolygon*> PolygonList;
	typedef PolygonList::iterator PolygonIterator;
	PolygonList m_GridPolys;

public:											 
	CFFDeformer();
	~CFFDeformer();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectRenamed(SDPCSTRING NewName);

	// k3dIDocumentEventClient overrides
	sdpResult OnPointsDeparting();
	sdpResult OnPolygonsDeparting();

	// k3dIAnimate overrides
	sdpResult PopAnimate();

	// k3dITimeServer overrides

	// k3dIPersist overrides
	sdpResult InitNew();
	sdpResult Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies);
	sdpResult Load(sdpXmlIElement* Element);
	sdpResult LoadComplete();

	sdpResult SaveActivePoints(LPSTORAGE Storage);
	sdpResult SaveGridPoints(LPSTORAGE Storage);
	sdpResult SaveGridPolygons(LPSTORAGE Storage);
	sdpResult LoadActivePoints(LPSTORAGE Storage);
	sdpResult LoadGridPoints(LPSTORAGE Storage);
	sdpResult LoadGridPolygons(LPSTORAGE Storage);

	// User methods
	void OnActiveAdd();
	void OnActiveRemove();
	void OnActiveRemoveAll();
	void OnActiveSelect();
	void OnGridAdd();
	void OnGridRemove();
	void OnGridRemoveAll();
	void OnGridSelect();
};







/////////////////////////////////////////////////////////////////////////////
// CFFDeformerSheet

class CFFDeformerSheet : public KSheet
{
public:
	CFFDeformerSheet(CFFDeformer* FFDeformer);
	~CFFDeformerSheet();

	bool Create();
	void OnHelp();


	CFFDeformer* m_Object;
	CFFDeformerPage* m_FFDeformerPage;

	//{{AFX_VIRTUAL(CFFDeformerSheet)
	//}}AFX_VIRTUAL
	//{{AFX_MSG(CFFDeformerSheet)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};





/////////////////////////////////////////////////////////////////////////////
// CFFDeformerPage dialog

class CFFDeformerPage : public KPage
{
public:
	CFFDeformerPage(CFFDeformerSheet* Parent);

	void OnUpdateNumbers();

	CFFDeformerSheet* m_Sheet;

	//{{AFX_DATA(CFFDeformerPage)
	//}}AFX_DATA

	//{{AFX_VIRTUAL(CFFDeformerPage)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CFFDeformerPage)
	virtual BOOL OnInitDialog();
	afx_msg void OnActiveadd();
	afx_msg void OnActiveremove();
	afx_msg void OnActiveremoveall();
	afx_msg void OnActiveselect();
	afx_msg void OnGridadd();
	afx_msg void OnGridremove();
	afx_msg void OnGridremoveall();
	afx_msg void OnGridselect();
	afx_msg void OnBones();
	afx_msg void OnMagnets();
	afx_msg void OnReset();
	afx_msg void OnCustomstrength();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};






