// SDPGL
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief The top-level include file for SDPGL
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPGL_H
#define SDPGL_H

#include <sdpcompiler.h>

#if defined SDPWIN32
#include <windows.h>
#endif // SDPWIN32

#include <GL/gl.h>
#include <GL/glu.h>

#if defined SDPUNIX
#include <GL/glx.h>
#endif // SDPUNIX

#include <iostream>

////////////////////////////////////////////////////////////////////////////////
// Functions that are always available

#include <sdpglprototypes.h>

////////////////////////////////////////////////////////////////////////////////
// Functions that are only available when tracing is enabled

#if defined SDPGL_TRACE

#if defined SDPMICROSOFT
#pragma message("*** Compiling-in OpenGL tracing through sdpgl ...")
#endif // SDPMICROSOFT

// Bring in the sdpglXXXX() functions ...
#include <sdpgltraceprototypes.h>

// Hijack OpenGL calls ...
#define glAccum sdpglAccum
#define glAlphaFunc sdpglAlphaFunc
#define glAreTexturesResident sdpglAreTexturesResident
#define glArrayElement sdpglArrayElement
#define glBegin sdpglBegin
#define glBindTexture sdpglBindTexture
#define glBitmap sdpglBitmap
#define glBlendFunc sdpglBlendFunc
#define glCallList sdpglCallList
#define glCallLists sdpglCallLists
#define glClear sdpglClear
#define glClearAccum sdpglClearAccum
#define glClearColor sdpglClearColor
#define glClearDepth sdpglClearDepth
#define glClearIndex sdpglClearIndex
#define glClearStencil sdpglClearStencil
#define glClipPlane sdpglClipPlane
#define glColor3b sdpglColor3b
#define glColor3bv sdpglColor3bv
#define glColor3d sdpglColor3d
#define glColor3dv sdpglColor3dv
#define glColor3f sdpglColor3f
#define glColor3fv sdpglColor3fv
#define glColor3i sdpglColor3i
#define glColor3iv sdpglColor3iv
#define glColor3s sdpglColor3s
#define glColor3sv sdpglColor3sv
#define glColor3ub sdpglColor3ub
#define glColor3ubv sdpglColor3ubv
#define glColor3ui sdpglColor3ui
#define glColor3uiv sdpglColor3uiv
#define glColor3us sdpglColor3us
#define glColor3usv sdpglColor3usv
#define glColor4b sdpglColor4b
#define glColor4bv sdpglColor4bv
#define glColor4d sdpglColor4d
#define glColor4dv sdpglColor4dv
#define glColor4f sdpglColor4f
#define glColor4fv sdpglColor4fv
#define glColor4i sdpglColor4i
#define glColor4iv sdpglColor4iv
#define glColor4s sdpglColor4s
#define glColor4sv sdpglColor4sv
#define glColor4ub sdpglColor4ub
#define glColor4ubv sdpglColor4ubv
#define glColor4ui sdpglColor4ui
#define glColor4uiv sdpglColor4uiv
#define glColor4us sdpglColor4us
#define glColor4usv sdpglColor4usv
#define glColorMask sdpglColorMask
#define glColorMaterial sdpglColorMaterial
#define glColorPointer sdpglColorPointer
#define glCopyPixels sdpglCopyPixels
#define glCopyTexImage1D sdpglCopyTexImage1D
#define glCopyTexImage2D sdpglCopyTexImage2D
#define glCopyTexSubImage1D sdpglCopyTexSubImage1D
#define glCopyTexSubImage2D sdpglCopyTexSubImage2D
#define glCullFace sdpglCullFace
#define glDeleteLists sdpglDeleteLists
#define glDeleteTextures sdpglDeleteTextures
#define glDepthFunc sdpglDepthFunc
#define glDepthMask sdpglDepthMask
#define glDepthRange sdpglDepthRange
#define glDisable sdpglDisable
#define glDisableClientState sdpglDisableClientState
#define glDrawArrays sdpglDrawArrays
#define glDrawBuffer sdpglDrawBuffer
#define glDrawElements sdpglDrawElements
#define glDrawPixels sdpglDrawPixels
#define glEdgeFlag sdpglEdgeFlag
#define glEdgeFlagPointer sdpglEdgeFlagPointer
#define glEdgeFlagv sdpglEdgeFlagv
#define glEnable sdpglEnable
#define glEnableClientState sdpglEnableClientState
#define glEnd sdpglEnd
#define glEndList sdpglEndList
#define glEvalCoord1d sdpglEvalCoord1d
#define glEvalCoord1dv sdpglEvalCoord1dv
#define glEvalCoord1f sdpglEvalCoord1f
#define glEvalCoord1fv sdpglEvalCoord1fv
#define glEvalCoord2d sdpglEvalCoord2d
#define glEvalCoord2dv sdpglEvalCoord2dv
#define glEvalCoord2f sdpglEvalCoord2f
#define glEvalCoord2fv sdpglEvalCoord2fv
#define glEvalMesh1 sdpglEvalMesh1
#define glEvalMesh2 sdpglEvalMesh2
#define glEvalPoint1 sdpglEvalPoint1
#define glEvalPoint2 sdpglEvalPoint2
#define glFeedbackBuffer sdpglFeedbackBuffer
#define glFinish sdpglFinish
#define glFlush sdpglFlush
#define glFogf sdpglFogf
#define glFogfv sdpglFogfv
#define glFogi sdpglFogi
#define glFogiv sdpglFogiv
#define glFrontFace sdpglFrontFace
#define glFrustum sdpglFrustum
#define glGenLists sdpglGenLists
#define glGenTextures sdpglGenTextures
#define glGetBooleanv sdpglGetBooleanv
#define glGetClipPlane sdpglGetClipPlane
#define glGetDoublev sdpglGetDoublev
#define glGetError sdpglGetError
#define glGetFloatv sdpglGetFloatv
#define glGetIntegerv sdpglGetIntegerv
#define glGetLightfv sdpglGetLightfv
#define glGetLightiv sdpglGetLightiv
#define glGetMapdv sdpglGetMapdv
#define glGetMapfv sdpglGetMapfv
#define glGetMapiv sdpglGetMapiv
#define glGetMaterialfv sdpglGetMaterialfv
#define glGetMaterialiv sdpglGetMaterialiv
#define glGetPixelMapfv sdpglGetPixelMapfv
#define glGetPixelMapuiv sdpglGetPixelMapuiv
#define glGetPixelMapusv sdpglGetPixelMapusv
#define glGetPointerv sdpglGetPointerv
#define glGetPolygonStipple sdpglGetPolygonStipple
#define glGetString sdpglGetString
#define glGetTexEnvfv sdpglGetTexEnvfv
#define glGetTexEnviv sdpglGetTexEnviv
#define glGetTexGendv sdpglGetTexGendv
#define glGetTexGenfv sdpglGetTexGenfv
#define glGetTexGeniv sdpglGetTexGeniv
#define glGetTexImage sdpglGetTexImage
#define glGetTexLevelParameterfv sdpglGetTexLevelParameterfv
#define glGetTexLevelParameteriv sdpglGetTexLevelParameteriv
#define glGetTexParameterfv sdpglGetTexParameterfv
#define glGetTexParameteriv sdpglGetTexParameteriv
#define glHint sdpglHint
#define glIndexMask sdpglIndexMask
#define glIndexPointer sdpglIndexPointer
#define glIndexd sdpglIndexd
#define glIndexdv sdpglIndexdv
#define glIndexf sdpglIndexf
#define glIndexfv sdpglIndexfv
#define glIndexi sdpglIndexi
#define glIndexiv sdpglIndexiv
#define glIndexs sdpglIndexs
#define glIndexsv sdpglIndexsv
#define glIndexub sdpglIndexub
#define glIndexubv sdpglIndexubv
#define glInitNames sdpglInitNames
#define glInterleavedArrays sdpglInterleavedArrays
#define glIsEnabled sdpglIsEnabled
#define glIsList sdpglIsList
#define glIsTexture sdpglIsTexture
#define glLightModelf sdpglLightModelf
#define glLightModelfv sdpglLightModelfv
#define glLightModeli sdpglLightModeli
#define glLightModeliv sdpglLightModeliv
#define glLightf sdpglLightf
#define glLightfv sdpglLightfv
#define glLighti sdpglLighti
#define glLightiv sdpglLightiv
#define glLineStipple sdpglLineStipple
#define glLineWidth sdpglLineWidth
#define glListBase sdpglListBase
#define glLoadIdentity sdpglLoadIdentity
#define glLoadMatrixd sdpglLoadMatrixd
#define glLoadMatrixf sdpglLoadMatrixf
#define glLoadName sdpglLoadName
#define glLogicOp sdpglLogicOp
#define glMap1d sdpglMap1d
#define glMap1f sdpglMap1f
#define glMap2d sdpglMap2d
#define glMap2f sdpglMap2f
#define glMapGrid1d sdpglMapGrid1d
#define glMapGrid1f sdpglMapGrid1f
#define glMapGrid2d sdpglMapGrid2d
#define glMapGrid2f sdpglMapGrid2f
#define glMaterialf sdpglMaterialf
#define glMaterialfv sdpglMaterialfv
#define glMateriali sdpglMateriali
#define glMaterialiv sdpglMaterialiv
#define glMatrixMode sdpglMatrixMode
#define glMultMatrixd sdpglMultMatrixd
#define glMultMatrixf sdpglMultMatrixf
#define glNewList sdpglNewList
#define glNormal3b sdpglNormal3b
#define glNormal3bv sdpglNormal3bv
#define glNormal3d sdpglNormal3d
#define glNormal3dv sdpglNormal3dv
#define glNormal3f sdpglNormal3f
#define glNormal3fv sdpglNormal3fv
#define glNormal3i sdpglNormal3i
#define glNormal3iv sdpglNormal3iv
#define glNormal3s sdpglNormal3s
#define glNormal3sv sdpglNormal3sv
#define glNormalPointer sdpglNormalPointer
#define glOrtho sdpglOrtho
#define glPassThrough sdpglPassThrough
#define glPixelMapfv sdpglPixelMapfv
#define glPixelMapuiv sdpglPixelMapuiv
#define glPixelMapusv sdpglPixelMapusv
#define glPixelStoref sdpglPixelStoref
#define glPixelStorei sdpglPixelStorei
#define glPixelTransferf sdpglPixelTransferf
#define glPixelTransferi sdpglPixelTransferi
#define glPixelZoom sdpglPixelZoom
#define glPointSize sdpglPointSize
#define glPolygonMode sdpglPolygonMode
#define glPolygonOffset sdpglPolygonOffset
#define glPolygonStipple sdpglPolygonStipple
#define glPopAttrib sdpglPopAttrib
#define glPopClientAttrib sdpglPopClientAttrib
#define glPopMatrix sdpglPopMatrix
#define glPopName sdpglPopName
#define glPrioritizeTextures sdpglPrioritizeTextures
#define glPushAttrib sdpglPushAttrib
#define glPushClientAttrib sdpglPushClientAttrib
#define glPushMatrix sdpglPushMatrix
#define glPushName sdpglPushName
#define glRasterPos2d sdpglRasterPos2d
#define glRasterPos2dv sdpglRasterPos2dv
#define glRasterPos2f sdpglRasterPos2f
#define glRasterPos2fv sdpglRasterPos2fv
#define glRasterPos2i sdpglRasterPos2i
#define glRasterPos2iv sdpglRasterPos2iv
#define glRasterPos2s sdpglRasterPos2s
#define glRasterPos2sv sdpglRasterPos2sv
#define glRasterPos3d sdpglRasterPos3d
#define glRasterPos3dv sdpglRasterPos3dv
#define glRasterPos3f sdpglRasterPos3f
#define glRasterPos3fv sdpglRasterPos3fv
#define glRasterPos3i sdpglRasterPos3i
#define glRasterPos3iv sdpglRasterPos3iv
#define glRasterPos3s sdpglRasterPos3s
#define glRasterPos3sv sdpglRasterPos3sv
#define glRasterPos4d sdpglRasterPos4d
#define glRasterPos4dv sdpglRasterPos4dv
#define glRasterPos4f sdpglRasterPos4f
#define glRasterPos4fv sdpglRasterPos4fv
#define glRasterPos4i sdpglRasterPos4i
#define glRasterPos4iv sdpglRasterPos4iv
#define glRasterPos4s sdpglRasterPos4s
#define glRasterPos4sv sdpglRasterPos4sv
#define glReadBuffer sdpglReadBuffer
#define glReadPixels sdpglReadPixels
#define glRectd sdpglRectd
#define glRectdv sdpglRectdv
#define glRectf sdpglRectf
#define glRectfv sdpglRectfv
#define glRecti sdpglRecti
#define glRectiv sdpglRectiv
#define glRects sdpglRects
#define glRectsv sdpglRectsv
#define glRenderMode sdpglRenderMode
#define glRotated sdpglRotated
#define glRotatef sdpglRotatef
#define glScaled sdpglScaled
#define glScalef sdpglScalef
#define glScissor sdpglScissor
#define glSelectBuffer sdpglSelectBuffer
#define glShadeModel sdpglShadeModel
#define glStencilFunc sdpglStencilFunc
#define glStencilMask sdpglStencilMask
#define glStencilOp sdpglStencilOp
#define glTexCoord1d sdpglTexCoord1d
#define glTexCoord1dv sdpglTexCoord1dv
#define glTexCoord1f sdpglTexCoord1f
#define glTexCoord1fv sdpglTexCoord1fv
#define glTexCoord1i sdpglTexCoord1i
#define glTexCoord1iv sdpglTexCoord1iv
#define glTexCoord1s sdpglTexCoord1s
#define glTexCoord1sv sdpglTexCoord1sv
#define glTexCoord2d sdpglTexCoord2d
#define glTexCoord2dv sdpglTexCoord2dv
#define glTexCoord2f sdpglTexCoord2f
#define glTexCoord2fv sdpglTexCoord2fv
#define glTexCoord2i sdpglTexCoord2i
#define glTexCoord2iv sdpglTexCoord2iv
#define glTexCoord2s sdpglTexCoord2s
#define glTexCoord2sv sdpglTexCoord2sv
#define glTexCoord3d sdpglTexCoord3d
#define glTexCoord3dv sdpglTexCoord3dv
#define glTexCoord3f sdpglTexCoord3f
#define glTexCoord3fv sdpglTexCoord3fv
#define glTexCoord3i sdpglTexCoord3i
#define glTexCoord3iv sdpglTexCoord3iv
#define glTexCoord3s sdpglTexCoord3s
#define glTexCoord3sv sdpglTexCoord3sv
#define glTexCoord4d sdpglTexCoord4d
#define glTexCoord4dv sdpglTexCoord4dv
#define glTexCoord4f sdpglTexCoord4f
#define glTexCoord4fv sdpglTexCoord4fv
#define glTexCoord4i sdpglTexCoord4i
#define glTexCoord4iv sdpglTexCoord4iv
#define glTexCoord4s sdpglTexCoord4s
#define glTexCoord4sv sdpglTexCoord4sv
#define glTexCoordPointer sdpglTexCoordPointer
#define glTexEnvf sdpglTexEnvf
#define glTexEnvfv sdpglTexEnvfv
#define glTexEnvi sdpglTexEnvi
#define glTexEnviv sdpglTexEnviv
#define glTexGend sdpglTexGend
#define glTexGendv sdpglTexGendv
#define glTexGenf sdpglTexGenf
#define glTexGenfv sdpglTexGenfv
#define glTexGeni sdpglTexGeni
#define glTexGeniv sdpglTexGeniv
#define glTexImage1D sdpglTexImage1D
#define glTexImage2D sdpglTexImage2D
#define glTexParameterf sdpglTexParameterf
#define glTexParameterfv sdpglTexParameterfv
#define glTexParameteri sdpglTexParameteri
#define glTexParameteriv sdpglTexParameteriv
#define glTexSubImage1D sdpglTexSubImage1D
#define glTexSubImage2D sdpglTexSubImage2D
#define glTranslated sdpglTranslated
#define glTranslatef sdpglTranslatef
#define glVertex2d sdpglVertex2d
#define glVertex2dv sdpglVertex2dv
#define glVertex2f sdpglVertex2f
#define glVertex2fv sdpglVertex2fv
#define glVertex2i sdpglVertex2i
#define glVertex2iv sdpglVertex2iv
#define glVertex2s sdpglVertex2s
#define glVertex2sv sdpglVertex2sv
#define glVertex3d sdpglVertex3d
#define glVertex3dv sdpglVertex3dv
#define glVertex3f sdpglVertex3f
#define glVertex3fv sdpglVertex3fv
#define glVertex3i sdpglVertex3i
#define glVertex3iv sdpglVertex3iv
#define glVertex3s sdpglVertex3s
#define glVertex3sv sdpglVertex3sv
#define glVertex4d sdpglVertex4d
#define glVertex4dv sdpglVertex4dv
#define glVertex4f sdpglVertex4f
#define glVertex4fv sdpglVertex4fv
#define glVertex4i sdpglVertex4i
#define glVertex4iv sdpglVertex4iv
#define glVertex4s sdpglVertex4s
#define glVertex4sv sdpglVertex4sv
#define glVertexPointer sdpglVertexPointer
#define glViewport sdpglViewport

#else // SDPGL_TRACE

// Get rid of trace calls ...
#define sdpglSetStream(Stream)
#define sdpglPushTrace(State)
#define sdpglPopTrace()
#define sdpglTrace(Text)

#endif // !SDPGL_TRACE

#endif // SDPGL_H
