// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkaspectframe.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>
#include <sdpmath.h>

sdpGtkAspectFrame::sdpGtkAspectFrame() : sdpGtkFrame()
{
}

sdpGtkAspectFrame::sdpGtkAspectFrame(GtkAspectFrame* Frame) : sdpGtkFrame(GTK_FRAME(Frame))
{
}

bool sdpGtkAspectFrame::Create(SDPCSTRING Label, gdouble XAlignment, gdouble YAlignment, gdouble Ratio, bool IgnoreRatio)
{
	// We don't assert on our label because we allow empties ...

	m_Object = GTK_OBJECT(gtk_aspect_frame_new(Label, XAlignment, YAlignment, Ratio, IgnoreRatio));
	return Attached();
}

bool sdpGtkAspectFrame::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// X alignment ...
	double xalign = 0.5;
	sdpXmlGetAttribute(Element, "xalign", xalign);
	sdpGtkMarkAttribute(Element, "xalign");

	// Y alignment ...
	double yalign = 0.5;
	sdpXmlGetAttribute(Element, "yalign", yalign);
	sdpGtkMarkAttribute(Element, "yalign");

	// Ratio ...
	double ratio = 1.0;
	sdpXmlGetAttribute(Element, "ratio", ratio);
	sdpGtkMarkAttribute(Element, "ratio");

	// Ignore ratio ...
	bool ignoreratio = false;
	sdpXmlGetAttribute(Element, "ignoreratio", ignoreratio);
	sdpGtkMarkAttribute(Element, "ignoreratio");

	// Create the control ...
	if(strlen(Element->Text()))
		return Create(Element->Text(), xalign, yalign, ratio, ignoreratio);
	else
		return Create(0, xalign, yalign, ratio, ignoreratio);
}

void sdpGtkAspectFrame::Set(gdouble XAlignment, gdouble YAlignment, gdouble Ratio, bool IgnoreRatio)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_aspect_frame_set(GTK_ASPECT_FRAME(m_Object), XAlignment, YAlignment, Ratio, IgnoreRatio);
}

