
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



#include <sdpgtkbox.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>


////////////////////////////////////////////////////////////////////////////////////////
// sdpGtkBox

sdpGtkBox::sdpGtkBox() : sdpGtkContainer()
{
}

sdpGtkBox::sdpGtkBox(GtkBox* Box) : sdpGtkContainer(GTK_CONTAINER(Box))
{
}

bool sdpGtkBox::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	bool homogeneous = true;
	sdpXmlGetAttribute(Element, "homogeneous", homogeneous);
	sdpGtkMarkAttribute(Element, "homogeneous");
	SetHomogeneous(homogeneous);

	gint spacing = 0;
	sdpXmlGetAttribute(Element, "spacing", spacing);
	sdpGtkMarkAttribute(Element, "spacing");
	SetSpacing(spacing);

	return sdpGtkContainer::CreateOptions(ObjectContainer, Element);
}

void sdpGtkBox::PackStart(GtkWidget* Widget, bool Expand, bool Fill, guint Padding)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_box_pack_start(GTK_BOX(m_Object), Widget, Expand, Fill, Padding);
}

void sdpGtkBox::PackEnd(GtkWidget* Widget, bool Expand, bool Fill, guint Padding)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_box_pack_end(GTK_BOX(m_Object), Widget, Expand, Fill, Padding);
}

void sdpGtkBox::SetHomogeneous(bool Homogeneous)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_box_set_homogeneous(GTK_BOX(m_Object), Homogeneous);
}

void sdpGtkBox::SetSpacing(gint Spacing)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_box_set_spacing(GTK_BOX(m_Object), Spacing);
}

bool sdpGtkBox::AttachChild(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* ChildElement, sdpGtkObject* Child)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), false);
	g_assert(ChildElement);
	g_assert(Child);

	sdpString pack = "start";
	sdpXmlGetAttribute(ChildElement, "pack", pack);
	sdpGtkMarkAttribute(ChildElement, "pack");

	bool expand = false;
	sdpXmlGetAttribute(ChildElement, "expand", expand);
	sdpGtkMarkAttribute(ChildElement, "expand");

	bool fill = false;
	sdpXmlGetAttribute(ChildElement, "fill", fill);
	sdpGtkMarkAttribute(ChildElement, "fill");

	gint padding = 0;
	sdpXmlGetAttribute(ChildElement, "padding", padding);
	sdpGtkMarkAttribute(ChildElement, "padding");

	if(pack == "start")
		PackStart(GTK_WIDGET(Child->Object()), expand, fill, padding);
	else if(pack == "end")
		PackEnd(GTK_WIDGET(Child->Object()), expand, fill, padding);
	else
		g_assert_not_reached();

	return true;
}


