
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



#include <sdpgtkbutton.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>
#include <sdputility.h>

///////////////////////////////////////////////////////////////////////////
// sdpGtkButton

sdpGtkButton::sdpGtkButton() : sdpGtkBin()
{
}

sdpGtkButton::sdpGtkButton(GtkButton* Button) : sdpGtkBin(GTK_BIN(Button))
{
}

bool sdpGtkButton::Create()
{
	m_Object = GTK_OBJECT(gtk_button_new());
	return Attached();
}

bool sdpGtkButton::Create(SDPCSTRING Label)
{
	// Sanity checks ...
	g_assert(Label);

	m_Object = GTK_OBJECT(gtk_button_new_with_label(Label));
	return Attached();
}

bool sdpGtkButton::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	if(strlen(Element->Text()))
		return Create(Element->Text());

	return Create();
}


bool sdpGtkButton::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Relief ...
	sdpString relieftext = "normal";
	sdpXmlGetAttribute(Element, "relief", relieftext);
	sdpGtkMarkAttribute(Element, "relief");

	GtkReliefStyle relief = GTK_RELIEF_NORMAL;
	if(relieftext == "normal")
		relief = GTK_RELIEF_NORMAL;
	else if(relieftext == "half")
		relief = GTK_RELIEF_HALF;
	else if(relieftext == "none")
		relief = GTK_RELIEF_NONE;

	SetRelief(relief);

	return sdpGtkBin::CreateOptions(ObjectContainer, Element);
}

void sdpGtkButton::SetRelief(GtkReliefStyle Style)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_button_set_relief(GTK_BUTTON(m_Object), Style);
}

GtkReliefStyle sdpGtkButton::GetRelief()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), GtkReliefStyle(-1));

	return gtk_button_get_relief(GTK_BUTTON(m_Object));
}

static const gulong delay = 500;

void sdpGtkButton::FakeInteractiveActivate()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	GtkButton* button = GTK_BUTTON(m_Object);

	if(!GTK_WIDGET_IS_SENSITIVE(GTK_WIDGET(button)))
		return;

	SetState(GTK_STATE_ACTIVE);
	Draw(0);
	sdpGtkHandlePendingEvents();

	sdpSleep(delay);

	SetState(button->in_button ? GTK_STATE_PRELIGHT : GTK_STATE_NORMAL);
	Draw(0);
	sdpGtkHandlePendingEvents();

	sdpSleep(delay);

	sdpGtkHandlePendingEvents();
}

void sdpGtkButton::RealInteractiveActivate()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	GtkButton* button = GTK_BUTTON(m_Object);

	if(!GTK_WIDGET_IS_SENSITIVE(GTK_WIDGET(button)))
		return;

	gtk_button_pressed(button);
	sdpGtkHandlePendingEvents();
	
	sdpSleep(delay);

	gtk_button_clicked(button);
	gtk_button_released(button);
	sdpGtkHandlePendingEvents();
	
	sdpSleep(delay);
	
	sdpGtkHandlePendingEvents();
}


