
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



#include <sdpgtkbuttonbox.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>


////////////////////////////////////////////////////////////////////////////////////////
// sdpGtkButtonBox

sdpGtkButtonBox::sdpGtkButtonBox() : sdpGtkBox()
{
}

sdpGtkButtonBox::sdpGtkButtonBox(GtkButtonBox* ButtonBox) : sdpGtkBox(GTK_BOX(ButtonBox))
{
}

bool sdpGtkButtonBox::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Layout ...
	sdpString layouttext = "spread";
	sdpXmlGetAttribute(Element, "layout", layouttext);
	sdpGtkMarkAttribute(Element, "layout");
	
	GtkButtonBoxStyle layout = GTK_BUTTONBOX_SPREAD;
	if(layouttext == "spread")
		layout = GTK_BUTTONBOX_SPREAD;
	else if(layouttext == "edge")
		layout = GTK_BUTTONBOX_EDGE;
	else if(layouttext == "start")
		layout = GTK_BUTTONBOX_START;
	else if(layouttext == "end")
		layout = GTK_BUTTONBOX_END;
	else
		g_assert_not_reached();

	SetLayout(layout);

	return sdpGtkBox::CreateOptions(ObjectContainer, Element);
}

void sdpGtkButtonBox::GetChildSize(gint* MinWidth, gint* MinHeight)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_button_box_get_child_size(GTK_BUTTON_BOX(m_Object), MinWidth, MinHeight);
}

void sdpGtkButtonBox::SetChildSize(gint MinWidth, gint MinHeight)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_button_box_set_child_size(GTK_BUTTON_BOX(m_Object), MinWidth, MinHeight);
}

void sdpGtkButtonBox::GetChildPadding(gint* PadX, gint* PadY)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_button_box_get_child_ipadding(GTK_BUTTON_BOX(m_Object), PadX, PadY);
}

void sdpGtkButtonBox::SetChildPadding(gint PadX, gint PadY)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_button_box_set_child_ipadding(GTK_BUTTON_BOX(m_Object), PadX, PadY);
}

gint sdpGtkButtonBox::GetSpacing()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_button_box_get_spacing(GTK_BUTTON_BOX(m_Object));
}

void sdpGtkButtonBox::SetSpacing(gint Spacing)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_button_box_set_spacing(GTK_BUTTON_BOX(m_Object), Spacing);
}

GtkButtonBoxStyle sdpGtkButtonBox::GetLayout()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), GtkButtonBoxStyle(-1));

	return gtk_button_box_get_layout(GTK_BUTTON_BOX(m_Object));
}

void sdpGtkButtonBox::SetLayout(GtkButtonBoxStyle Style)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_button_box_set_layout(GTK_BUTTON_BOX(m_Object), Style);
}


