
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <sdpgtkclist.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>


sdpGtkCList::sdpGtkCList() : sdpGtkContainer()
{
}


sdpGtkCList::sdpGtkCList(GtkCList* CList) : sdpGtkContainer(GTK_CONTAINER(CList))
{
}


bool sdpGtkCList::Create(gint Columns)
{
	// Sanity checks ...
	g_assert(Columns);

	m_Object = GTK_OBJECT(gtk_clist_new(Columns));
	return Attached();
}


bool sdpGtkCList::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	gulong columns = 0;
	sdpXmlGetAttribute(Element, "columns", columns);
	sdpGtkMarkAttribute(Element, "columns");

	g_return_val_if_fail(columns, false);	// You MUST specify how many columns you want!

	return Create(columns);
}


bool sdpGtkCList::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Selection mode ...
	sdpString selectionmodetext = "single";
	sdpXmlGetAttribute(Element, "selectionmode", selectionmodetext);
	sdpGtkMarkAttribute(Element, "selectionmode");
	
	GtkSelectionMode selectionmode = GTK_SELECTION_SINGLE;
	if(selectionmodetext == "single")
		selectionmode = GTK_SELECTION_SINGLE;
	else if(selectionmodetext == "browse")
		selectionmode = GTK_SELECTION_BROWSE;
	else if(selectionmodetext == "multiple")
		selectionmode = GTK_SELECTION_MULTIPLE;
	else if(selectionmodetext == "extended")
		selectionmode = GTK_SELECTION_EXTENDED;
	else
		g_assert_not_reached();	// Unknown selection mode!

	SetSelectionMode(selectionmode);

	// Shadow type ...
	SetShadowType(sdpGtkGetShadowType(GTK_SHADOW_ETCHED_IN, Element));

	return sdpGtkContainer::CreateOptions(ObjectContainer, Element);
}


void sdpGtkCList::SetShadowType(GtkShadowType ShadowType)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_shadow_type(GTK_CLIST(m_Object), ShadowType);
}

void sdpGtkCList::SetSelectionMode(GtkSelectionMode SelectionMode)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_selection_mode(GTK_CLIST(m_Object), SelectionMode);
}

void sdpGtkCList::Freeze()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_freeze(GTK_CLIST(m_Object));
}

void sdpGtkCList::Thaw()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_thaw(GTK_CLIST(m_Object));
}

void sdpGtkCList::SelectAll()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_select_all(GTK_CLIST(m_Object));
}

void sdpGtkCList::DeselectAll()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_unselect_all(GTK_CLIST(m_Object));
}

void sdpGtkCList::UndoSelection()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_undo_selection(GTK_CLIST(m_Object));
}

void sdpGtkCList::Clear()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_clear(GTK_CLIST(m_Object));
}

void sdpGtkCList::MoveTo(gint Row, gint Column, gfloat RowAlign, gfloat ColumnAlign)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_moveto(GTK_CLIST(m_Object), Row, Column, RowAlign, ColumnAlign);
}

gint sdpGtkCList::GetHitInfo(gint X, gint Y, gint* Row, gint* Column)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_clist_get_selection_info(GTK_CLIST(m_Object), X, Y, Row, Column);
}

gint sdpGtkCList::ColumnCount()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return GTK_CLIST(m_Object)->columns;
}

void sdpGtkCList::ShowColumnTitles()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_column_titles_show(GTK_CLIST(m_Object));
}

void sdpGtkCList::HideColumnTitles()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_column_titles_hide(GTK_CLIST(m_Object));
}

void sdpGtkCList::ColumnTitlesActive()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_column_titles_active(GTK_CLIST(m_Object));
}

void sdpGtkCList::ColumnTitlesPassive()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_column_titles_passive(GTK_CLIST(m_Object));
}

void sdpGtkCList::ColumnTitleActive(gint Column)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_column_title_active(GTK_CLIST(m_Object), Column);
}

void sdpGtkCList::ColumnTitlePassive(gint Column)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_column_title_passive(GTK_CLIST(m_Object), Column);
}

void sdpGtkCList::SetColumnTitle(gint Column, SDPCSTRING Title)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_column_title(GTK_CLIST(m_Object), Column, Title);
}

void sdpGtkCList::SetColumnWidget(gint Column, GtkWidget* Widget)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_column_widget(GTK_CLIST(m_Object), Column, Widget);
}

SDPSTRING sdpGtkCList::GetColumnTitle(gint Column)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_clist_get_column_title(GTK_CLIST(m_Object), Column);
}

GtkWidget* sdpGtkCList::GetColumnWidget(gint Column)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_clist_get_column_widget(GTK_CLIST(m_Object), Column);
}

void sdpGtkCList::SetColumnJustification(gint Column, GtkJustification Justification)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_column_justification(GTK_CLIST(m_Object), Column, Justification);
}

void sdpGtkCList::SetColumnVisibility(gint Column, gboolean Visible)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_column_visibility(GTK_CLIST(m_Object), Column, Visible);
}

void sdpGtkCList::SetColumnResizeable(gint Column, gboolean Resizeable)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_column_resizeable(GTK_CLIST(m_Object), Column, Resizeable);
}

void sdpGtkCList::SetColumnAutoResize(gint Column, gboolean AutoResize)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_column_auto_resize(GTK_CLIST(m_Object), Column, AutoResize);
}

gint sdpGtkCList::GetOptimalColumnWidth(gint Column)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_clist_optimal_column_width(GTK_CLIST(m_Object), Column);
}

void sdpGtkCList::SetColumnWidth(gint Column, gint Width)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_column_width(GTK_CLIST(m_Object), Column, Width);
}

void sdpGtkCList::SetColumnMinWidth(gint Column, gint MinWidth)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_column_min_width(GTK_CLIST(m_Object), Column, MinWidth);
}

void sdpGtkCList::SetColumnMaxWidth(gint Column, gint MaxWidth)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_column_max_width(GTK_CLIST(m_Object), Column, MaxWidth);
}

gint sdpGtkCList::RowCount()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return GTK_CLIST(m_Object)->rows;
}

void sdpGtkCList::SetRowHeight(guint Height)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_row_height(GTK_CLIST(m_Object), Height);
}

GtkVisibility sdpGtkCList::GetRowVisible(gint Row)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), GtkVisibility(-1));

	return gtk_clist_row_is_visible(GTK_CLIST(m_Object), Row);
}

void sdpGtkCList::SetRowStyle(gint Row, GtkStyle* Style)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_row_style(GTK_CLIST(m_Object), Row, Style);
}

GtkStyle* sdpGtkCList::GetRowStyle(gint Row)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_clist_get_row_style(GTK_CLIST(m_Object), Row);
}

void sdpGtkCList::SetSelectable(gint Row, gboolean Selectable)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_selectable(GTK_CLIST(m_Object), Row, Selectable);
}

gboolean sdpGtkCList::GetSelectable(gint Row)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), false);

	return gtk_clist_get_selectable(GTK_CLIST(m_Object), Row) ? true : false;
}

void sdpGtkCList::SetForegroundColor(gint Row, GdkColor* Color)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_foreground(GTK_CLIST(m_Object), Row, Color);
}

void sdpGtkCList::SetBackgroundColor(gint Row, GdkColor* Color)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_background(GTK_CLIST(m_Object), Row, Color);
}

gint sdpGtkCList::Prepend(SDPCSTRING Text[])
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_clist_prepend(GTK_CLIST(m_Object), const_cast<gchar**>(Text));
}

gint sdpGtkCList::Append(SDPCSTRING Text[])
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_clist_append(GTK_CLIST(m_Object), const_cast<gchar**>(Text));
}

gint sdpGtkCList::Insert(gint Row, SDPCSTRING Text[])
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_clist_insert(GTK_CLIST(m_Object), Row, const_cast<gchar**>(Text));
}

void sdpGtkCList::Remove(gint Row)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_remove(GTK_CLIST(m_Object), Row);
}

void sdpGtkCList::SetRowData(gint Row, gpointer Data)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_row_data(GTK_CLIST(m_Object), Row, Data);
}

gpointer sdpGtkCList::GetRowData(gint Row)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_clist_get_row_data(GTK_CLIST(m_Object), Row);
}

gint sdpGtkCList::FindRowFromData(gpointer Data)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_clist_find_row_from_data(GTK_CLIST(m_Object), Data);
}

void sdpGtkCList::SelectRow(gint Row, gint Column)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_select_row(GTK_CLIST(m_Object), Row, Column);
}

void sdpGtkCList::DeselectRow(gint Row, gint Column)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_unselect_row(GTK_CLIST(m_Object), Row, Column);
}

void sdpGtkCList::SwapRows(gint Row1, gint Row2)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_swap_rows(GTK_CLIST(m_Object), Row1, Row2);
}

void sdpGtkCList::MoveRow(gint Source, gint Destination)
{
	// Not implemented ...
	g_assert_not_reached();

/*
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_row_move(GTK_CLIST(m_Object), Source, Destination);
*/
}

GtkCellType sdpGtkCList::GetCellType(gint Row, gint Column)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), GtkCellType(-1));

	return gtk_clist_get_cell_type(GTK_CLIST(m_Object), Row, Column);
}

void sdpGtkCList::SetCellStyle(gint Row, gint Column, GtkStyle* Style)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_cell_style(GTK_CLIST(m_Object), Row, Column, Style);
}

GtkStyle* sdpGtkCList::GetCellStyle(gint Row, gint Column)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_clist_get_cell_style(GTK_CLIST(m_Object), Row, Column);
}

void sdpGtkCList::SetShift(gint Row, gint Column, gint Vertical, gint Horizontal)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_shift(GTK_CLIST(m_Object), Row, Column, Vertical, Horizontal);
}

void sdpGtkCList::SetText(gint Row, gint Column, SDPCSTRING Text)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_text(GTK_CLIST(m_Object), Row, Column, Text);
}

void sdpGtkCList::SetPixmap(gint Row, gint Column, GdkPixmap* Pixmap, GdkBitmap* Mask)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_pixmap(GTK_CLIST(m_Object), Row, Column, Pixmap, Mask);
}

void sdpGtkCList::SetPixmapText(gint Row, gint Column, SDPCSTRING Text, guint8 Spacing, GdkPixmap* Pixmap, GdkBitmap* Mask)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_pixtext(GTK_CLIST(m_Object), Row, Column, Text, Spacing, Pixmap, Mask);
}

gint sdpGtkCList::GetText(gint Row, gint Column, SDPSTRING* Text)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_clist_get_text(GTK_CLIST(m_Object), Row, Column, Text);
}

gint sdpGtkCList::GetPixmap(gint Row, gint Column, GdkPixmap** Pixmap, GdkBitmap** Mask)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_clist_get_pixmap(GTK_CLIST(m_Object), Row, Column, Pixmap, Mask);
}

gint sdpGtkCList::GetPixmapText(gint Row, gint Column, SDPSTRING* Text, guint8* Spacing, GdkPixmap** Pixmap, GdkBitmap** Mask)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_clist_get_pixtext(GTK_CLIST(m_Object), Row, Column, Text, Spacing, Pixmap, Mask);
}

void sdpGtkCList::SetCompareFunction(GtkCListCompareFunc Function)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_compare_func(GTK_CLIST(m_Object), Function);
}

void sdpGtkCList::SetSortColumn(gint Column)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_sort_column(GTK_CLIST(m_Object), Column);
}

void sdpGtkCList::SetSortType(GtkSortType Type)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_sort_type(GTK_CLIST(m_Object), Type);
}

void sdpGtkCList::Sort()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_sort(GTK_CLIST(m_Object));
}

void sdpGtkCList::SetAutoSort(gboolean AutoSort)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_clist_set_auto_sort(GTK_CLIST(m_Object), AutoSort);
}


