// SDPGTK
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the sdpGtkColorSelector class, a standard color selection dialog
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpgtkcolorselector.h>
#include <sdpgtkevents.h>

/////////////////////////////////////////////////////////////////////////
// sdpGtkColorSelector

sdpGtkColorSelector::sdpGtkColorSelector(SDPCSTRING Title, sdpVector3 Color)
{
	m_OK = false;
	m_Title = Title;
	m_Color = Color;
}

void sdpGtkColorSelector::DoModal()
{
	// Load the user interface ...
	g_return_if_fail(Create());

	// Enter our modal loop ...
	sdpGtkObjectContainer::DoModal();
}

void sdpGtkColorSelector::DoModal(GtkWindow* TransientOwner)
{
	// Load the user interface ...
	g_return_if_fail(Create());

	// Enter our modal loop ...
	sdpGtkObjectContainer::DoModal(TransientOwner);
}

void sdpGtkColorSelector::DoModal(sdpGtkWindow& TransientOwner)
{
	// Load the user interface ...
	g_return_if_fail(Create());

	// Enter our modal loop ...
	sdpGtkObjectContainer::DoModal(TransientOwner);
}

// UI creation ...
bool sdpGtkColorSelector::Create()
{
	// Load the GTKML document ...
	sdpString dialogtemplate="<gtkml><colorselectiondialog/></gtkml>\n";
	g_return_val_if_fail(sdpGtkObjectContainer::Load(dialogtemplate, ""), false);

	// Set the title ...
	RootWindow().SetTitle(m_Title);

	// Set the initial color ...
	RootColorSelectionDialog().Selection().SetColor(m_Color);

	// Map some events ...
	MapEvent("delete-event", "ondelete", RootObject(), true);
	MapEvent("destroy", "ondestroy", RootObject(), true);
	MapEvent("clicked", "onok", RootColorSelectionDialog().OKButton(), true);
	MapEvent("clicked", "oncancel", RootColorSelectionDialog().CancelButton(), true);
	MapEvent("color-changed", "oncolorchanged", RootColorSelectionDialog().Selection(), true);

	return true;
}

bool sdpGtkColorSelector::OK()
{
	return m_OK;
}

bool sdpGtkColorSelector::Cancelled()
{
	return !m_OK;
}

SDPCSTRING sdpGtkColorSelector::Title()
{
	return m_Title;
}

sdpVector3 sdpGtkColorSelector::Color()
{
	return m_Color;
}

// Custom event handling ...
void sdpGtkColorSelector::OnEvent(sdpGtkEvent* Event)
{
	// Sanity checks ...
	g_assert(Event);

	if(Event->Name() == "oncolorchanged")
		OnColorChanged();
	else if(Event->Name() == "ondelete")
		OnDelete(Event);
	else if(Event->Name() == "ondestroy")
		OnDestroy();
	else if(Event->Name() == "onok")
		OnOK();
	else if(Event->Name() == "oncancel")
		OnCancel();
	else if(Event->Name() == "onhelp")
		OnHelp();
	else
		sdpGtkObjectContainer::OnEvent(Event);
}

// Called anytime the color changes ...
void sdpGtkColorSelector::OnColorChanged()
{
	// Store the new color ...
	RootColorSelectionDialog().Selection().GetColor(m_Color);
}

// Called when the user hits the WM "close" button ...
void sdpGtkColorSelector::OnDelete(sdpGtkEvent* Event)
{
	// Sanity checks ...
	g_assert(Event);

	// Don't allow the window to close ... treat it like a cancel, instead ...
	((sdpGtkEventWidgetDeleteEvent*)Event)->SetResult(true);
	
	OnCancel();
}

// Called when we're going away ...
void sdpGtkColorSelector::OnDestroy()
{
	// No more events from this point forward ...
	DisconnectAllEvents();

	// Let the modal loop end ...
	CancelModal();
}

// Called when the user hits the standard "OK" button ...
void sdpGtkColorSelector::OnOK()
{
	m_OK = true;
	RootWidget().Destroy();
}

// Called when the user hits the standard "Cancel" button ...
void sdpGtkColorSelector::OnCancel()
{
	m_OK = false;
	RootWidget().Destroy();
}

// Called when the user hits the standard "Help" button ...
void sdpGtkColorSelector::OnHelp()
{
}


