
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkdrawingarea.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

#include <iostream>

sdpGtkDrawingArea::sdpGtkDrawingArea() : sdpGtkWidget()
{
}

sdpGtkDrawingArea::sdpGtkDrawingArea(GtkDrawingArea* DrawingArea) : sdpGtkWidget(GTK_WIDGET(DrawingArea))
{
}

bool sdpGtkDrawingArea::Create()
{
	m_Object = GTK_OBJECT(gtk_drawing_area_new());
	return Attached();
}

bool sdpGtkDrawingArea::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Create the widget ...
	return Create();
}


bool sdpGtkDrawingArea::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Handle options ...
	sdpGtkMarkAttribute(Element, "width");
	sdpGtkMarkAttribute(Element, "height");

	sdpXmlIAttribute* attributewidth = sdpXmlFindAttribute(Element, "width");
	sdpXmlIAttribute* attributeheight = sdpXmlFindAttribute(Element, "height");

	// If we have neither we're done ...
	if(!attributewidth && !attributeheight)
		return sdpGtkWidget::CreateOptions(ObjectContainer, Element);

	// If we have both, set sizes ...
	if(attributewidth && attributeheight)
		{	
			gint width;
			gint height;
			sdpXmlGetAttribute(Element, "width", width);
			sdpXmlGetAttribute(Element, "height", height);

			SetSize(width, height);

			return sdpGtkWidget::CreateOptions(ObjectContainer, Element);
		}

	// We have one but not the other, so complain ...
	std::cout << "*** sdpGtkDrawingArea::Create() - you must specify both 'width' and 'height', or neither" << std::endl;

	return sdpGtkWidget::CreateOptions(ObjectContainer, Element);
}

void sdpGtkDrawingArea::SetSize(gint Width, gint Height)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_drawing_area_size(GTK_DRAWING_AREA(m_Object), Width, Height);
}

