
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



#include <sdpgtkeditable.h>

sdpGtkEditable::sdpGtkEditable() : sdpGtkWidget()
{
}

sdpGtkEditable::sdpGtkEditable(GtkEditable* Editable) : sdpGtkWidget(GTK_WIDGET(Editable))
{
}

void sdpGtkEditable::SetPosition(gint Position)
{
	// Sanity checks ...
	g_return_if_fail(m_Object);

	gtk_editable_set_position(GTK_EDITABLE(m_Object), Position);
}

gint sdpGtkEditable::GetPosition()
{
	// Sanity checks ...
	g_return_val_if_fail(m_Object, -1);

	return gtk_editable_get_position(GTK_EDITABLE(m_Object));
}

void sdpGtkEditable::SelectRegion(gint Start, gint End)
{
	// Sanity checks ...
	g_return_if_fail(m_Object);

	gtk_editable_select_region(GTK_EDITABLE(m_Object), Start, End);
}

void sdpGtkEditable::DeleteSelection()
{
	// Sanity checks ...
	g_return_if_fail(m_Object);

	gtk_editable_delete_selection(GTK_EDITABLE(m_Object));
}

void sdpGtkEditable::InsertText(SDPCSTRING Text, gint TextLength, gint* Position)
{
	// Sanity checks ...
	g_return_if_fail(m_Object);

	gtk_editable_insert_text(GTK_EDITABLE(m_Object), Text, TextLength, Position);
}

void sdpGtkEditable::DeleteText(gint Start, gint End)
{
	// Sanity checks ...
	g_return_if_fail(m_Object);

	gtk_editable_delete_text(GTK_EDITABLE(m_Object), Start, End);
}

gchar* sdpGtkEditable::GetText(gint Start, gint End)
{
	// Sanity checks ...
	g_return_val_if_fail(m_Object, 0);

	return gtk_editable_get_chars(GTK_EDITABLE(m_Object), Start, End);
}

void sdpGtkEditable::Cut()
{
	// Sanity checks ...
	g_return_if_fail(m_Object);

	gtk_editable_cut_clipboard(GTK_EDITABLE(m_Object));
}

void sdpGtkEditable::Copy()
{
	// Sanity checks ...
	g_return_if_fail(m_Object);

	gtk_editable_copy_clipboard(GTK_EDITABLE(m_Object));
}

void sdpGtkEditable::Paste()
{
	// Sanity checks ...
	g_return_if_fail(m_Object);

	gtk_editable_paste_clipboard(GTK_EDITABLE(m_Object));
}

void sdpGtkEditable::SetEditable(gboolean Editable)
{
	// Sanity checks ...
	g_return_if_fail(m_Object);

	gtk_editable_set_editable(GTK_EDITABLE(m_Object), Editable);
}



