
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkmenubar.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

//////////////////////////////////////////////////////////////
// sdpGtkMenuBar

sdpGtkMenuBar::sdpGtkMenuBar() : sdpGtkMenuShell()
{
}

sdpGtkMenuBar::sdpGtkMenuBar(GtkMenuBar* MenuBar) : sdpGtkMenuShell(GTK_MENU_SHELL(MenuBar))
{
}

bool sdpGtkMenuBar::Create()
{
	m_Object = GTK_OBJECT(gtk_menu_bar_new());
	return Attached();
}

bool sdpGtkMenuBar::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	return Create();
}


bool sdpGtkMenuBar::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Shadow type ...
	SetShadowType(sdpGtkGetShadowType(GTK_SHADOW_ETCHED_IN, Element));

	// Call the base-class implementation ...
	return sdpGtkMenuShell::CreateOptions(ObjectContainer, Element);
}

bool sdpGtkMenuBar::AttachChild(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* ChildElement, sdpGtkObject* Child)
{
	Append(GTK_WIDGET(Child->Object()));
	return true;
}

void sdpGtkMenuBar::Append(GtkWidget* Item)
{
	// Sanity checks ...
	g_assert(Item);
	g_return_if_fail(Attached());

	gtk_menu_bar_append(GTK_MENU_BAR(m_Object), Item);
}

void sdpGtkMenuBar::Prepend(GtkWidget* Item)
{
	// Sanity checks ...
	g_assert(Item);
	g_return_if_fail(Attached());

	gtk_menu_bar_prepend(GTK_MENU_BAR(m_Object), Item);
}

void sdpGtkMenuBar::Insert(GtkWidget* Item, gint Position)
{
	// Sanity checks ...
	g_assert(Item);
	g_return_if_fail(Attached());

	gtk_menu_bar_insert(GTK_MENU_BAR(m_Object), Item, Position);
}

void sdpGtkMenuBar::SetShadowType(GtkShadowType ShadowType)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_menu_bar_set_shadow_type(GTK_MENU_BAR(m_Object), ShadowType);
}
