
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkmenuitem.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>
#include <sdputility.h>

///////////////////////////////////////////////////////////////
// sdpGtkMenuItem

sdpGtkMenuItem::sdpGtkMenuItem() : sdpGtkItem()
{
}

sdpGtkMenuItem::sdpGtkMenuItem(GtkMenuItem* MenuItem) : sdpGtkItem(GTK_ITEM(MenuItem))
{
}

bool sdpGtkMenuItem::Create()
{
	m_Object = GTK_OBJECT(gtk_menu_item_new());
	return Attached();
}

bool sdpGtkMenuItem::Create(SDPCSTRING Label)
{
	// Sanity checks ...
	g_assert(Label);

	m_Object = GTK_OBJECT(gtk_menu_item_new_with_label(Label));
	return Attached();
}

bool sdpGtkMenuItem::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	bool separator = false;
	sdpXmlGetAttribute(Element, "separator", separator);
	sdpGtkMarkAttribute(Element, "separator");

	if(separator)
		return Create();
	else
		return Create(Element->Text());
}

bool sdpGtkMenuItem::AttachChild(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* ChildElement, sdpGtkObject* Child)
{
	// Sanity checks ...
	g_assert(ChildElement);
	g_assert(Child);
	g_return_val_if_fail(Attached(), false);

	SetSubmenu(GTK_WIDGET(Child->Object()));
	return true;
}

void sdpGtkMenuItem::SetSubmenu(GtkWidget* Submenu)
{
	// Sanity checks ...
	g_assert(Submenu);
	g_return_if_fail(Attached());

	gtk_menu_item_set_submenu(GTK_MENU_ITEM(m_Object), Submenu);
}

void sdpGtkMenuItem::RemoveSubmenu()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_menu_item_remove_submenu(GTK_MENU_ITEM(m_Object));
}

void sdpGtkMenuItem::SetSubmenuPlacement(GtkSubmenuPlacement Placement)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_menu_item_set_placement(GTK_MENU_ITEM(m_Object), Placement);
}

void sdpGtkMenuItem::Configure(bool ShowToggleIndicator, bool ShowSubmenuIndicator)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_menu_item_configure(GTK_MENU_ITEM(m_Object), ShowToggleIndicator, ShowSubmenuIndicator);
}

void sdpGtkMenuItem::Select()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_menu_item_select(GTK_MENU_ITEM(m_Object));
}

void sdpGtkMenuItem::Deselect()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_menu_item_deselect(GTK_MENU_ITEM(m_Object));
}

void sdpGtkMenuItem::Activate()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_menu_item_activate(GTK_MENU_ITEM(m_Object));
}

void sdpGtkMenuItem::RightJustify()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_menu_item_right_justify(GTK_MENU_ITEM(m_Object));
}

void sdpGtkMenuItem::InteractiveSelect()
{
	Select();
	sdpGtkHandlePendingEvents();
	sdpSleep(250);
	sdpGtkHandlePendingEvents();
}

void sdpGtkMenuItem::InteractiveActivate()
{
	Activate();
	sdpGtkHandlePendingEvents();
}


