
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



#include <string.h>
#include <sdpgtkoptionmenu.h>

///////////////////////////////////////////////////////////////////////////
// sdpGtkOptionMenu

sdpGtkOptionMenu::sdpGtkOptionMenu() : sdpGtkButton()
{
}

sdpGtkOptionMenu::sdpGtkOptionMenu(GtkOptionMenu* OptionMenu) : sdpGtkButton(GTK_BUTTON(OptionMenu))
{
}

bool sdpGtkOptionMenu::Create()
{
	m_Object = GTK_OBJECT(gtk_option_menu_new());
	return Attached();
}

bool sdpGtkOptionMenu::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	return Create();
}

bool sdpGtkOptionMenu::AttachChild(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* ChildElement, sdpGtkObject* Child)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), false);
	g_assert(ObjectContainer);
	g_assert(ChildElement);
	g_assert(Child);

	SetMenu(GTK_WIDGET(Child->Object()));

	return true;
}

GtkWidget* sdpGtkOptionMenu::GetMenu()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);
	
	return gtk_option_menu_get_menu(GTK_OPTION_MENU(m_Object));
}

void sdpGtkOptionMenu::SetMenu(GtkWidget* Menu)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	
	gtk_option_menu_set_menu(GTK_OPTION_MENU(m_Object), Menu);
}

void sdpGtkOptionMenu::RemoveMenu()
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	
	gtk_option_menu_remove_menu(GTK_OPTION_MENU(m_Object));
}

void sdpGtkOptionMenu::SetHistory(guint Index)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	
	gtk_option_menu_set_history(GTK_OPTION_MENU(m_Object), Index);
}


