
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkpaned.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

#include <iostream>

////////////////////////////////////////////////////////////////////////////////////////
// sdpGtkPaned

sdpGtkPaned::sdpGtkPaned() : sdpGtkContainer()
{
	m_ChildCount = 0;
}

sdpGtkPaned::sdpGtkPaned(GtkPaned* Paned) : sdpGtkContainer(GTK_CONTAINER(Paned))
{
	m_ChildCount = 0;
}

void sdpGtkPaned::SetHandleSize(guint16 HandleSize)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_paned_set_handle_size(GTK_PANED(m_Object), HandleSize);
}

void sdpGtkPaned::SetGutterSize(guint16 GutterSize)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_paned_set_gutter_size(GTK_PANED(m_Object), GutterSize);
}

void sdpGtkPaned::SetPosition(gint Position)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_paned_set_position(GTK_PANED(m_Object), Position);
}

void sdpGtkPaned::AddTopLeft(GtkWidget* Widget)
{
	// Sanity checks ...
	g_assert(Widget);
	g_return_if_fail(Attached());

	gtk_paned_add1(GTK_PANED(m_Object), Widget);
}

void sdpGtkPaned::AddBottomRight(GtkWidget* Widget)
{
	// Sanity checks ...
	g_assert(Widget);
	g_return_if_fail(Attached());

	gtk_paned_add2(GTK_PANED(m_Object), Widget);
}

void sdpGtkPaned::PackTopLeft(GtkWidget* Widget, bool Resize, bool Shrink)
{
	// Sanity checks ...
	g_assert(Widget);
	g_return_if_fail(Attached());

	gtk_paned_pack1(GTK_PANED(m_Object), Widget, Resize, Shrink);
}

void sdpGtkPaned::PackBottomRight(GtkWidget* Widget, bool Resize, bool Shrink)
{
	// Sanity checks ...
	g_assert(Widget);
	g_return_if_fail(Attached());

	gtk_paned_pack2(GTK_PANED(m_Object), Widget, Resize, Shrink);
}

bool sdpGtkPaned::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(Element);
	g_return_val_if_fail(Attached(), false);

	// Handle ...
	sdpGtkMarkAttribute(Element, "handle");
	if(sdpXmlFindAttribute(Element, "handle"))
		{
			gushort handle = 5;
			sdpXmlGetAttribute(Element, "handle", handle);
			SetHandleSize(handle);
		}

	// Gutter ...
	sdpGtkMarkAttribute(Element, "gutter");
	if(sdpXmlFindAttribute(Element, "gutter"))
		{
			gushort gutter = 5;
			sdpXmlGetAttribute(Element, "gutter", gutter);
			SetGutterSize(gutter);
		}

	// Call the base-class implementation ...
	return sdpGtkContainer::CreateOptions(ObjectContainer, Element);
}

bool sdpGtkPaned::AttachChild(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* ChildElement, sdpGtkObject* Child)
{
	// Sanity checks ...
	g_assert(ChildElement);
	g_assert(Child);

	// Should the child be resized with the paned?
	bool resize = true;
	sdpXmlGetAttribute(ChildElement, "resize", resize);
	sdpGtkMarkAttribute(ChildElement, "resize");
	
	// Should the child be shrunk below a minimum size?
	bool shrink = false;
	sdpXmlGetAttribute(ChildElement, "shrink", shrink);
	sdpGtkMarkAttribute(ChildElement, "shrink");

	// Figure out which pane to add this child to ...
	if(0 == m_ChildCount)
		PackTopLeft(GTK_WIDGET(Child->Object()), resize, shrink);
	else if(1 == m_ChildCount)
		PackBottomRight(GTK_WIDGET(Child->Object()), resize, shrink);
	else
		std::cout << "sdpGtk: tried to add too many children to <hpaned> or <vpaned> ... excess children will be ignored" << std::endl;

	m_ChildCount++;

	return true;
}


