
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkradiomenuitem.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

////////////////////////////////////////////////////////////////
// sdpGtkRadioMenuItem

sdpGtkRadioMenuItem::sdpGtkRadioMenuItem() : sdpGtkCheckMenuItem()
{
}

sdpGtkRadioMenuItem::sdpGtkRadioMenuItem(GtkRadioMenuItem* RadioMenuItem) : sdpGtkCheckMenuItem(GTK_CHECK_MENU_ITEM(RadioMenuItem))
{
}

bool sdpGtkRadioMenuItem::Create(GSList* Group)
{
	m_Object = GTK_OBJECT(gtk_radio_menu_item_new(Group));
	return Attached();
}

bool sdpGtkRadioMenuItem::Create(GSList* Group, SDPCSTRING Label)
{
	// Sanity checks ...
	g_assert(Label);

	m_Object = GTK_OBJECT(gtk_radio_menu_item_new_with_label(Group, Label));
	return Attached();
}

bool sdpGtkRadioMenuItem::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Get a group name (required) ...
	sdpString group;
	sdpXmlGetAttribute(Element, "group", group);
	sdpGtkMarkAttribute(Element, "group");
	g_return_val_if_fail(group.size(), false);
	
	// Create the object ...
	if(strlen(Element->Text()))
		g_return_val_if_fail(Create(ObjectContainer->GetGroup(group), Element->Text()), false);
	else
		g_return_val_if_fail(Create(ObjectContainer->GetGroup(group)), false);

	ObjectContainer->MapGroup(group, gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(m_Object)));

	return true;
}

