
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkscrolledwindow.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

#include <iostream>

sdpGtkScrolledWindow::sdpGtkScrolledWindow() : sdpGtkBin()
{
}

sdpGtkScrolledWindow::sdpGtkScrolledWindow(GtkScrolledWindow* ScrolledWindow) : sdpGtkBin(GTK_BIN(ScrolledWindow))
{
}

void sdpGtkScrolledWindow::AttachWithViewport(GtkWidget* Widget)
{
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(m_Object), Widget);
}

bool sdpGtkScrolledWindow::Create()
{
	m_Object = GTK_OBJECT(gtk_scrolled_window_new(0, 0));
	return Attached();
}

bool sdpGtkScrolledWindow::Create(GtkAdjustment* HorizontalAdjustment, GtkAdjustment* VerticalAdjustment)
{
	m_Object = GTK_OBJECT(gtk_scrolled_window_new(HorizontalAdjustment, VerticalAdjustment));
	return Attached();
}

bool sdpGtkScrolledWindow::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	return Create();
}


bool sdpGtkScrolledWindow::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	sdpString hscrollbar = "automatic";
	sdpXmlGetAttribute(Element, "hscrollbar", hscrollbar);
	sdpGtkMarkAttribute(Element, "hscrollbar");

	sdpString vscrollbar = "automatic";
	sdpXmlGetAttribute(Element, "vscrollbar", vscrollbar);
	sdpGtkMarkAttribute(Element, "vscrollbar");


	GtkPolicyType hscrollbarpolicy = GTK_POLICY_AUTOMATIC;
	if(hscrollbar == "automatic")
		hscrollbarpolicy = GTK_POLICY_AUTOMATIC;
	else if(hscrollbar == "always")
		hscrollbarpolicy = GTK_POLICY_ALWAYS;
	else
		g_assert_not_reached();

	GtkPolicyType vscrollbarpolicy = GTK_POLICY_AUTOMATIC;
	if(vscrollbar == "automatic")
		vscrollbarpolicy = GTK_POLICY_AUTOMATIC;
	else if(vscrollbar == "always")
		vscrollbarpolicy = GTK_POLICY_ALWAYS;
	else
		g_assert_not_reached();

	SetScrollbarPolicy(hscrollbarpolicy, vscrollbarpolicy);

	return sdpGtkBin::CreateOptions(ObjectContainer, Element);
}

bool sdpGtkScrolledWindow::AttachChild(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* ChildElement, sdpGtkObject* Child)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(ChildElement);
	g_assert(Child);

	// Is the child an adjustment?
	if(ChildElement->Name() == sdpString("adjustment"))
		{
			// Figure out which one ...
			sdpString target;
			sdpXmlGetAttribute(ChildElement, "target", target);
			sdpGtkMarkAttribute(ChildElement, "target");

			if(target == "horizontal")
				SetHorizontalAdjustment(GTK_ADJUSTMENT(Child->Object()));
			else if(target == "vertical")
				SetVerticalAdjustment(GTK_ADJUSTMENT(Child->Object()));
			else
				std::cout << "<scrolledwindow> child <adjustment> needs to specify a target: horizontal or vertical" << std::endl;

			return true;
		}
	// Is the child an object that handles its own scrollbars?
	else if(ChildElement->Name() == sdpString("clist") ||
					ChildElement->Name() == sdpString("ctree") ||
					ChildElement->Name() == sdpString("layout"))
		{
			Attach(GTK_WIDGET(Child->Object()));
			return true;
		}

	// Default behavior ...
	AttachWithViewport(GTK_WIDGET(Child->Object()));
	return true;
}

void sdpGtkScrolledWindow::SetScrollbarPolicy(GtkPolicyType HorizontalScrollbarPolicy, GtkPolicyType VerticalScrollbarPolicy)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m_Object), HorizontalScrollbarPolicy, VerticalScrollbarPolicy);
}

void sdpGtkScrolledWindow::SetPlacement(GtkCornerType Placement)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_scrolled_window_set_placement(GTK_SCROLLED_WINDOW(m_Object), Placement);
}

/// Sets the horizontal adjustment
void sdpGtkScrolledWindow::SetHorizontalAdjustment(GtkAdjustment* Adjustment)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_scrolled_window_set_hadjustment(GTK_SCROLLED_WINDOW(m_Object), Adjustment);
}

/// Sets the vertical adjustment
void sdpGtkScrolledWindow::SetVerticalAdjustment(GtkAdjustment* Adjustment)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_scrolled_window_set_vadjustment(GTK_SCROLLED_WINDOW(m_Object), Adjustment);
}

/// Returns the horizontal adjustment
sdpGtkAdjustment sdpGtkScrolledWindow::HorizontalAdjustment()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), sdpGtkAdjustment());

	return sdpGtkAdjustment(gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(m_Object)));
}

/// Returns the vertical adjustment
sdpGtkAdjustment sdpGtkScrolledWindow::VerticalAdjustment()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), sdpGtkAdjustment());

	return sdpGtkAdjustment(gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(m_Object)));
}

