
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtktext.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

sdpGtkText::sdpGtkText() : sdpGtkEditable()
{
}

sdpGtkText::sdpGtkText(GtkText* Text) : sdpGtkEditable(GTK_EDITABLE(Text))
{
}

bool sdpGtkText::Create()
{
	m_Object = GTK_OBJECT(gtk_text_new(0, 0));
	return Attached();
}

bool sdpGtkText::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	g_return_val_if_fail(Create(), false);
	
	if(strlen(Element->Text()))
		Insert(0, 0, 0, Element->Text(), -1);
		
	return true;
}


bool sdpGtkText::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	bool wordwrap = true;
	sdpXmlGetAttribute(Element, "wordwrap", wordwrap);
	sdpGtkMarkAttribute(Element, "wordwrap");
	SetWordWrap(wordwrap);
	
	bool linewrap = true;
	sdpXmlGetAttribute(Element, "linewrap", linewrap);
	sdpGtkMarkAttribute(Element, "linewrap");
	SetLineWrap(linewrap);

	bool editable = true;
	sdpXmlGetAttribute(Element, "editable", editable);
	sdpGtkMarkAttribute(Element, "editable");
	SetEditable(editable);

	return sdpGtkEditable::CreateOptions(ObjectContainer, Element);
}

void sdpGtkText::SetEditable(bool Editable)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_text_set_editable(GTK_TEXT(m_Object), Editable);
}

void sdpGtkText::SetWordWrap(bool WordWrap)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_text_set_word_wrap(GTK_TEXT(m_Object), WordWrap);
}

void sdpGtkText::SetLineWrap(bool LineWrap)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	
	gtk_text_set_line_wrap(GTK_TEXT(m_Object), LineWrap);
}

void sdpGtkText::SetPoint(guint Index)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_text_set_point(GTK_TEXT(m_Object), Index);
}

guint sdpGtkText::GetPoint()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_text_get_point(GTK_TEXT(m_Object));
}

guint sdpGtkText::GetLength()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_text_get_length(GTK_TEXT(m_Object));
}

void sdpGtkText::Freeze()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_text_freeze(GTK_TEXT(m_Object));
}

void sdpGtkText::Thaw()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_text_thaw(GTK_TEXT(m_Object));
}

void sdpGtkText::Insert(GdkFont* Font, GdkColor* ForegroundColor, GdkColor* BackgroundColor, const char* Text, gint Length)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_text_insert(GTK_TEXT(m_Object), Font, ForegroundColor, BackgroundColor, Text, Length);
}

gint sdpGtkText::BackwardDelete(guint Count)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_text_backward_delete(GTK_TEXT(m_Object), Count);
}

gint sdpGtkText::ForwardDelete(guint Count)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_text_forward_delete(GTK_TEXT(m_Object), Count);
}

