
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtktipsquery.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

///////////////////////////////////////////////////////////////////////
// sdpGtkTipsQuery

sdpGtkTipsQuery::sdpGtkTipsQuery() : sdpGtkLabel()
{
}

sdpGtkTipsQuery::sdpGtkTipsQuery(GtkTipsQuery* TipsQuery) : sdpGtkLabel(GTK_LABEL(TipsQuery))
{
}

bool sdpGtkTipsQuery::Create()
{
	// Sanity checks ...

	m_Object = GTK_OBJECT(gtk_tips_query_new());
	return Attached();
}

bool sdpGtkTipsQuery::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	return Create();
}

bool sdpGtkTipsQuery::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	sdpString inactive;
	sdpXmlGetAttribute(Element, "inactive", inactive);
	sdpGtkMarkAttribute(Element, "inactive");

	sdpString notip;
	sdpXmlGetAttribute(Element, "notip", notip);
	sdpGtkMarkAttribute(Element, "notip");
	
	SetLabels(inactive, notip);

	return sdpGtkLabel::CreateOptions(ObjectContainer, Element);
}

void sdpGtkTipsQuery::StartQuery()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_tips_query_start_query(GTK_TIPS_QUERY(m_Object));
}

void sdpGtkTipsQuery::StopQuery()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_tips_query_stop_query(GTK_TIPS_QUERY(m_Object));
}

void sdpGtkTipsQuery::SetCaller(GtkWidget* Widget)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Widget);
	
	gtk_tips_query_set_caller(GTK_TIPS_QUERY(m_Object), Widget);
}

void sdpGtkTipsQuery::SetLabels(SDPCSTRING Inactive, SDPCSTRING NoTip)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	
	gtk_tips_query_set_labels(GTK_TIPS_QUERY(m_Object), Inactive, NoTip);
}

