
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

extern "C"
{
#include <gtk/gtk.h>
}

#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

#include <iostream>

void sdpGtkMarkAttribute(sdpXmlIElement* Element, SDPCSTRING Attribute)
{
	// Sanity checks ...
	g_assert(Element);
	g_assert_string(Attribute);

	// Get the named attribute ...
	sdpXmlIAttribute* attribute = sdpXmlFindAttribute(Element, Attribute);
	if(attribute)
		{
			// Make sure someone else hasn't used it ...
			sdpString usedby = attribute->UserData()->GetData("usedby");
			if(usedby.size())
				{
					std::cout << "In element \"" << Element->Name() << "\" at line " << Element->Line() << ", attribute \"" << Attribute << "\" already used." << std::endl;
					return;
				}

			// Record the fact that we're using it ...
			attribute->UserData()->SetData("usedby", Element->Name());
		}
}

GtkShadowType sdpGtkGetShadowType(GtkShadowType Default, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(Element);

	GtkShadowType result = Default;
	if(!sdpXmlFindAttribute(Element, "shadowtype"))
		return result;
		
	sdpString resulttext;
	sdpXmlGetAttribute(Element, "shadowtype", resulttext);
	sdpGtkMarkAttribute(Element, "shadowtype");

	if(resulttext == "none")
		result = GTK_SHADOW_NONE;
	else if(resulttext == "in")
		result = GTK_SHADOW_IN;
	else if(resulttext == "out")
		result = GTK_SHADOW_OUT;
	else if(resulttext == "etched-in")
		result = GTK_SHADOW_ETCHED_IN;
	else if(resulttext == "etched-out")
		result = GTK_SHADOW_ETCHED_OUT;
	else
		std::cout << "In element \"" << Element->Name() << "\" at line " << Element->Line() << ", attribute \"shadowtype\" contains an unknown value." << std::endl;

	return result;
}

void sdpGtkHandlePendingEvents()
{
	while(gtk_events_pending())
		gtk_main_iteration();
}
