
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkviewport.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

#include <iostream>

sdpGtkViewport::sdpGtkViewport() : sdpGtkBin()
{
}

sdpGtkViewport::sdpGtkViewport(GtkViewport* Viewport) : sdpGtkBin(GTK_BIN(Viewport))
{
}

bool sdpGtkViewport::Create()
{
	return Create((GtkAdjustment*)0, (GtkAdjustment*)0);
}

bool sdpGtkViewport::Create(GtkAdjustment* HAdjustment, GtkAdjustment* VAdjustment)
{
	m_Object = GTK_OBJECT(gtk_viewport_new(HAdjustment, VAdjustment));
	return Attached();
}

bool sdpGtkViewport::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Create the control ...
	return Create((GtkAdjustment*)0, (GtkAdjustment*)0);
}

bool sdpGtkViewport::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Shadow type ...
	SetShadowType(sdpGtkGetShadowType(GTK_SHADOW_ETCHED_IN, Element));

	return sdpGtkBin::CreateOptions(ObjectContainer, Element);
}

bool sdpGtkViewport::AttachChild(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* ChildElement, sdpGtkObject* Child)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(ChildElement);
	g_assert(Child);

	// Is the child an adjustment?
	if(ChildElement->Name() == sdpString("adjustment"))
		{
			// Figure out which one ...
			sdpString target;
			sdpXmlGetAttribute(ChildElement, "target", target);
			sdpGtkMarkAttribute(ChildElement, "target");

			if(target == "horizontal")
				SetHorizontalAdjustment(GTK_ADJUSTMENT(Child->Object()));
			else if(target == "vertical")
				SetVerticalAdjustment(GTK_ADJUSTMENT(Child->Object()));
			else
				std::cout << "<viewport> child <adjustment> needs to specify a target: horizontal or vertical" << std::endl;

			return true;
		}

	// Default behavior ...
	Attach(GTK_WIDGET(Child->Object()));
	return true;
}

sdpGtkAdjustment sdpGtkViewport::HorizontalAdjustment()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), sdpGtkAdjustment());

	return sdpGtkAdjustment(gtk_viewport_get_hadjustment(GTK_VIEWPORT(m_Object)));
}

sdpGtkAdjustment sdpGtkViewport::VerticalAdjustment()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), sdpGtkAdjustment());

	return sdpGtkAdjustment(gtk_viewport_get_vadjustment(GTK_VIEWPORT(m_Object)));
}

void sdpGtkViewport::SetHorizontalAdjustment(GtkAdjustment* Adjustment)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_viewport_set_hadjustment(GTK_VIEWPORT(m_Object), Adjustment);
}

void sdpGtkViewport::SetVerticalAdjustment(GtkAdjustment* Adjustment)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_viewport_set_vadjustment(GTK_VIEWPORT(m_Object), Adjustment);
}

void sdpGtkViewport::SetShadowType(GtkShadowType ShadowType)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_viewport_set_shadow_type(GTK_VIEWPORT(m_Object), ShadowType);
}

