
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkwidget.h>
#include <sdpgtktooltips.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>
#include <sdpmath.h>
#include <sdputility.h>

#include <iostream>
#include <stdlib.h>

#if defined SDPWIN32
#include <gdk/win32/gdkwin32.h>
#else // SDPWIN32
#include <gdk/gdkx.h>
#include <X11/X.h>
#endif // !SDPWIN32

sdpGtkWidget::sdpGtkWidget() : sdpGtkObject()
{
}

sdpGtkWidget::sdpGtkWidget(GtkWidget* Widget) : sdpGtkObject(GTK_OBJECT(Widget))
{
}

sdpGtkWidget::~sdpGtkWidget()
{
}

void sdpGtkWidget::Reference()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_ref(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::UnReference()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_unref(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::Destroy()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_destroy(GTK_WIDGET(m_Object));
	m_Object = 0;
}

void sdpGtkWidget::Map()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_map(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::UnMap()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_unmap(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::Realize()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_realize(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::UnRealize()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_unrealize(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::SetState(GtkStateType State)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	
	gtk_widget_set_state(GTK_WIDGET(m_Object), State);
}

void sdpGtkWidget::UnParent()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_unparent(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::SetParent(GtkWidget* Parent)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Parent);

	gtk_widget_set_parent(GTK_WIDGET(m_Object), Parent);
}

void sdpGtkWidget::SetParentWindow(GdkWindow* Parent)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Parent);
	
	gtk_widget_set_parent_window(GTK_WIDGET(m_Object), Parent);
}

GdkWindow* sdpGtkWidget::GetParentWindow()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_widget_get_parent_window(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::Reparent(GtkWidget* NewParent)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(NewParent);

	gtk_widget_reparent(GTK_WIDGET(m_Object), NewParent);
}

GtkWidget* sdpGtkWidget::GetTopLevel()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_widget_get_toplevel(GTK_WIDGET(m_Object));
}

GtkWidget* sdpGtkWidget::GetAncestor(GtkType WidgetType)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_widget_get_ancestor(GTK_WIDGET(m_Object), WidgetType);
}

gint sdpGtkWidget::IsAncestor(GtkWidget* Child)
{
	// Sanity checks ...
	g_assert(Child);
	g_return_val_if_fail(Attached(), -1);

	return gtk_widget_is_ancestor(Child, GTK_WIDGET(m_Object));
}

void sdpGtkWidget::SetName(SDPCSTRING Name)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Name);

	gtk_widget_set_name(GTK_WIDGET(m_Object), Name);
}

gchar* sdpGtkWidget::GetName()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_widget_get_name(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::GetPath(guint* PathLength, gchar** Path, gchar** PathReversed)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(PathLength);

	gtk_widget_path(GTK_WIDGET(m_Object), PathLength, Path, PathReversed);
}

void sdpGtkWidget::GetClassPath(guint* PathLength, gchar** Path, gchar** PathReversed)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(PathLength);

	gtk_widget_class_path(GTK_WIDGET(m_Object), PathLength, Path, PathReversed);
}

void sdpGtkWidget::SetStyle(GtkStyle* Style)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Style);

	gtk_widget_set_style(GTK_WIDGET(m_Object), Style);
}

GtkStyle* sdpGtkWidget::GetStyle()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_widget_get_style(GTK_WIDGET(m_Object));
}

GdkColormap* sdpGtkWidget::GetColormap()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_widget_get_colormap(GTK_WIDGET(m_Object));
}

GdkVisual* sdpGtkWidget::GetVisual()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_widget_get_visual(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::EnsureStyle()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_ensure_style(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::RestoreDefaultStyle()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_restore_default_style(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::ModifyStyle(GtkRcStyle* Style)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Style);

	gtk_widget_modify_style(GTK_WIDGET(m_Object), Style);
}

void sdpGtkWidget::SetSensitive(bool Sensitive)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_set_sensitive(GTK_WIDGET(m_Object), Sensitive);
}

void sdpGtkWidget::CombineShapeMask(GdkBitmap* Mask, gint OffsetX, gint OffsetY)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Mask);

	gtk_widget_shape_combine_mask(GTK_WIDGET(m_Object), Mask, OffsetX, OffsetY);
}

void sdpGtkWidget::ResetShapeMask()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_reset_shapes(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::Show()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_show(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::ShowNow()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_show_now(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::ShowAll()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_show_all(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::Hide()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_hide(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::HideAll()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_hide_all(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::QueueDraw()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_queue_draw(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::QueueDrawArea(gint Left, gint Top, gint Width, gint Height)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_queue_draw_area(GTK_WIDGET(m_Object), Left, Top, Width, Height);
}

void sdpGtkWidget::QueueClear()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_queue_clear(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::QueueClearArea(gint Left, gint Top, gint Width, gint Height)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_queue_clear_area(GTK_WIDGET(m_Object), Left, Top, Width, Height);
}

void sdpGtkWidget::Draw(GdkRectangle* Area)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
//	g_assert(Area);

	gtk_widget_draw(GTK_WIDGET(m_Object), Area);
}

void sdpGtkWidget::DrawFocus()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_draw_focus(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::DrawDefault()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_draw_default(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::QueueResize()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_queue_resize(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::SizeRequest(GtkRequisition* Requisition)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Requisition);

	gtk_widget_size_request(GTK_WIDGET(m_Object), Requisition);
}

void sdpGtkWidget::SizeAllocate(GtkAllocation* Allocation)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Allocation);

	gtk_widget_size_allocate(GTK_WIDGET(m_Object), Allocation);
}

void sdpGtkWidget::GetChildRequisition(GtkRequisition* Requisition)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Requisition);

	gtk_widget_get_child_requisition(GTK_WIDGET(m_Object), Requisition);
}

void sdpGtkWidget::AddAccelerator(SDPCSTRING Signal, GtkAccelGroup* Group, guint Key, guint Modifiers, GtkAccelFlags Flags)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert_string(Signal);
	g_assert(Group);

	gtk_widget_add_accelerator(GTK_WIDGET(m_Object), Signal, Group, Key, Modifiers, Flags);
}

void sdpGtkWidget::RemoveAccelerator(GtkAccelGroup* Group, guint Key, guint Modifiers)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Group);

	gtk_widget_remove_accelerator(GTK_WIDGET(m_Object), Group, Key, Modifiers);
}

void sdpGtkWidget::RemoveAccelerators(SDPCSTRING Signal, bool VisibleOnly)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert_string(Signal);

	gtk_widget_remove_accelerators(GTK_WIDGET(m_Object), Signal, VisibleOnly);
}

void sdpGtkWidget::LockAccelerators()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_lock_accelerators(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::UnlockAccelerators()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_unlock_accelerators(GTK_WIDGET(m_Object));
}

bool sdpGtkWidget::AcceleratorsLocked()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), false);

	return gtk_widget_accelerators_locked(GTK_WIDGET(m_Object)) ? true : false;
}

void sdpGtkWidget::SetPosition(gint Left, gint Top)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_set_uposition(GTK_WIDGET(m_Object), Left, Top);
}

gint sdpGtkWidget::Width() const
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return GTK_WIDGET(m_Object)->allocation.width;
}

gint sdpGtkWidget::Height() const
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return GTK_WIDGET(m_Object)->allocation.height;
}

void sdpGtkWidget::SetSize(gint Width, gint Height)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_set_usize(GTK_WIDGET(m_Object), Width, Height);
}

void sdpGtkWidget::SetCharacterSize(gdouble Width, gdouble Height)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	const static sdpString metrictext("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_^");

	GtkWidget* widget = GTK_WIDGET(m_Object);

	gint lbearing = 0;
	gint rbearing = 0;
	gint width = 0;
	gint ascent = 0;
	gint descent = 0;
	
	gdk_string_extents(widget->style->font, metrictext, &lbearing, &rbearing, &width, &ascent, &descent);
	
	gdouble averagewidth = gdouble(lbearing + width + rbearing) / gdouble(metrictext.size());
	gdouble averageheight = gdouble(ascent + descent);
	
	gdouble targetwidth = Width * averagewidth;
	gdouble targetheight = Height * averageheight;
	
	gtk_widget_set_usize(widget, gint(targetwidth) + widget->style->klass->xthickness, gint(targetheight) + widget->style->klass->ythickness);
}

void sdpGtkWidget::Activate()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_activate(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::GrabAdd()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_grab_add(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::GrabRemove()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_grab_remove(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::GrabFocus()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_grab_focus(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::GrabDefault()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_grab_default(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::SetDragSource(GdkModifierType StartButton, GtkTargetEntry* TargetArray, gint TargetCount, GdkDragAction DragAction)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(TargetArray);
	g_assert(TargetCount);

	gtk_drag_source_set(GTK_WIDGET(m_Object), StartButton, TargetArray, TargetCount, DragAction);
}

void sdpGtkWidget::RemoveDragSource()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

g_assert_not_reached();	// Not implemented, yet!
//	gtk_drag_source_unset(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::SetDragDestination(GtkDestDefaults Flags, GtkTargetEntry* TargetArray, gint TargetCount, GdkDragAction DragAction)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(TargetArray);
	g_assert(TargetCount);

	gtk_drag_dest_set(GTK_WIDGET(m_Object), Flags, TargetArray, TargetCount, DragAction);
}

void sdpGtkWidget::RemoveDragDestination()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_drag_dest_unset(GTK_WIDGET(m_Object));
}

void sdpGtkWidget::GetPointer(gint* X, gint* Y)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_get_pointer(GTK_WIDGET(m_Object), X, Y);
}

void sdpGtkWidget::Popup(gint X, gint Y)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_widget_popup(GTK_WIDGET(m_Object), X, Y);
}

gint sdpGtkWidget::Intersect(GdkRectangle* Area, GdkRectangle* Intersection)
{
	// Sanity checks ...
	g_assert(Area);
	g_assert(Intersection);
	g_return_val_if_fail(Attached(), -1);

	return gtk_widget_intersect(GTK_WIDGET(m_Object), Area, Intersection);
}

bool sdpGtkWidget::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Size (in pixels) ...
	sdpGtkMarkAttribute(Element, "size");
	if(sdpXmlFindAttribute(Element, "size"))
		{
			sdpVector2 size;
			sdpXmlGetAttribute(Element, "size", size);
			SetSize(gint(size[0]), gint(size[1]));
		}

	// Size (in characters) ...
	sdpGtkMarkAttribute(Element, "charactersize");
	if(sdpXmlFindAttribute(Element, "charactersize"))
		{
			sdpVector2 charactersize;
			sdpXmlGetAttribute(Element, "charactersize", charactersize);
			SetCharacterSize(charactersize[0], charactersize[1]);
		}

	// Drag-and-drop ...
	sdpString dragsource;
	sdpXmlGetAttribute(Element, "dragsource", dragsource);
	sdpGtkMarkAttribute(Element, "dragsource");
	if(dragsource.size())
		{
			gchar* temp = const_cast<gchar*>(SDPCSTRING(dragsource));
			GtkTargetEntry targets[] = {{temp, 0, 1}};
			SetDragSource(GDK_MODIFIER_MASK, targets, 1, GDK_ACTION_DEFAULT);
		}

	sdpString dragdestination;
	sdpXmlGetAttribute(Element, "dragdestination", dragdestination);
	sdpGtkMarkAttribute(Element, "dragdestination");
	if(dragdestination.size())
		{
			gchar* temp = const_cast<gchar*>(SDPCSTRING(dragdestination));
			GtkTargetEntry targets[] = {{temp, 0, 1}};
			SetDragDestination(GTK_DEST_DEFAULT_ALL, targets, 1, GDK_ACTION_DEFAULT);
		}

	// Built-in tooltip support ...
	CreateTooltip(ObjectContainer, Element);

	// Visibility ...
	bool show = true;
	sdpXmlGetAttribute(Element, "show", show);
	sdpGtkMarkAttribute(Element, "show");
	show ? Show() : Hide();

	return sdpGtkObject::CreateOptions(ObjectContainer, Element);
}

// We have an unfortunate clash with windoze.h!
#ifdef SDPWIN32
#ifdef GetObject
#undef GetObject
#endif // GetObject
#endif // SDPWIN32

bool sdpGtkWidget::CreateTooltip(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// If a tooltip isn't specified, we're done ...
	sdpXmlIAttribute* attribute = sdpXmlFindAttribute(Element, "tooltip");
	if(!attribute)
		return true;

	// Make sure the tooltip hasn't already been implemented (by a toolbar, for example) ...
	sdpString usedby = attribute->UserData()->GetData("usedby");
	if(usedby.size())
		return true;
	
	// Get the tooltip text ...
	sdpGtkMarkAttribute(Element, "tooltip");
	sdpString tooltiptext = attribute->Value();
	if(!tooltiptext.size())
		return true;

	// Get the built-in tooltips object ...
	GtkObject* tooltipsobject = ObjectContainer->GetObject("sdpgtk_tooltips");
	if(!tooltipsobject)
		{
			// built-in tooltips object doesn't exist, yet, so create it ...
			sdpGtkTooltips tooltips;
			g_assert(tooltips.Create());
			tooltips.Enable();

			ObjectContainer->MapObject("sdpgtk_tooltips", tooltips.Object());

			tooltipsobject = ObjectContainer->GetObject("sdpgtk_tooltips");
		}

	// Make sure we got the built-in tooltips object ...
	g_assert(tooltipsobject);

	// Add our tooltip ...
	sdpGtkTooltips(GTK_TOOLTIPS(tooltipsobject)).Set(*this, tooltiptext);

	return true;
}

void sdpGtkWidget::InteractiveWarpPointer()
{
	InteractiveWarpPointer(0.5, 0.5);
}

void sdpGtkWidget::InteractiveWarpPointer(gdouble XPercent, gdouble YPercent)
{
	// Get our current window dimensions ...
	GtkWidget* widget = GTK_WIDGET(m_Object);
	
	InteractiveWarpPointer(gint(XPercent * widget->allocation.width), gint(YPercent * widget->allocation.height));
}

void sdpGtkWidget::InteractiveWarpPointer(gint XOffset, gint YOffset)
{
#ifdef SDPWIN32

std::cerr << "sdpGtkWidget::InteractiveWarpPointer() not implemented on this platform!!!" << std::endl;

#else // SDPWIN32

	// Get the X display ...
	Display* display = (Display*)GDK_DISPLAY();
	g_return_if_fail(display);

	// Get our GDK window ...
	GdkWindow* window = GTK_WIDGET(m_Object)->window;
	g_return_if_fail(window);
	
	// Get our X window ...
	Window xwindow = GDK_WINDOW_XWINDOW(window);
	g_return_if_fail(xwindow);

	// Get the current mouse pointer position, relative to our window ...
	gint pointerx = 0;
	gint pointery = 0;
	GdkModifierType modifiers;
	gdk_window_get_pointer(GTK_WIDGET(m_Object)->window, &pointerx, &pointery, &modifiers);

	// Make it our source ...
	sdpVector2 source(pointerx, pointery);

	// Setup our destination ...
	sdpVector2 destination(XOffset, YOffset);

	// Setup some sloppiness ...
	static const gdouble sloppiness = 75;
	static const gdouble randomscale = RAND_MAX * 0.5;
	sdpVector2 slop = sdpVector2((rand()-randomscale) / randomscale, (rand()-randomscale) / randomscale) * sloppiness;
			
	static const gulong steps = 50;
	static const gdouble delta = 1.0 / gdouble(steps);
	static const gulong delay = 10;
	
	std::vector<gdouble> controlpoints;
	controlpoints.push_back(0);
	controlpoints.push_back(0.1);
	controlpoints.push_back(0.9);
	controlpoints.push_back(1);
									
	for(gulong i = 1; i <= steps; i++)
		{
			// Calculate our position as a percentage (with acceleration / deceleration) ...
			gdouble position = delta * i;
			position = sdpBezier(controlpoints, position);
		
			sdpVector2 actualdestination = sdpLerp(destination+slop, destination, position);
			sdpVector2 finalposition = sdpLerp(source, actualdestination, position);
			
			XWarpPointer(display, None, xwindow, 0, 0, 0, 0, int(finalposition[0]), int(finalposition[1]));
			XFlush(display);
			sdpSleep(delay);

			sdpGtkHandlePendingEvents();
		}
		
	sdpSleep(500);
	sdpGtkHandlePendingEvents();
	
#endif // !SDPWIN32
}


