#ifndef SDPREGEX_H
#define SDPREGEX_H

// SDPTypes Library
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/** \file
		\brief Declares the sdpRegEx wrapper class for regular expressions
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpstring.h>

#include <regex.h>

#include <vector>
#include <utility>

/// Used to store a regular expression match as the range (first, second]
typedef std::pair<gulong, gulong> sdpRegExMatch;

/// Stores a set of matches, from a regular expression containing subexpressions
typedef std::vector<sdpRegExMatch> sdpRegExMatchArray;
typedef sdpRegExMatchArray::iterator sdpRegExMatchIterator;

/// Wrapper class for GNU regular expression handling
class sdpRegEx
{
public:
	sdpRegEx();
	sdpRegEx(SDPCSTRING Expression);
	sdpRegEx(SDPCSTRING Expression, const bool CaseSensitive);
	
	~sdpRegEx();
	
	/// Called to set (compile) the given regular expression
	bool Create(SDPCSTRING Expression);
	/// Called to set (compile) the given regular expression, with optional case-insensitive comparisons
	bool Create(SDPCSTRING Expression, const bool CaseSensitive);
	
	/// Returns the start index (-1 if none) of the compiled regular expression within the given string
	gint Search(SDPCSTRING String) const;
	/// Returns the start index (-1 if none) and subexpression ranges of the compiled regular expression within the given string
	gint Search(SDPCSTRING String, sdpRegExMatchArray& Subexpressions) const;
	/// Returns the start index (-1 if none) of the compiled regular expression within the range (Start, Start+Range) of the given string.  Negative values of Range are allowed.
	gint Search(SDPCSTRING String, const gint Start, const gint Range) const;
	/// Returns the start index (-1 if none) and subexpression ranges of the compiled regular expression within the range (Start, Start+Range) of the given string.  Negative values of Range are allowed.
	gint Search(SDPCSTRING String, const gint Start, const gint Range, sdpRegExMatchArray& Subexpressions) const;

protected:
	re_pattern_buffer m_State;
	char* m_CaseTable;
};

#endif SDPREGEX_H
