#ifndef SDPXMLIATTRIBUTE_H
#define SDPXMLIATTRIBUTE_H

// SDPXML Library
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/** \file
		\brief Declares the sdpXmlIAttribute interface
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpstring.h>
#include <sdpiuserdata.h>

#include <vector>
#include <list>
#include <set>

// Forward class declarations
class sdpXmlIElement;

/// Opaque handle for working with XML attributes
SDP_DECLARE_HANDLE(SDPXMLATTRIBUTEHANDLE);

/// sdpCOM interface for an XML attribute
class sdpXmlIAttribute
{
public:
	/// Returns the owning element
	virtual sdpXmlIElement* ParentElement() const = 0;
	/// Returns the attribute name
	virtual SDPCSTRING Name() const = 0;
	/// Sets the attribute name
	virtual sdpResult SetName(SDPCSTRING Name) = 0;
	/// Returns the attribute value
	virtual SDPCSTRING Value() const = 0;
	/// Sets the attribute value
	virtual sdpResult SetValue(SDPCSTRING Value) = 0;
	/// Returns the XML file line number of the attribute
	virtual gulong Line() const = 0;
	/// Returns the XML file column of the attribute
	virtual gulong Column() const = 0;
	/// Returns an sdpIUserData interface to the underlying element
	virtual sdpIUserData* UserData() = 0;
};

/// Contains an array of attributes
typedef std::vector<sdpXmlIAttribute*> sdpXmlAttributeArray;
/// Contains a linked-list of attributes
typedef std::list<sdpXmlIAttribute*> sdpXmlAttributeList;
/// Contains a set of attributes
typedef std::set<sdpXmlIAttribute*> sdpXmlAttributeSet;

#endif SDPXMLIATTRIBUTE_H
