#ifndef SDPXMLUTILITY_H
#define SDPXMLUTILITY_H

// SDPXML Library
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/** \file
		\brief Declares utility functions for use with XML documents
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpxmliattribute.h>
#include <sdpxmlielement.h>
#include <sdpxmlidocument.h>

#include <sdpstring.h>

////////////////////////////////////////////////////////////////////////////////
// Handy-dandy helper functions ...

/// Returns the set of elements that match the given XPath criteria
sdpXmlElementArray sdpXmlXPathSelect(sdpXmlIDocument* Document, SDPCSTRING Selection);
/// Returns the set of elements that match the given XPath criteria
sdpXmlElementArray sdpXmlXPathSelect(sdpXmlIElement* Element, SDPCSTRING Selection);

/// Returns the first element that matches the given XPath criteria; warns if there are more than one
sdpXmlIElement* sdpXmlGetXPathElement(sdpXmlIElement* Parent, SDPCSTRING Path);
/// Returns the first element that matches the given XPath criteria; creates one of there is none (last argument MUST be a NULL)
sdpXmlIElement* sdpXmlGetSafeXPathElement(sdpXmlIElement* Parent, SDPCSTRING Path, SDPCSTRING ElementName, SDPCSTRING ElementText, ...);
/// Creates an element and sets all its attributes in one step, using varargs notation (last argument MUST be a NULL)
sdpXmlIElement* sdpXmlCreateElementV(sdpXmlIDocument* Document, SDPCSTRING ElementName, SDPCSTRING ElementText, ...);
/// Creates an element and sets all its attributes in one step, using varargs notation (last argument MUST be a NULL)
sdpXmlIElement* sdpXmlCreateElementV(sdpXmlIElement* Parent, SDPCSTRING ElementName, SDPCSTRING ElementText, ...);
/// Creates an empty element with no attributes
sdpXmlIElement* sdpXmlCreateElement(sdpXmlIElement* Parent, SDPCSTRING ElementName);
/// Creates an element with no attributes
sdpXmlIElement* sdpXmlCreateElement(sdpXmlIElement* Parent, SDPCSTRING ElementName, SDPCSTRING ElementText);
/// Creates an element and sets all its attributes in one step, using a type-safe template
template<class TYPE1>
sdpXmlIElement* sdpXmlCreateElement(sdpXmlIElement* Parent,
																		SDPCSTRING ElementName,
																		SDPCSTRING ElementText,
																		SDPCSTRING AttributeName1,
																		const TYPE1& AttributeValue1)
{
	return sdpXmlCreateElementV(Parent,
														ElementName,
														ElementText,
														AttributeName1,
														SDPCSTRING(sdpString(AttributeValue1)),
														0);
}

/// Creates an element and sets all its attributes in one step, using a type-safe template
template<class TYPE1, class TYPE2>
sdpXmlIElement* sdpXmlCreateElement(sdpXmlIElement* Parent,
																		SDPCSTRING ElementName,
																		SDPCSTRING ElementText,
																		SDPCSTRING AttributeName1,
																		const TYPE1& AttributeValue1,
																		SDPCSTRING AttributeName2,
																		const TYPE2& AttributeValue2)
{
	return sdpXmlCreateElementV(Parent,
														ElementName,
														ElementText,
														AttributeName1,
														SDPCSTRING(sdpString(AttributeValue1)),
														AttributeName2,
														SDPCSTRING(sdpString(AttributeValue2)),
														0);
}

/// Creates an element and sets all its attributes in one step, using a type-safe template
template<class TYPE1, class TYPE2, class TYPE3>
sdpXmlIElement* sdpXmlCreateElement(sdpXmlIElement* Parent,
																		SDPCSTRING ElementName,
																		SDPCSTRING ElementText,
																		SDPCSTRING AttributeName1,
																		const TYPE1& AttributeValue1,
																		SDPCSTRING AttributeName2,
																		const TYPE2& AttributeValue2,
																		SDPCSTRING AttributeName3,
																		const TYPE3& AttributeValue3)
{
	return sdpXmlCreateElementV(Parent,
														ElementName,
														ElementText,
														AttributeName1,
														SDPCSTRING(sdpString(AttributeValue1)),
														AttributeName2,
														SDPCSTRING(sdpString(AttributeValue2)),
														AttributeName3,
														SDPCSTRING(sdpString(AttributeValue3)),
														0);
}

/// Creates an element and sets all its attributes in one step, using a type-safe template
template<class TYPE1, class TYPE2, class TYPE3, class TYPE4>
sdpXmlIElement* sdpXmlCreateElement(sdpXmlIElement* Parent,
																		SDPCSTRING ElementName,
																		SDPCSTRING ElementText,
																		SDPCSTRING AttributeName1,
																		const TYPE1& AttributeValue1,
																		SDPCSTRING AttributeName2,
																		const TYPE2& AttributeValue2,
																		SDPCSTRING AttributeName3,
																		const TYPE3& AttributeValue3,
																		SDPCSTRING AttributeName4,
																		const TYPE4& AttributeValue4)
{
	return sdpXmlCreateElementV(Parent,
														ElementName,
														ElementText,
														AttributeName1,
														SDPCSTRING(sdpString(AttributeValue1)),
														AttributeName2,
														SDPCSTRING(sdpString(AttributeValue2)),
														AttributeName3,
														SDPCSTRING(sdpString(AttributeValue3)),
														AttributeName4,
														SDPCSTRING(sdpString(AttributeValue4)),
														0);
}

/// Creates an element and sets all its attributes in one step, using a type-safe template
template<class TYPE1, class TYPE2, class TYPE3, class TYPE4, class TYPE5>
sdpXmlIElement* sdpXmlCreateElement(sdpXmlIElement* Parent,
																		SDPCSTRING ElementName,
																		SDPCSTRING ElementText,
																		SDPCSTRING AttributeName1,
																		const TYPE1& AttributeValue1,
																		SDPCSTRING AttributeName2,
																		const TYPE2& AttributeValue2,
																		SDPCSTRING AttributeName3,
																		const TYPE3& AttributeValue3,
																		SDPCSTRING AttributeName4,
																		const TYPE4& AttributeValue4,
																		SDPCSTRING AttributeName5,
																		const TYPE5& AttributeValue5)
{
	return sdpXmlCreateElementV(Parent,
														ElementName,
														ElementText,
														AttributeName1,
														SDPCSTRING(sdpString(AttributeValue1)),
														AttributeName2,
														SDPCSTRING(sdpString(AttributeValue2)),
														AttributeName3,
														SDPCSTRING(sdpString(AttributeValue3)),
														AttributeName4,
														SDPCSTRING(sdpString(AttributeValue4)),
														AttributeName5,
														SDPCSTRING(sdpString(AttributeValue5)),
														0);
}

/// Creates an element and sets all its attributes in one step, using a type-safe template
template<class TYPE1, class TYPE2, class TYPE3, class TYPE4, class TYPE5, class TYPE6>
sdpXmlIElement* sdpXmlCreateElement(sdpXmlIElement* Parent,
																		SDPCSTRING ElementName,
																		SDPCSTRING ElementText,
																		SDPCSTRING AttributeName1,
																		const TYPE1& AttributeValue1,
																		SDPCSTRING AttributeName2,
																		const TYPE2& AttributeValue2,
																		SDPCSTRING AttributeName3,
																		const TYPE3& AttributeValue3,
																		SDPCSTRING AttributeName4,
																		const TYPE4& AttributeValue4,
																		SDPCSTRING AttributeName5,
																		const TYPE5& AttributeValue5,
																		SDPCSTRING AttributeName6,
																		const TYPE6& AttributeValue6)
{
	return sdpXmlCreateElementV(Parent,
														ElementName,
														ElementText,
														AttributeName1,
														SDPCSTRING(sdpString(AttributeValue1)),
														AttributeName2,
														SDPCSTRING(sdpString(AttributeValue2)),
														AttributeName3,
														SDPCSTRING(sdpString(AttributeValue3)),
														AttributeName4,
														SDPCSTRING(sdpString(AttributeValue4)),
														AttributeName5,
														SDPCSTRING(sdpString(AttributeValue5)),
														AttributeName6,
														SDPCSTRING(sdpString(AttributeValue6)),
														0);
}

/// Finds the first child XML element with the given name
sdpXmlIElement* sdpXmlFindElement(sdpXmlIElement* Parent, SDPCSTRING ElementName);
/// Finds the first child XML element with the given name, that contains the given attribute
sdpXmlIElement* sdpXmlFindElement(sdpXmlIElement* Parent, SDPCSTRING ElementName, SDPCSTRING AttributeName);
/// Finds the first child XML element with the given name, that contains an attribute with the given name and value
sdpXmlIElement* sdpXmlFindElement(sdpXmlIElement* Parent, SDPCSTRING ElementName, SDPCSTRING AttributeName, SDPCSTRING AttributeValue);

/// Finds the attribute with the given name
sdpXmlIAttribute* sdpXmlFindAttribute(sdpXmlIElement* Element, SDPCSTRING Name);
/// Counts child elements that have the given name
gulong sdpXmlCountElements(sdpXmlIElement* Parent, SDPCSTRING ChildName);

/// Encodes a "normal" string for safe insertion into an XML document
sdpString sdpXmlEncodeString(SDPCSTRING PlainText);
/// Encodes a "normal" string for safe insertion into an XML document
sdpString sdpXmlEncodeString(const sdpString& PlainText);
/// Decodes a string extracted from an XML document into normal text
sdpString sdpXmlDecodeString(SDPCSTRING EncodedText);
/// Decodes a string extracted from an XML document into normal text
sdpString sdpXmlDecodeString(const sdpString& EncodedText);

/// Quietly tries to find an attribute, parse its value, and place it into the result
template<class TYPE>
bool sdpXmlGetAttribute(sdpXmlIElement* Element, SDPCSTRING Name, TYPE& Result)
{
	// Sanity checks ...
	g_assert(Element);
	g_assert_string(Name);

	sdpXmlIAttribute* attribute = sdpXmlFindAttribute(Element, Name);
	if(!attribute)
		return false;	// We don't assert here because attributes tend to be optional by nature ...

	g_return_val_if_fail(sdpString(attribute->Value()).getvalue(Result), false);

	return true;
}

/// Applies a function to every child of the given element
template<class ChildFunctor>
bool sdpXmlForEachElement(sdpXmlIElement* Parent, ChildFunctor& Functor)
{
	// Sanity checks ...
	g_assert(Parent);

	for(SDPXMLELEMENTHANDLE e = Parent->GetFirstElement(); e; e = Parent->GetNextElement(e))
		{
			sdpXmlIElement* child = Parent->GetElement(e);
			g_assert(child);

			if(!Functor.HandleElement(child))
				return false;
		}
	return true;
}


/// Applies a function to every child element with the given name
template<class ChildFunctor>
bool sdpXmlForEachElement(sdpXmlIElement* Parent, SDPCSTRING ChildName, ChildFunctor& Functor)
{
	// Sanity checks ...
	g_assert(Parent);
	g_assert_string(ChildName);

	sdpString childname(ChildName);

	for(SDPXMLELEMENTHANDLE e = Parent->GetFirstElement(); e; e = Parent->GetNextElement(e))
		{
			sdpXmlIElement* child = Parent->GetElement(e);
			g_assert(child);

			if(sdpString(child->Name()) != childname)
				continue;

			if(!Functor.HandleElement(child))
				return false;
		}
	return true;
}


#endif SDPXMLUTILITY_H
