
// SDPXML Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/** \file
		\brief Implements the sdpXmlAttribute class, which implements sdpXmlIAttribute
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpxmlattribute.h>

/////////////////////////////////////////////////////
// sdpXmlAttribute

sdpXmlAttribute::sdpXmlAttribute(sdpXmlIElement* ParentElement, SDPCSTRING Name, SDPCSTRING Value, const gulong Line, const gulong Column)
{
	// Sanity checks ...
	g_assert(ParentElement);
	g_assert_string(Name);

	m_ParentElement = ParentElement;
	m_Name = Name;
	m_Value = Value;
	m_Line = Line;
	m_Column = Column;
}

sdpXmlIElement* sdpXmlAttribute::ParentElement() const
{
	return m_ParentElement;
}

SDPCSTRING sdpXmlAttribute::Name() const
{
	return m_Name;
}

sdpResult sdpXmlAttribute::SetName(SDPCSTRING Name)
{
	// Sanity checks ...
	g_return_val_if_string_fail(Name, SDP_E_FAIL);

	m_Name = Name;
	return SDP_S_OK;
}

SDPCSTRING sdpXmlAttribute::Value() const
{
	return m_Value;
}

sdpResult sdpXmlAttribute::SetValue(SDPCSTRING Value)
{
	m_Value = Value;
	return SDP_S_OK;
}

gulong sdpXmlAttribute::Line() const
{
	return m_Line;
}

gulong sdpXmlAttribute::Column() const
{
	return m_Column;
}

sdpIUserData* sdpXmlAttribute::UserData()
{
	return this;
}


